@@ -11,10 +11,10 @@
  ****************************************************************************/
 
 #ifdef RCSID
-static char RcsId[] = "@(#)1.3";
+static char RcsId[] = "@(#)$Revision: 1.6 $";
 #endif
 
-/* atom.c,v 1.3 1996/04/16 17:14:39 koziol Exp */
+/* $Id: atom.c,v 1.6 1997/11/13 14:50:40 koziol Exp $ */
 
 /*
 FILE
@@ -61,7 +61,7 @@ MODIFICATION HISTORY
 #define ATOM_MASTER
 #include "hdf.h"
 #include "atom.h"
-
+#include <assert.h>
 
 /* Private function prototypes */
 static atom_info_t *HAIfind_atom(atom_t atm);
@@ -100,6 +100,11 @@ intn HAinit_group(group_t grp,      /* IN: Group to initialize */
     if((grp<=BADGROUP || grp>=MAXGROUP) && hash_size>0)
         HGOTO_ERROR(DFE_ARGS, FAIL);
 
+#ifdef ATOMS_CACHE_INLINE
+/* Assertion necessary for faster pointer swapping */
+assert(sizeof(hdf_pint_t)==sizeof(void *));
+#endif /* ATOMS_CACHE_INLINE */
+
 #ifdef HASH_SIZE_POWER_2
     /* If anyone knows a faster test for a power of two, please change this silly code -QAK */
     if(!(hash_size==2 || hash_size==4 || hash_size==8 || hash_size==16
@@ -275,7 +280,7 @@ atom_t HAregister_atom(group_t grp,     /* IN: Group to register the object in *
     atm_ptr->next=NULL;
 
     /* hash bucket already full, prepend to front of chain */
-    hash_loc=grp_ptr->nextid%grp_ptr->hash_size;
+    hash_loc=grp_ptr->nextid%(uintn)grp_ptr->hash_size;
     if(grp_ptr->atom_list[hash_loc]!=NULL)
         atm_ptr->next=grp_ptr->atom_list[hash_loc];
 
@@ -311,13 +316,20 @@ done:
     Returns object ptr if successful and NULL otherwise
 
 *******************************************************************************/
+#ifdef ATOMS_CACHE_INLINE
+VOIDP HAPatom_object(atom_t atm   /* IN: Atom to retrieve object for */
+)
+#else /* ATOMS_CACHE_INLINE */
 VOIDP HAatom_object(atom_t atm   /* IN: Atom to retrieve object for */
 )
+#endif /* ATOMS_CACHE_INLINE */
 {
     CONSTR(FUNC, "HAatom_object");	/* for HERROR */
+#ifndef ATOMS_CACHE_INLINE
 #ifdef ATOMS_ARE_CACHED
     uintn i;                        /* local counter */
 #endif /* ATOMS_ARE_CACHED */
+#endif /* ATOMS_CACHE_INLINE */
     atom_info_t *atm_ptr=NULL;      /* ptr to the new atom */
     VOIDP ret_value=NULL;
 
@@ -327,6 +339,7 @@ VOIDP HAatom_object(atom_t atm   /* IN: Atom to retrieve object for */
 
     HEclear();
 
+#ifndef ATOMS_CACHE_INLINE
 #ifdef ATOMS_ARE_CACHED
     /* Look for the atom in the cache first */
     for(i=0; i<ATOM_CACHE_SIZE; i++)
@@ -346,6 +359,7 @@ VOIDP HAatom_object(atom_t atm   /* IN: Atom to retrieve object for */
             HGOTO_DONE(ret_value);
           } /* end if */
 #endif /* ATOMS_ARE_CACHED */
+#endif /* ATOMS_CACHE_INLINE */
 
     /* General lookup of the atom */
     if((atm_ptr=HAIfind_atom(atm))==NULL)
@@ -448,7 +462,7 @@ VOIDP HAremove_atom(atom_t atm   /* IN: Atom to remove */
         HGOTO_ERROR(DFE_INTERNAL, NULL);
 
     /* Get the location in which the atom is located */
-    hash_loc=ATOM_TO_LOC(atm,grp_ptr->hash_size);
+    hash_loc=(uintn)ATOM_TO_LOC(atm,grp_ptr->hash_size);
     curr_atm=grp_ptr->atom_list[hash_loc];
     if(curr_atm==NULL)
         HGOTO_ERROR(DFE_INTERNAL, NULL);
@@ -516,16 +530,16 @@ done:
     Returns pointer an atom's object if successful and NULL otherwise
 
 *******************************************************************************/
-VOIDP HAsearch_atom(group_t grp,        /* IN: Group to search for the object in */
+void * HAsearch_atom(group_t grp,        /* IN: Group to search for the object in */
     HAsearch_func_t func,               /* IN: Ptr to the comparison function */
-    const VOIDP key                     /* IN: pointer to key to compare against */
+    const void * key                     /* IN: pointer to key to compare against */
 )
 {
     CONSTR(FUNC, "HAsearch_atom");	/* for HERROR */
     atom_group_t *grp_ptr=NULL;     /* ptr to the atomic group */
     atom_info_t *atm_ptr=NULL;      /* ptr to the new atom */
     intn i;                         /* local counting variable */
-    VOIDP ret_value=NULL;
+    void * ret_value=NULL;
 
 #ifdef HAVE_PABLO
     TRACE_ON(HA_mask, ID_HAsearch_atom);
@@ -596,7 +610,7 @@ static atom_info_t *HAIfind_atom(atom_t atm   /* IN: Atom to retrieve atom for *
         HGOTO_ERROR(DFE_INTERNAL, NULL);
 
     /* Get the location in which the atom is located */
-    hash_loc=ATOM_TO_LOC(atm,grp_ptr->hash_size);
+    hash_loc=(uintn)ATOM_TO_LOC(atm,grp_ptr->hash_size);
     atm_ptr=grp_ptr->atom_list[hash_loc];
     if(atm_ptr==NULL)
         HGOTO_ERROR(DFE_INTERNAL, NULL);
