@@ -11,10 +11,10 @@
  ****************************************************************************/
 
 #ifdef RCSID
-static char RcsId[] = "1.5";
+static char RcsId[] = "$Revision: 1.10 $";
 #endif
 
-/* cdeflate.c,v 1.5 1995/11/27 21:35:22 koziol Exp */
+/* $Id: cdeflate.c,v 1.10 1997/10/24 21:00:16 koziol Exp $ */
 
 /*
    FILE
@@ -85,6 +85,8 @@ HCIcdeflate_init(accrec_t * access_rec)
 
     /* Initialize deflation state information */
     deflate_info->offset = 0;   /* start at the beginning of the data */
+    deflate_info->acc_init = 0; /* second stage of initializing not performed */
+    deflate_info->acc_mode = 0; /* init access mode to illegal value */
 
     /* initialize compression context */
     deflate_info->deflate_context.zalloc=(alloc_func)Z_NULL;
@@ -93,7 +95,7 @@ HCIcdeflate_init(accrec_t * access_rec)
     deflate_info->deflate_context.data_type=Z_BINARY;
 
     /* Allocate compression I/O buffer */
-    if ((deflate_info->io_buf= (VOIDP) HDmalloc(DEFLATE_BUF_SIZE)) == NULL)
+    if ((deflate_info->io_buf= HDmalloc(DEFLATE_BUF_SIZE)) == NULL)
         HRETURN_ERROR(DFE_NOSPACE, FAIL);
     
     return (SUCCEED);
@@ -131,9 +133,20 @@ HCIcdeflate_decode(compinfo_t * info, int32 length, uint8 *buf)
 
     /* Set up the deflation buffers to point to the user's buffer to fill */
     deflate_info->deflate_context.next_out=buf;
-    deflate_info->deflate_context.avail_out=length;
+    deflate_info->deflate_context.avail_out=(uInt)length;
     while(deflate_info->deflate_context.avail_out>0)
       {
+#ifdef QAK
+int32 t1, t2;
+
+if(t1!=deflate_info->deflate_context.avail_out ||
+    t2!=deflate_info->deflate_context.avail_in)
+  {
+    printf("%s: avail_out=%d, avail_in=%d\n",FUNC,(int)deflate_info->deflate_context.avail_out,(int) deflate_info->deflate_context.avail_in);
+    t1=deflate_info->deflate_context.avail_out;
+    t2=deflate_info->deflate_context.avail_in;
+  }
+#endif
           /* Get more bytes from the file, if we've run out */
           if(deflate_info->deflate_context.avail_in==0)
             {
@@ -142,14 +155,14 @@ HCIcdeflate_decode(compinfo_t * info, int32 length, uint8 *buf)
                 deflate_info->deflate_context.next_in=deflate_info->io_buf;
                 if((file_bytes=Hread(info->aid,DEFLATE_BUF_SIZE,deflate_info->deflate_context.next_in))==FAIL)
                     HRETURN_ERROR(DFE_READERROR,FAIL);
-                deflate_info->deflate_context.avail_in=file_bytes;
+                deflate_info->deflate_context.avail_in=(uInt)file_bytes;
             } /* end if */
 
           /* break out if we've reached the end of the compressed data somehow */
           if(inflate(&(deflate_info->deflate_context),Z_NO_FLUSH)==Z_STREAM_END)
               break;
       } /* end while */
-    bytes_read=length-deflate_info->deflate_context.avail_out;
+    bytes_read=(int32)length-(int32)deflate_info->deflate_context.avail_out;
     deflate_info->offset+=bytes_read;
 
     return(bytes_read);
@@ -178,7 +191,7 @@ HCIcdeflate_decode(compinfo_t * info, int32 length, uint8 *buf)
  REVISION LOG
 --------------------------------------------------------------------------*/
 PRIVATE int32
-HCIcdeflate_encode(compinfo_t * info, int32 length, const VOIDP buf)
+HCIcdeflate_encode(compinfo_t * info, int32 length, void * buf)
 {
     CONSTR(FUNC, "HCIcdeflate_encode");
     comp_coder_deflate_info_t *deflate_info;    /* ptr to skipping Huffman info */
@@ -187,7 +200,7 @@ HCIcdeflate_encode(compinfo_t * info, int32 length, const VOIDP buf)
 
     /* Set up the deflation buffers to point to the user's buffer to empty */
     deflate_info->deflate_context.next_in=buf;
-    deflate_info->deflate_context.avail_in=length;
+    deflate_info->deflate_context.avail_in=(uInt)length;
     while(deflate_info->deflate_context.avail_in>0)
       {
           /* Write more bytes from the file, if we've filled our buffer */
@@ -237,39 +250,44 @@ HCIcdeflate_term(compinfo_t * info,uint32 acc_mode)
 
     deflate_info = &(info->cinfo.coder_info.deflate_info);
 
-    if(acc_mode&DFACC_WRITE)
-      { /* flush the "deflated" data to the file */
-          intn status;
+    /* set flag to indicate second stage of initialization is finished */
 
-          do
-            {
-              /* Write more bytes from the file, if we've filled our buffer */
-              if(deflate_info->deflate_context.avail_out==0)
+    if(deflate_info->acc_init==1)
+      {
+        if(acc_mode&DFACC_WRITE)
+          { /* flush the "deflated" data to the file */
+              intn status;
+
+              do
                 {
-                    if(Hwrite(info->aid,DEFLATE_BUF_SIZE,deflate_info->io_buf)==FAIL)
-                        HRETURN_ERROR(DFE_WRITEERROR,FAIL);
-                    deflate_info->deflate_context.next_out=deflate_info->io_buf;
-                    deflate_info->deflate_context.avail_out=DEFLATE_BUF_SIZE;
-                } /* end if */
+                  /* Write more bytes from the file, if we've filled our buffer */
+                  if(deflate_info->deflate_context.avail_out==0)
+                    {
+                        if(Hwrite(info->aid,DEFLATE_BUF_SIZE,deflate_info->io_buf)==FAIL)
+                            HRETURN_ERROR(DFE_WRITEERROR,FAIL);
+                        deflate_info->deflate_context.next_out=deflate_info->io_buf;
+                        deflate_info->deflate_context.avail_out=DEFLATE_BUF_SIZE;
+                    } /* end if */
 
-                status=deflate(&(deflate_info->deflate_context),Z_FINISH);
-            } while(status==Z_OK);
-          if(status!=Z_STREAM_END)
-              HRETURN_ERROR(DFE_CENCODE,FAIL);
-          if(deflate_info->deflate_context.avail_out<DEFLATE_BUF_SIZE)
-              if(Hwrite(info->aid,(DEFLATE_BUF_SIZE-deflate_info->deflate_context.avail_out),deflate_info->io_buf)==FAIL)
-                  HRETURN_ERROR(DFE_WRITEERROR,FAIL);
+                    status=deflate(&(deflate_info->deflate_context),Z_FINISH);
+                } while(status==Z_OK);
+              if(status!=Z_STREAM_END)
+                  HRETURN_ERROR(DFE_CENCODE,FAIL);
+              if(deflate_info->deflate_context.avail_out<DEFLATE_BUF_SIZE)
+                  if(Hwrite(info->aid,(int32)(DEFLATE_BUF_SIZE-deflate_info->deflate_context.avail_out),deflate_info->io_buf)==FAIL)
+                      HRETURN_ERROR(DFE_WRITEERROR,FAIL);
 
-          /* Close down the deflation buffer */
-          if(deflateEnd(&(deflate_info->deflate_context))!=Z_OK)
-              HRETURN_ERROR(DFE_CTERM,FAIL);
+              /* Close down the deflation buffer */
+              if(deflateEnd(&(deflate_info->deflate_context))!=Z_OK)
+                  HRETURN_ERROR(DFE_CTERM,FAIL);
+          } /* end if */
+        else
+          { /* finish up any inflated data */
+              /* Close down the inflation buffer */
+              if(inflateEnd(&(deflate_info->deflate_context))!=Z_OK)
+                  HRETURN_ERROR(DFE_CTERM,FAIL);
+          } /* end else */
       } /* end if */
-    else
-      { /* finish up any inflated data */
-          /* Close down the inflation buffer */
-          if(inflateEnd(&(deflate_info->deflate_context))!=Z_OK)
-              HRETURN_ERROR(DFE_CTERM,FAIL);
-      } /* end else */
 
     /* Get rid of the I/O buffer */
     HDfree(deflate_info->io_buf);
@@ -309,6 +327,7 @@ HCIcdeflate_staccess(accrec_t * access_rec, int16 acc_mode)
 
     /* need to check for not writing, as opposed to read access */
     /* because of the way the access works */
+#ifdef OLD_WAY
     if (!(acc_mode&DFACC_WRITE))
         info->aid = Hstartread(access_rec->file_id, DFTAG_COMPRESSED,
                                   info->comp_ref);
@@ -317,6 +336,16 @@ HCIcdeflate_staccess(accrec_t * access_rec, int16 acc_mode)
                                    info->comp_ref, info->length);
     if (info->aid == FAIL)
         HRETURN_ERROR(DFE_DENIED, FAIL);
+#else /* OLD_WAY */
+    if (!(acc_mode&DFACC_WRITE))
+        info->aid = Hstartread(access_rec->file_id, DFTAG_COMPRESSED,
+                                  info->comp_ref);
+    else
+        info->aid = Hstartaccess(access_rec->file_id, DFTAG_COMPRESSED,
+                                   info->comp_ref, DFACC_RDWR|DFACC_APPENDABLE);
+    if (info->aid == FAIL)
+        HRETURN_ERROR(DFE_DENIED, FAIL);
+#endif /* OLD_WAY */
 
     /* Make certain we can append to the data when writing */
     if ((acc_mode&DFACC_WRITE) && Happendable(info->aid) == FAIL)
@@ -326,12 +355,49 @@ HCIcdeflate_staccess(accrec_t * access_rec, int16 acc_mode)
     if(HCIcdeflate_init(access_rec)==FAIL)
         HRETURN_ERROR(DFE_CODER,FAIL);
 
+    return (SUCCEED);
+}   /* end HCIcdeflate_staccess() */
+
+/*--------------------------------------------------------------------------
+ NAME
+    HCIcdeflate_staccess2 -- 2nd half of start accessing a gzip 'deflate'
+                            compressed data element.
+
+ USAGE
+    int32 HCIcdeflate_staccess2(access_rec, access)
+    accrec_t *access_rec;   IN: the access record of the data element
+    int16 access;           IN: the type of access wanted
+
+ RETURNS
+    Returns SUCCEED or FAIL
+
+ DESCRIPTION
+    Common code called by HCIcdeflate_stread and HCIcdeflate_stwrite
+
+ GLOBAL VARIABLES
+ COMMENTS, BUGS, ASSUMPTIONS
+ EXAMPLES
+ REVISION LOG
+--------------------------------------------------------------------------*/
+PRIVATE int32
+HCIcdeflate_staccess2(accrec_t * access_rec, int16 acc_mode)
+{
+    CONSTR(FUNC, "HCIcdeflate_staccess2");
+    compinfo_t *info;           /* special element information */
+    comp_coder_deflate_info_t *deflate_info;    /* ptr to deflate info */
+
+    info = (compinfo_t *) access_rec->special_info;
+    deflate_info = &(info->cinfo.coder_info.deflate_info);
+
     /* Initialize the gzip library */
     if(acc_mode&DFACC_WRITE)
       {
         if(deflateInit(&(deflate_info->deflate_context),deflate_info->deflate_level)!=Z_OK)
             HRETURN_ERROR(DFE_CINIT, FAIL);
 
+        /* set access mode */
+        deflate_info->acc_mode=DFACC_WRITE;
+
         /* force I/O with the file at first */
         deflate_info->deflate_context.next_out=NULL;
         deflate_info->deflate_context.avail_out=0;
@@ -341,12 +407,18 @@ HCIcdeflate_staccess(accrec_t * access_rec, int16 acc_mode)
         if(inflateInit(&(deflate_info->deflate_context))!=Z_OK)
             HRETURN_ERROR(DFE_CINIT, FAIL);
 
+        /* set access mode */
+        deflate_info->acc_mode=DFACC_READ;
+        
         /* force I/O with the file at first */
         deflate_info->deflate_context.avail_in=0;
       } /* end else */
 
+    /* set flag to indicate second stage of initialization is finished */
+    deflate_info->acc_init=1;
+
     return (SUCCEED);
-}   /* end HCIcdeflate_staccess() */
+}   /* end HCIcdeflate_staccess2() */
 
 /*--------------------------------------------------------------------------
  NAME
@@ -446,6 +518,13 @@ HCPcdeflate_seek(accrec_t * access_rec, int32 offset, int origin)
     info = (compinfo_t *) access_rec->special_info;
     deflate_info = &(info->cinfo.coder_info.deflate_info);
 
+    /* Check if second stage of initialization has been performed */
+    if(deflate_info->acc_init==0)
+      {
+        if (HCIcdeflate_staccess2(access_rec, DFACC_READ) == FAIL)
+            HRETURN_ERROR(DFE_CINIT, FAIL);
+      } /* end if */
+
 #ifdef QAK
     /* Don't allow seeks when writing (currently) -QAK */
     if (access_rec->access&DFACC_WRITE)
@@ -469,13 +548,13 @@ HCPcdeflate_seek(accrec_t * access_rec, int32 offset, int origin)
         if (HCIcdeflate_decode(info, DEFLATE_TMP_BUF_SIZE, tmp_buf) == FAIL)
           {
               HDfree(tmp_buf);
-              HRETURN_ERROR(DFE_CDECODE, FAIL);
+              HRETURN_ERROR(DFE_CDECODE, FAIL)
           }     /* end if */
     if (deflate_info->offset < offset)  /* grab the last chunk */
         if (HCIcdeflate_decode(info, offset - deflate_info->offset, tmp_buf) == FAIL)
           {
               HDfree(tmp_buf);
-              HRETURN_ERROR(DFE_CDECODE, FAIL);
+              HRETURN_ERROR(DFE_CDECODE, FAIL)
           }     /* end if */
 
     HDfree(tmp_buf);
@@ -490,7 +569,7 @@ HCPcdeflate_seek(accrec_t * access_rec, int32 offset, int origin)
     int32 HCPcdeflate_read(access_rec,length,data)
     accrec_t *access_rec;   IN: the access record of the data element
     int32 length;           IN: the number of bytes to read
-    VOIDP data;             OUT: the buffer to place the bytes read
+    void * data;             OUT: the buffer to place the bytes read
 
  RETURNS
     Returns the number of bytes read or FAIL
@@ -504,12 +583,21 @@ HCPcdeflate_seek(accrec_t * access_rec, int32 offset, int origin)
  REVISION LOG
 --------------------------------------------------------------------------*/
 int32
-HCPcdeflate_read(accrec_t * access_rec, int32 length, VOIDP data)
+HCPcdeflate_read(accrec_t * access_rec, int32 length, void * data)
 {
     CONSTR(FUNC, "HCPcdeflate_read");
     compinfo_t *info;           /* special element information */
+    comp_coder_deflate_info_t *deflate_info;    /* ptr to gzip 'deflate' info */
 
     info = (compinfo_t *) access_rec->special_info;
+    deflate_info = &(info->cinfo.coder_info.deflate_info);
+
+    /* Check if second stage of initialization has been performed */
+    if(deflate_info->acc_init==0)
+      {
+        if (HCIcdeflate_staccess2(access_rec, DFACC_READ) == FAIL)
+            HRETURN_ERROR(DFE_CINIT, FAIL);
+      } /* end if */
 
     if ((length=HCIcdeflate_decode(info, length, data)) == FAIL)
         HRETURN_ERROR(DFE_CDECODE, FAIL);
@@ -525,7 +613,7 @@ HCPcdeflate_read(accrec_t * access_rec, int32 length, VOIDP data)
     int32 HCPcdeflate_write(access_rec,length,data)
     accrec_t *access_rec;   IN: the access record of the data element
     int32 length;           IN: the number of bytes to write
-    VOIDP data;             IN: the buffer to retrieve the bytes written
+    void * data;             IN: the buffer to retrieve the bytes written
 
  RETURNS
     Returns the number of bytes written or FAIL
@@ -539,7 +627,7 @@ HCPcdeflate_read(accrec_t * access_rec, int32 length, VOIDP data)
  REVISION LOG
 --------------------------------------------------------------------------*/
 int32
-HCPcdeflate_write(accrec_t * access_rec, int32 length, const VOIDP data)
+HCPcdeflate_write(accrec_t * access_rec, int32 length, const void * data)
 {
     CONSTR(FUNC, "HCPcdeflate_write");
     compinfo_t *info;           /* special element information */
@@ -555,7 +643,14 @@ HCPcdeflate_write(accrec_t * access_rec, int32 length, const VOIDP data)
         && (deflate_info->offset != 0 || length < info->length))
         HRETURN_ERROR(DFE_UNSUPPORTED, FAIL);
 
-    if ((length=HCIcdeflate_encode(info, length, data)) == FAIL)
+    /* Check if second stage of initialization has been performed */
+    if(deflate_info->acc_init==0)
+      {
+        if (HCIcdeflate_staccess2(access_rec, DFACC_WRITE) == FAIL)
+            HRETURN_ERROR(DFE_CINIT, FAIL);
+      } /* end if */
+
+    if ((length=HCIcdeflate_encode(info, length, (void *)data)) == FAIL)
         HRETURN_ERROR(DFE_CENCODE, FAIL);
 
     return (length);
@@ -633,11 +728,13 @@ HCPcdeflate_endaccess(accrec_t * access_rec)
 {
     CONSTR(FUNC, "HCPcdeflate_endaccess");
     compinfo_t *info;           /* special element information */
+    comp_coder_deflate_info_t *deflate_info;    /* ptr to gzip 'deflate' info */
 
     info = (compinfo_t *) access_rec->special_info;
+    deflate_info = &(info->cinfo.coder_info.deflate_info);
 
     /* flush out buffer */
-    if (HCIcdeflate_term(info,access_rec->access) == FAIL)
+    if (HCIcdeflate_term(info,deflate_info->acc_mode) == FAIL)
         HRETURN_ERROR(DFE_CTERM, FAIL);
 
     /* close the compressed data AID */
