@@ -11,10 +11,10 @@
  ****************************************************************************/
 
 #ifdef RCSID
-static char RcsId[] = "@(#)1.22";
+static char RcsId[] = "@(#)$Revision: 1.25 $";
 #endif
 
-/* cnbit.c,v 1.22 1995/09/14 20:21:11 georgev Exp */
+/* $Id: cnbit.c,v 1.25 1997/10/24 21:00:19 koziol Exp $ */
 
 /*
    FILE
@@ -63,7 +63,7 @@ static const uint32 mask_arr32[33] =
     0x0001FFFF, 0x0003FFFF, 0x0007FFFF, 0x000FFFFF,
     0x001FFFFF, 0x003FFFFF, 0x007FFFFF, 0x00FFFFFF,
     0x01FFFFFF, 0x03FFFFFF, 0x07FFFFFF, 0x0FFFFFFF,
-    0x1FFFFFFF, 0x3FFFFFFF, 0x7FFFFFFF, 0xFFFFFFFF
+    0x1FFFFFFF, 0x3FFFFFFF, 0x7FFFFFFF, 0xFFFFFFFFUL
 };
 
 /* declaration of the functions provided in this module */
@@ -77,7 +77,7 @@ PRIVATE int32 HCIcnbit_decode
             (compinfo_t * info, int32 length, uint8 *buf);
 
 PRIVATE int32 HCIcnbit_encode
-            (compinfo_t * info, int32 length, uint8 *buf);
+            (compinfo_t * info, int32 length, const uint8 *buf);
 
 PRIVATE int32 HCIcnbit_term
             (compinfo_t * info);
@@ -288,7 +288,7 @@ HCIcnbit_decode(compinfo_t * info, int32 length, uint8 *buf)
                 rbuf = (uint8 *) nbit_info->buffer;     /* get a ptr to the buffer */
 
                 /* get initial copy of the mask */
-                HDmemfill(rbuf, (const VOIDP)nbit_info->mask_buf, (uint32)nbit_info->nt_size, (uint32)buf_items);
+                HDmemfill(rbuf, nbit_info->mask_buf, (uint32)nbit_info->nt_size, (uint32)buf_items);
 
                 for (i = 0; i < buf_items; i++)
                   {
@@ -307,7 +307,7 @@ HCIcnbit_decode(compinfo_t * info, int32 length, uint8 *buf)
                                     {   /* check if we need to read bits */
                                         Hbitread(info->aid, mask_info->length, &input_bits);
                                         input_bits <<= (mask_info->offset - mask_info->length) + 1;
-                                        *rbuf2 |= mask_info->mask & input_bits;
+                                        *rbuf2 |= (uint8)(mask_info->mask & (uint8)input_bits);
                                         if (j == sign_byte)     /* check if this is the sign byte */
                                             sign_bit = sign_mask & input_bits ? 1 : 0;
                                     }   /* end if */
@@ -325,13 +325,13 @@ HCIcnbit_decode(compinfo_t * info, int32 length, uint8 *buf)
                                     {   /* fill with ones */
                                         for (j = 0; j < sign_byte; j++, rbuf2++)
                                             *rbuf2 = 0xff;
-                                        *rbuf2 |= sign_ext_mask;
+                                        *rbuf2 |= (uint8)sign_ext_mask;
                                     }   /* end if */
                                   else
                                     {   /* fill with zeroes */
                                         for (j = 0; j < sign_byte; j++, rbuf2++)
                                             *rbuf2 = 0x00;
-                                        *rbuf2 &= ~sign_ext_mask;
+                                        *rbuf2 &= (uint8)~sign_ext_mask;
                                     }   /* end else */
                               }     /* end if */
                             rbuf += nbit_info->nt_size;     /* increment buffer ptr */
@@ -350,8 +350,8 @@ HCIcnbit_decode(compinfo_t * info, int32 length, uint8 *buf)
 #ifdef TESTING
                                         printf("HCInbit_decode(): input_bits=%d\n", (int) input_bits);
 #endif
-                                        *rbuf |= mask_info->mask & (input_bits <<
-                                                                    ((mask_info->offset - mask_info->length) + 1));
+                                        *rbuf |= (uint8)(mask_info->mask & (uint8)(input_bits <<
+                                                                    ((mask_info->offset - mask_info->length) + 1)));
 #ifdef TESTING
                                         printf("HCInbit_decode(): j=%d, length=%d, *rbuf=%x\n", j, mask_info->length, (unsigned) *rbuf);
 #endif
@@ -385,7 +385,7 @@ HCIcnbit_decode(compinfo_t * info, int32 length, uint8 *buf)
     int32 HCIcnbit_encode(info,length,buf)
     compinfo_t *info;   IN: the info about the compressed element
     int32 length;       IN: number of bytes to store from the buffer
-    uint8 *buf;         OUT: buffer to get the bytes from
+    const uint8 *buf;         OUT: buffer to get the bytes from
 
  RETURNS
     Returns SUCCEED or FAIL
@@ -399,7 +399,7 @@ HCIcnbit_decode(compinfo_t * info, int32 length, uint8 *buf)
  REVISION LOG
 --------------------------------------------------------------------------*/
 PRIVATE int32
-HCIcnbit_encode(compinfo_t * info, int32 length, uint8 *buf)
+HCIcnbit_encode(compinfo_t * info, int32 length, const uint8 *buf)
 {
 #ifdef LATER
     CONSTR(FUNC, "HCIcnbit_encode");
@@ -427,8 +427,8 @@ HCIcnbit_encode(compinfo_t * info, int32 length, uint8 *buf)
 #endif
           if (mask_info->length > 0)
             {   /* check if we need to output bits */
-                output_bits = ((*buf) & (mask_info->mask)) >>
-                    ((mask_info->offset - mask_info->length) + 1);
+                output_bits = (uint32)(((*buf) & (mask_info->mask)) >>
+                    ((mask_info->offset - mask_info->length) + 1));
 #ifdef TESTING
                 printf("HCIcnbit_encode(): output_bits=%x\n", (unsigned) output_bits);
 #endif
@@ -657,7 +657,7 @@ HCPcnbit_seek(accrec_t * access_rec, int32 offset, int origin)
     int32 HCPcnbit_read(access_rec,length,data)
     accrec_t *access_rec;   IN: the access record of the data element
     int32 length;           IN: the number of bytes to read
-    VOIDP data;             OUT: the buffer to place the bytes read
+    void * data;             OUT: the buffer to place the bytes read
 
  RETURNS
     Returns the number of bytes read or FAIL
@@ -671,7 +671,7 @@ HCPcnbit_seek(accrec_t * access_rec, int32 offset, int origin)
  REVISION LOG
 --------------------------------------------------------------------------*/
 int32
-HCPcnbit_read(accrec_t * access_rec, int32 length, VOIDP data)
+HCPcnbit_read(accrec_t * access_rec, int32 length, void * data)
 {
     CONSTR(FUNC, "HCPcnbit_read");
     compinfo_t *info;           /* special element information */
@@ -692,7 +692,7 @@ HCPcnbit_read(accrec_t * access_rec, int32 length, VOIDP data)
     int32 HCPwrite(access_rec,length,data)
     accrec_t *access_rec;   IN: the access record of the data element
     int32 length;           IN: the number of bytes to write
-    VOIDP data;             IN: the buffer to retrieve the bytes written
+    void * data;             IN: the buffer to retrieve the bytes written
 
  RETURNS
     Returns the number of bytes written or FAIL
@@ -706,7 +706,7 @@ HCPcnbit_read(accrec_t * access_rec, int32 length, VOIDP data)
  REVISION LOG
 --------------------------------------------------------------------------*/
 int32
-HCPcnbit_write(accrec_t * access_rec, int32 length, const VOIDP data)
+HCPcnbit_write(accrec_t * access_rec, int32 length, const void * data)
 {
     CONSTR(FUNC, "HCPcnbit_write");
     compinfo_t *info;           /* special element information */
@@ -797,13 +797,11 @@ HCPcnbit_endaccess(accrec_t * access_rec)
 {
     CONSTR(FUNC, "HCPcnbit_endaccess");
     compinfo_t *info;           /* special element information */
-    comp_coder_nbit_info_t *nbit_info;  /* ptr to n-bit info */
 
 #ifdef TESTING
     printf("HCPcnbit_endaccess(): entering\n");
 #endif
     info = (compinfo_t *) access_rec->special_info;
-    nbit_info = &(info->cinfo.coder_info.nbit_info);
 
     /* flush out n-bit buffer */
     if (access_rec->access&DFACC_WRITE)
