@@ -11,10 +11,10 @@
  ****************************************************************************/
 
 #ifdef RCSID
-static char RcsId[] = "@(#)1.18";
+static char RcsId[] = "@(#)$Revision: 1.23 $";
 #endif
 
-/* df24.c,v 1.18 1995/10/19 19:47:01 georgev Exp */
+/* $Id: df24.c,v 1.23 1997/12/18 16:50:38 koziol Exp $ */
 
 /*-----------------------------------------------------------------------------
  * File:     df24.c
@@ -144,7 +144,7 @@ DF24reqil(intn il)
  USAGE
     intn DF24getimage(filename,image,xdim,ydim)
         char *filename;     IN: file name to retrieve image from
-        VOIDP image;        OUT: buffer to store image in
+        void * image;        OUT: buffer to store image in
         int32 xdim,ydim;    IN: dimensions of image buffer
  RETURNS
     SUCCEED on success, FAIL on failure.
@@ -160,7 +160,7 @@ DF24reqil(intn il)
  REVISION LOG
 --------------------------------------------------------------------------*/
 intn
-DF24getimage(const char *filename, VOIDP image, int32 xdim, int32 ydim)
+DF24getimage(const char *filename, void * image, int32 xdim, int32 ydim)
 {
   CONSTR(FUNC, "DF24getimage");
   intn        il;
@@ -354,7 +354,7 @@ DF24restart(void)
  USAGE
     intn DF24addimage(filename,image,xdim,ydim)
         char *filename;     IN: name of HDF file to write to
-        VOIDP image;        IN: Pointer to image data
+        void * image;        IN: Pointer to image data
         int32 xdim,ydim;    IN: Dimensions of image to write
  RETURNS
     SUCCEED on success, FAIL on failure.
@@ -369,7 +369,7 @@ DF24restart(void)
  REVISION LOG
 --------------------------------------------------------------------------*/
 intn
-DF24addimage(const char *filename, VOIDP image, int32 xdim, int32 ydim)
+DF24addimage(const char *filename, const void * image, int32 xdim, int32 ydim)
 {
   CONSTR(FUNC, "DF24addimage");
   intn ret_value = SUCCEED;
@@ -405,7 +405,7 @@ done:
  USAGE
     intn DF24addimage(filename,image,xdim,ydim)
         char *filename;     IN: name of HDF file to write to
-        VOIDP image;        IN: Pointer to image data
+        void * image;        IN: Pointer to image data
         int32 xdim,ydim;    IN: Dimensions of image to write
  RETURNS
     SUCCEED on success, FAIL on failure.
@@ -420,7 +420,7 @@ done:
  REVISION LOG
 --------------------------------------------------------------------------*/
 intn
-DF24putimage(const char *filename, VOIDP image, int32 xdim, int32 ydim)
+DF24putimage(const char *filename, const void * image, int32 xdim, int32 ydim)
 {
   CONSTR(FUNC, "DF24putimage");
   intn ret_value = SUCCEED;
@@ -502,21 +502,24 @@ DF24nimages(const char *filename)
             {     /* just look for ID tags to get the number of components */
               if (Hgetelement(file_id, elt_tag, elt_ref, GRtbuf) != FAIL)
                 {
+                  uint16      uint16var;
                   int32       temp;   /* temporary holding variable */
-                  int32       ncomponents;    /* number of image components */
+                  int16       ncomponents;    /* number of image components */
                   uint8      *p;
 
                   p = GRtbuf;
                   INT32DECODE(p, temp);
                   INT32DECODE(p, temp);
-                  UINT16DECODE(p, temp);
-                  UINT16DECODE(p, temp);
+                  UINT16DECODE(p, uint16var);
+                  UINT16DECODE(p, uint16var);
                   INT16DECODE(p, ncomponents);
                   if (ncomponents == 3)   /* whew, all that work and we finally found a 24-bit image */
                     nimages++;
                 }   /* end if */
-              else
+              else	{
+              	DFdifree(group_id);
                 HGOTO_ERROR(DFE_GETELEM, FAIL);
+              }
             }     /* end if */
         }   /* end while */
     }     /* end while */
@@ -581,7 +584,7 @@ DF24readref(const char *filename, uint16 ref)
  USAGE
     uint16 DF24lastref(void)
  RETURNS
-    Last ref # on success, FAIL on failure.
+    Last ref # on success, 0 on failure.
  DESCRIPTION
     Returns the last ref # of a 24-bit RIG read to or written from a file.
  GLOBAL VARIABLES
