@@ -11,10 +11,10 @@
  ****************************************************************************/
 
 #ifdef RCSID
-static char RcsId[] = "@(#)1.36";
+static char RcsId[] = "@(#)$Revision: 1.39 $";
 #endif
 
-/* dfan.c,v 1.36 1996/05/20 20:51:24 koziol Exp */
+/* $Id: dfan.c,v 1.39 1997/10/08 17:01:15 georgev Exp $ */
 
 /*-----------------------------------------------------------------------------
  * File:     dfan.c
@@ -580,7 +580,7 @@ DFANaddfds(int32 file_id, char *desc, int32 desclen)
  USAGE
        uint16 DFANlastref()
  RETURNS
-       Reference number if successful and FAIL (-1) otherwise
+       Reference number if successful and 0 otherwise
  DESCRIPTION
  GLOBAL VARIABLES
        Lastref
@@ -735,7 +735,7 @@ DFANIclear(void)
   }
   DFANdir[0] = DFANdir[1] = NULL;
 
-  Lastref = 0;
+  Lastref = 0; /* 0 is invalid ref */
 
 done:
   if(ret_value == FAIL)   
@@ -876,7 +876,7 @@ DFANIlocate(int32 file_id, int type, uint16 tag, uint16 ref)
     /* Perform global, one-time initialization */
     if (library_terminate == FALSE)
         if(DFANIstart()==FAIL)
-            HGOTO_ERROR(DFE_CANTINIT, FAIL);
+            HGOTO_ERROR(DFE_CANTINIT, (uint16)FAIL);
 
   anntag = (uint16) ((type == DFAN_LABEL) ? DFTAG_DIL : DFTAG_DIA);
 
@@ -890,7 +890,7 @@ DFANIlocate(int32 file_id, int type, uint16 tag, uint16 ref)
       /* allocate directory space.  Note head struct includes 1 entry */
       DFANdir[type] = (DFANdirhead *)
         HDmalloc(((uint32) sizeof(DFANdirhead) +
-                  (nanns - 1) * sizeof(DFANdirentry)));
+                  (size_t)(nanns - 1) * sizeof(DFANdirentry)));
       if (DFANdir[type] == NULL)
         HGOTO_ERROR(DFE_NOSPACE, 0);
       DFANdir[type]->next = NULL;
@@ -1277,11 +1277,11 @@ DFANIputann(const char *filename, uint16 tag, uint16 ref, uint8 *ann,
     }
 
   /*
-   * if annotation exists, delete it and rewrite new annotation
+   * if annotation exists, reuse tag/ref and rewrite new annotation
    */
   if (newflag == 0)
     {     /* does prev annotation exist? */
-      if (Hdeldd(file_id, anntag, annref) == FAIL)
+      if (HDreuse_tagref(file_id, anntag, annref) == FAIL)
         {
           Hclose(file_id);
           HEreport("Unable to replace old annotation");
@@ -1373,7 +1373,7 @@ DFANIlablist(const char *filename, uint16 tag, uint16 reflist[],
   int32       i;
   int         j, k;
   int32       file_id, aid, len;
-  uint16      ref;
+  uint16      ref=0;
   DFANdirhead *p;
   uint8      *lp;             /* pointer to label */
   intn        nrefs, nlabs;
