@@ -11,10 +11,10 @@
  ****************************************************************************/
 
 #ifdef RCSID
-static char RcsId[] = "@(#)1.36";
+static char RcsId[] = "@(#)$Revision: 1.41 $";
 #endif
 
-/* dfgr.c,v 1.36 1996/05/20 19:23:56 koziol Exp */
+/* $Id: dfgr.c,v 1.41 1997/12/18 16:50:40 koziol Exp $ */
 
 /*-----------------------------------------------------------------------------
  * File:     dfgr.c
@@ -207,7 +207,7 @@ DFGRreqlutil(int il)
  *---------------------------------------------------------------------------*/
 
 int
-DFGRgetlut(const char *filename, VOIDP lut, int32 xdim, int32 ydim)
+DFGRgetlut(const char *filename, void * lut, int32 xdim, int32 ydim)
 {
     int compressed, has_pal;
     uint16 compr_type;
@@ -266,7 +266,7 @@ DFGRreqimil(int il)
  *---------------------------------------------------------------------------*/
 
 int
-DFGRgetimage(const char *filename, VOIDP image, int32 xdim, int32 ydim)
+DFGRgetimage(const char *filename, void * image, int32 xdim, int32 ydim)
 {
     int compressed, has_pal;
     uint16 compr_type;
@@ -312,7 +312,7 @@ DFGRsetcompress(int32 scheme, comp_info * cinfo)
     if (scheme == COMP_JPEG)
         Grcompr = DFTAG_JPEG5;   /* Set the compression scheme */
     else    /* otherwise, just use mapped tag */
-        Grcompr = compress_map[scheme];
+        Grcompr = (intn)compress_map[scheme];
     Grcinfo = (*cinfo);     /* Set the compression parameters */
 
 done:
@@ -358,7 +358,7 @@ DFGRsetlutdims(int32 xdim, int32 ydim, int ncomps, int il)
  *---------------------------------------------------------------------------*/
 
 int
-DFGRsetlut(VOIDP lut, int32 xdim, int32 ydim)
+DFGRsetlut(void * lut, int32 xdim, int32 ydim)
 {
     /* 0 == C, 0 == no newfile */
     return (DFGRIaddimlut((const char *) NULL, lut, xdim, ydim, LUT, 0, 0));
@@ -377,7 +377,7 @@ DFGRsetlut(VOIDP lut, int32 xdim, int32 ydim)
  *---------------------------------------------------------------------------*/
 
 int
-DFGRaddlut(const char *filename, VOIDP lut, int32 xdim, int32 ydim)
+DFGRaddlut(const char *filename, void * lut, int32 xdim, int32 ydim)
 {
     /* 0 == C, 0 == no new file */
     return (DFGRIaddimlut(filename, lut, xdim, ydim, LUT, 0, 0));
@@ -416,14 +416,14 @@ DFGRsetimdims(int32 xdim, int32 ydim, int ncomps, int il)
  *---------------------------------------------------------------------------*/
 
 int
-DFGRaddimage(const char *filename, VOIDP image, int32 xdim, int32 ydim)
+DFGRaddimage(const char *filename, void * image, int32 xdim, int32 ydim)
 {
     /* 0 == C, 0 == not new file */
     return (DFGRIaddimlut(filename, image, xdim, ydim, IMAGE, 0, 0));
 }
 
 int
-DFGRputimage(const char *filename, VOIDP image, int32 xdim, int32 ydim)
+DFGRputimage(const char *filename, void * image, int32 xdim, int32 ydim)
 {
     /* 0 == C, 1 == new file */
     return (DFGRIaddimlut(filename, image, xdim, ydim, IMAGE, 0, 1));
@@ -446,7 +446,9 @@ DFGRreadref(const char *filename, uint16 ref)
     CONSTR(FUNC, "DFGRreadref");
     intn    ret_value = SUCCEED;
     int32   file_id=(-1);
+#ifdef OLD_WAY
     int32   aid;
+#endif /* OLD_WAY */
 
     HEclear();
 
@@ -458,11 +460,17 @@ DFGRreadref(const char *filename, uint16 ref)
     if ((file_id = DFGRIopen(filename, DFACC_READ))== FAIL)
         HGOTO_ERROR(DFE_BADOPEN, FAIL);
 
+#ifdef OLD_WAY
     if ((aid = Hstartread(file_id, DFTAG_RIG, ref)) == FAIL)
         HGOTO_ERROR(DFE_BADAID, FAIL);
 
     if (Hendaccess(aid)==FAIL)
         HGOTO_ERROR(DFE_CANTENDACCESS, FAIL);
+#else /* OLD_WAY */
+    if (Hexist(file_id, DFTAG_RIG, ref) == FAIL)
+        HGOTO_ERROR(DFE_BADAID, FAIL);
+
+#endif /* OLD_WAY */
 
     Grrefset = ref;
     ret_value= Hclose(file_id);
@@ -541,6 +549,7 @@ DFGRgetrig(int32 file_id, uint16 ref, DFGRrig * rig)
                     type = (elt_tag == DFTAG_LD) ? LUT : IMAGE;
                     if (Hgetelement(file_id, elt_tag, elt_ref, GRtbuf) != FAIL)
                       {
+                          int16       int16var;
                           uint8      *p;
 
                           p = GRtbuf;
@@ -548,13 +557,18 @@ DFGRgetrig(int32 file_id, uint16 ref, DFGRrig * rig)
                           INT32DECODE(p, rig->datadesc[type].ydim);
                           UINT16DECODE(p, rig->datadesc[type].nt.tag);
                           UINT16DECODE(p, rig->datadesc[type].nt.ref);
-                          INT16DECODE(p, rig->datadesc[type].ncomponents);
-                          INT16DECODE(p, rig->datadesc[type].interlace);
+                          INT16DECODE(p, int16var);
+                          rig->datadesc[type].ncomponents=(intn)int16var;
+                          INT16DECODE(p, int16var);
+                          rig->datadesc[type].interlace=(intn)int16var;
                           UINT16DECODE(p, rig->datadesc[type].compr.tag);
                           UINT16DECODE(p, rig->datadesc[type].compr.ref);
                       }
                     else
+                      {
+                    	DFdifree(GroupID);
                         HGOTO_ERROR(DFE_READERROR, FAIL);
+                    }
 
                     if (rig->datadesc[type].nt.tag == 0)
                         break;  /* old RIGs */
@@ -562,9 +576,15 @@ DFGRgetrig(int32 file_id, uint16 ref, DFGRrig * rig)
                     /* read NT */
                     if (Hgetelement(file_id, rig->datadesc[type].nt.tag,
                               rig->datadesc[type].nt.ref, ntstring) == FAIL)
+                      {
+                    	DFdifree(GroupID);
                         HGOTO_ERROR(DFE_READERROR, FAIL);
+                      }
                     if ((ntstring[2] != 8) || (ntstring[1] != DFNT_UCHAR))
+                      {
+                     	DFdifree(GroupID);
                         HGOTO_ERROR(DFE_BADCALL, FAIL);
+                      }
                     break;
                 default:    /* ignore unknown tags */
                     break;
@@ -626,7 +646,7 @@ DFGRaddrig(int32 file_id, uint16 ref, DFGRrig * rig)
               HGOTO_ERROR(DFE_PUTELEM, FAIL);
           rig->datadesc[IMAGE].nt.tag = DFTAG_NT;
           rig->datadesc[IMAGE].nt.ref = ref;
-          Ref.nt = ref;
+          Ref.nt = (intn)ref;
       }
 
     if (Ref.dims[IMAGE] == 0)
@@ -647,7 +667,7 @@ DFGRaddrig(int32 file_id, uint16 ref, DFGRrig * rig)
                           GRtbuf, (int32) (p - GRtbuf)) == FAIL)
               HGOTO_ERROR(DFE_PUTELEM, FAIL);
 
-          Ref.dims[IMAGE] = ref;
+          Ref.dims[IMAGE] = (int16)ref;
       }
     if (!Ref.lut)
       {     /* associated lut not written to this file */
@@ -660,7 +680,7 @@ DFGRaddrig(int32 file_id, uint16 ref, DFGRrig * rig)
               HGOTO_ERROR(DFE_PUTELEM, FAIL);
           rig->data[LUT].tag = DFTAG_LUT;
           rig->data[LUT].ref = ref;
-          Ref.lut = ref;
+          Ref.lut = (intn)ref;
       }
 
     if (Ref.dims[LUT] == 0)
@@ -678,7 +698,7 @@ DFGRaddrig(int32 file_id, uint16 ref, DFGRrig * rig)
           if (Hputelement(file_id, DFTAG_LD, ref,
                           GRtbuf, (int32) (p - GRtbuf)) == FAIL)
               HGOTO_ERROR(DFE_PUTELEM, FAIL);
-          Ref.dims[LUT] = ref;
+          Ref.dims[LUT] = (int16)ref;
       }
 
     /* prepare to start writing rig */
@@ -806,7 +826,7 @@ DFGRIriginfo(int32 file_id)
 {
     CONSTR(FUNC, "DFGRIriginfo");
     int         i, isfirst;
-    uint16      newref = 0, newtag = 0, gettag, getref, ref, dummy;
+    uint16      newref = 0, newtag = 0, gettag, getref, ref=0, dummy=0;
     struct
       {
           uint16      xdim;
@@ -846,7 +866,7 @@ DFGRIriginfo(int32 file_id)
             {   /* not found */
                 if (gettag == DFTAG_RIG)
                   {     /* were looking for RIGs */
-                      if ((Grread.data[IMAGE].tag == DFTAG_RI)  /* file has Rigs */
+                      if ((Grread.data[IMAGE].tag == DFTAG_RI)  /* file has RIGs */
                           || (Grread.data[IMAGE].tag == DFTAG_CI))
                           HGOTO_DONE(FAIL);  /* no more to return */
                       gettag = DFTAG_RI8;   /* if no RIGs in file, look for RI8s */
@@ -878,6 +898,8 @@ DFGRIriginfo(int32 file_id)
       }
     else
       {
+          uint16    uint16var;
+
           Grread.data[IMAGE].ref = newref;
           Grread.data[IMAGE].tag = newtag;
           if (newtag == DFTAG_CI8)
@@ -888,8 +910,10 @@ DFGRIriginfo(int32 file_id)
           if (Hgetelement(file_id, DFTAG_ID8, newref, (uint8 *) &r8dims) == FAIL)
               HGOTO_ERROR(DFE_GETELEM, FAIL);
           p = (char *) &r8dims;
-          UINT16DECODE(p, Grread.datadesc[IMAGE].xdim);
-          UINT16DECODE(p, Grread.datadesc[IMAGE].ydim);
+          UINT16DECODE(p, uint16var);
+          Grread.datadesc[IMAGE].xdim=(int32)uint16var;
+          UINT16DECODE(p, uint16var);
+          Grread.datadesc[IMAGE].ydim=(int32)uint16var;
 
           aid = Hstartread(file_id, DFTAG_IP8, newref);
           if (aid != FAIL)
@@ -909,7 +933,11 @@ DFGRIriginfo(int32 file_id)
           Grread.datadesc[LUT].ncomponents = 3;
       }
 
+#ifdef OLD_WAY
     Grlastref = Grread.data[IMAGE].ref;     /* remember ref read */
+#else /* OLD_WAY */
+    Grlastref = newref;     /* remember ref read */
+#endif /* OLD_WAY */
 
 done:
   if(ret_value == FAIL)   
@@ -999,7 +1027,7 @@ done:
 int
 DFGRIreqil(intn il, intn type)
 {
-    CONSTR(FUNC, "DFGRreadref");
+    CONSTR(FUNC, "DFGRIreqil");
     intn    ret_value = SUCCEED;
 
     HEclear();
@@ -1044,7 +1072,7 @@ done:
 /* shut lint up */
 /* ARGSUSED */
 int
-DFGRIgetimlut(const char *filename, VOIDP imlut, int32 xdim, int32 ydim,
+DFGRIgetimlut(const char *filename, void * imlut, int32 xdim, int32 ydim,
               int type, int isfortran, int *compressed, uint16 *compr_type, 
 	      int *has_pal)
 {
@@ -1195,7 +1223,7 @@ DFGRIgetimlut(const char *filename, VOIDP imlut, int32 xdim, int32 ydim,
                               }
                         }
                       Hendaccess(aid);
-                      HDfree((VOIDP) buf);
+                      HDfree(buf);
                       HGOTO_DONE(Hclose(file_id));
                   }
             }
@@ -1315,7 +1343,7 @@ done:
 int
 DFGRIrestart(void)
 {
-    CONSTR(FUNC, "DFGRreadref");
+    CONSTR(FUNC, "DFGRIrestart");
     intn    ret_value = SUCCEED;
 
     /* Perform global, one-time initialization */
@@ -1357,12 +1385,13 @@ done:
 /* shut lint up */
 /* ARGSUSED */
 int
-DFGRIaddimlut(const char *filename, VOIDP imlut, int32 xdim, int32 ydim,
+DFGRIaddimlut(const char *filename, const void * imlut, int32 xdim, int32 ydim,
               int type, int isfortran, int newfile)
 {
     CONSTR(FUNC, "DFGRIaddimlut");
     int32       file_id=(-1);
     uint16      wtag, wref;     /* tag of image/lut being written */
+    uint16      rigref;         /* ref # for the RIG */
     uint8      *newlut = NULL;
     int32       lutsize = 0;
     int         is8bit;
@@ -1416,7 +1445,7 @@ DFGRIaddimlut(const char *filename, VOIDP imlut, int32 xdim, int32 ydim,
       {     /* set call */
           if (Grlutdata)
             {
-                HDfree((VOIDP) Grlutdata);
+                HDfree(Grlutdata);
                 Grlutdata = NULL;
             }
           Ref.lut = -1;
@@ -1456,14 +1485,14 @@ DFGRIaddimlut(const char *filename, VOIDP imlut, int32 xdim, int32 ydim,
                 if (( newlut = (uint8 *) HDmalloc((uint32) lutsize))==NULL)
                     HGOTO_ERROR(DFE_NOSPACE, FAIL);
             }
-          if (DFputcomp(file_id, wtag, wref, (uint8 *) imlut, xdim, ydim,
+          if (DFputcomp(file_id, wtag, wref, imlut, xdim, ydim,
            (uint8 *) Grlutdata, (uint8 *) newlut, (int16) Grcompr, &Grcinfo)
               == FAIL)
               HGOTO_ERROR(DFE_NOSPACE, FAIL);
       }
     else
       {     /* image need not be compressed */
-          if (Hputelement(file_id, (uint16) wtag, (uint16) wref, (uint8 *) imlut,
+          if (Hputelement(file_id, (uint16) wtag, (uint16) wref, imlut,
                   xdim * ydim * Grwrite.datadesc[type].ncomponents) == FAIL)
               HGOTO_ERROR(DFE_PUTELEM, FAIL);
       }
@@ -1488,10 +1517,12 @@ DFGRIaddimlut(const char *filename, VOIDP imlut, int32 xdim, int32 ydim,
       {
           if (Hputelement(file_id, DFTAG_LUT, wref, newlut, lutsize) == FAIL)
               HGOTO_ERROR(DFE_PUTELEM, FAIL);
-          Ref.lut = wref;
+          Ref.lut = (intn)wref;
       }
 
-    if (DFGRaddrig(file_id, wref, &Grwrite) == FAIL)    /* writes ID, NT */
+    if (( rigref = Htagnewref(file_id,DFTAG_RIG))==0)
+        HGOTO_ERROR(DFE_INTERNAL,FAIL);
+    if (DFGRaddrig(file_id, rigref, &Grwrite) == FAIL)    /* writes ID, NT */
         HGOTO_ERROR(DFE_INTERNAL, FAIL);
 
     if (is8bit)
@@ -1510,11 +1541,15 @@ DFGRIaddimlut(const char *filename, VOIDP imlut, int32 xdim, int32 ydim,
     if (Grcompr == DFTAG_IMC)
       {
           Ref.lut = 0;
-          HDfree((VOIDP) newlut);
+          HDfree(newlut);
           newlut = NULL;
       }
 
+#ifdef OLD_WAY
     Grlastref = wref;   /* remember the last ref */
+#else /* OLD_WAY */
+    Grlastref = rigref;   /* remember the last ref */
+#endif /* OLD_WAY */
 
     wref = 0;   /* don't know ref to write next */
 
