@@ -11,10 +11,10 @@
  ****************************************************************************/
 
 #ifdef RCSID
-static char RcsId[] = "@(#)1.19";
+static char RcsId[] = "@(#)$Revision: 1.24 $";
 #endif
 
-/* dfimcomp.c,v 1.19 1996/03/28 21:57:09 koziol Exp */
+/* $Id: dfimcomp.c,v 1.24 1998/02/02 21:38:40 smitchel Exp $ */
 
 /************************************************************************/
 /*  Module Name : imcomp                                                */
@@ -33,7 +33,7 @@ static char RcsId[] = "@(#)1.19";
 #define BIT8 0
 #define BIT24 1
 
-#if (!defined MAC && !defined __MWERKS__ && !defined SYMANTEC_C )
+#if (!defined MAC && !defined macintosh && !defined SYMANTEC_C )
 #define MAXCOLOR 32768
 #else  /*MAC */
 #define MAXCOLOR 768
@@ -122,7 +122,7 @@ PRIVATE int next_pt(int dim, int i, int rank[], int distinct);
 /************************************************************************/
 
 VOID
-DFCIimcomp(int32 xdim, int32 ydim, uint8 in[], uint8 out[],
+DFCIimcomp(int32 xdim, int32 ydim, const uint8 *in, uint8 out[],
            uint8 in_pal[], uint8 out_pal[], int mode)
 {
     unsigned char raster[48];
@@ -168,8 +168,7 @@ DFCIimcomp(int32 xdim, int32 ydim, uint8 in[], uint8 out[],
                         break;
 
                     default:    /* unsupported format */
-                        printf("Error : Unsupported Format \n");
-                        exit(1);
+                        break;
                 }   /* end of switch */
 
               blocks++;
@@ -266,7 +265,7 @@ compress(unsigned char raster[], int block)
                           c_lo[l] = c_lo[l] + (int) raster[3 * j + l];
                   }     /* end of if */
 
-                bit >>= 1;
+                bit = (uint8)(bit>>1);
             }   /* end of for j */
 
           k++;
@@ -279,8 +278,8 @@ compress(unsigned char raster[], int block)
               color_pt[hi].c[i] = (uint8) ((float) c_hi[i] / (float) high);
           if (high != 16)
               color_pt[lo].c[i] = (uint8) ((float) c_lo[i] / (float) (16 - high));
-          color_pt[hi].c[i] >>= 3;
-          color_pt[lo].c[i] >>= 3;
+          color_pt[hi].c[i] = (uint8)(color_pt[hi].c[i] >> 3);
+          color_pt[lo].c[i] = (uint8)(color_pt[lo].c[i] >> 3);
 
       }
 }   /* end of compress */
@@ -312,8 +311,7 @@ init_global(int32 xdim, int32 ydim, VOIDP out, VOIDP out_pal)
 
     if (image == NULL || color_pt == NULL || new_pal == NULL)
       {
-          printf("Error : Out of Memory\n");
-          exit(1);
+          return; /* punt! */
       }
 
     /* initialize */
@@ -351,7 +349,8 @@ cnt_color(int blocks)
       {
           k = indx(color_pt[i].c[RED], color_pt[i].c[GREEN], color_pt[i].c[BLUE]);
           /*    printf("cnt_color: k is %d\n",k); */
-          temp[k] = 0;
+          if (k < MAXCOLOR) /* Fortner Fix: supplied by Peter Lawton */
+              temp[k] = 0;
       }
 
     count = 0;
@@ -496,13 +495,13 @@ nearest_color(uint8 r, uint8 g, uint8 b)
     int         i, nearest;
     long int    min, error;
 
-    min = sqr((int16) (r - new_pal[0])) + sqr((int16) (g - new_pal[1])) +
-        sqr((int16) (b - new_pal[2]));
+    min = (long)(sqr((int16) (r - new_pal[0])) + sqr((int16) (g - new_pal[1])) +
+        sqr((int16) (b - new_pal[2])));
     nearest = 0;
     for (i = 1; i < PALSIZE; i++)
       {
-          error = sqr((int16) (r - new_pal[3 * i])) + sqr((int16) (g - new_pal[3 * i + 1])) +
-              sqr((int16) (b - new_pal[3 * i + 2]));
+          error = (long)(sqr((int16) (r - new_pal[3 * i])) + sqr((int16) (g - new_pal[3 * i + 1])) +
+              sqr((int16) (b - new_pal[3 * i + 2])));
           if (error < min)
             {
                 min = error;
@@ -526,7 +525,7 @@ nearest_color(uint8 r, uint8 g, uint8 b)
 PRIVATE uint32
 sqr(int16 x)
 {
-    return ((int32) x * (int32) x);
+    return ((uint32) x * (uint32) x);
 }
 
 /************************************************************************/
@@ -837,8 +836,7 @@ find_box(void)
 
     if (max == NULL)
       {
-          printf("Error : Number of color points < palette \n");
-          exit(1);
+          return(NULL); /* punt! */
       }
 
     return max;
