@@ -11,10 +11,10 @@
  ****************************************************************************/
 
 #ifdef RCSID
-static char RcsId[] = "@(#)1.14";
+static char RcsId[] = "@(#)$Revision: 1.14 $";
 #endif
 
-/* dfkfuji.c,v 1.14 1996/01/19 01:13:21 koziol Exp */
+/* $Id: dfkfuji.c,v 1.14 1996/01/19 01:13:21 koziol Exp $ */
 
 /*------------------------------------------------------------------
  File:  dfkfuji.c
