@@ -12,10 +12,10 @@
  ****************************************************************************/
 
 #ifdef RCSID
-static char RcsId[] = "@(#)1.13";
+static char RcsId[] = "@(#)$Revision: 1.13 $";
 #endif
 
-/* dfkvms.c,v 1.13 1994/10/19 20:55:46 koziol Exp */
+/* $Id: dfkvms.c,v 1.13 1994/10/19 20:55:46 koziol Exp $ */
 
 /*------------------------------------------------------------------
  File:  dfkvms.c
