@@ -11,10 +11,10 @@
  ****************************************************************************/
 
 #ifdef RCSID
-static char RcsId[] = "@(#)1.24";
+static char RcsId[] = "@(#)$Revision: 1.27 $";
 #endif
 
-/* dfp.c,v 1.24 1996/03/28 21:57:10 koziol Exp */
+/* $Id: dfp.c,v 1.27 1997/10/24 21:00:29 koziol Exp $ */
 
 /*-----------------------------------------------------------------------------
  * File:     dfp.c
@@ -34,6 +34,7 @@ static char RcsId[] = "@(#)1.24";
 
 #include "hdf.h"
 
+/* remember that '0' is invalid ref number */
 PRIVATE uint16 Readref = 0;
 PRIVATE uint16 Writeref = 0;
 PRIVATE uint16 Refset = 0;      /* Ref of palette to get next */
@@ -50,7 +51,7 @@ PRIVATE int32 DFPIopen
  USAGE
     intn DFPgetpal(filename,palette)
         char *filename;         IN: name of HDF file
-        VOIDP palette;          OUT: ptr to the buffer to store the palette in
+        void * palette;          OUT: ptr to the buffer to store the palette in
  RETURNS
     SUCCEED on success, FAIL on failure.
  DESCRIPTION
@@ -62,7 +63,7 @@ PRIVATE int32 DFPIopen
  REVISION LOG
 --------------------------------------------------------------------------*/
 intn
-DFPgetpal(const char *filename, VOIDP palette)
+DFPgetpal(const char *filename, void * palette)
 {
   CONSTR(FUNC, "DFPgetpal");
   int32       file_id;
@@ -160,7 +161,7 @@ done:
  USAGE
     intn DFPputpal(filename,palette,overwrite,filemode)
         char *filename;         IN: name of HDF file
-        VOIDP palette;          IN: ptr to the buffer retrieve the palette from
+        void * palette;          IN: ptr to the buffer retrieve the palette from
         intn overwrite;         IN: whether to (1) overwrite last palette written,
                                     or (0) write it as a fresh palette
         char *filemode;         IN: if "a" append palette to file, "w" create
@@ -177,7 +178,7 @@ done:
  REVISION LOG
 --------------------------------------------------------------------------*/
 intn
-DFPputpal(const char *filename, const VOIDP palette, intn overwrite, const char *filemode)
+DFPputpal(const char *filename, const void * palette, intn overwrite, const char *filemode)
 {
   CONSTR(FUNC, "DFPputpal");
   int32       file_id;
@@ -242,7 +243,7 @@ done:
  USAGE
     intn DFPaddpal(filename,palette)
         char *filename;         IN: name of HDF file
-        VOIDP palette;          IN: ptr to the buffer retrieve the palette from
+        void * palette;          IN: ptr to the buffer retrieve the palette from
  RETURNS
     SUCCEED on success, FAIL on failure.
  DESCRIPTION
@@ -253,7 +254,7 @@ done:
  REVISION LOG
 --------------------------------------------------------------------------*/
 intn
-DFPaddpal(const char *filename, const VOIDP palette)
+DFPaddpal(const char *filename, const void * palette)
 {
   intn ret_value;
 
@@ -361,14 +362,18 @@ DFPnpals(const char *filename)
   npals = curr_pal;   /* reset the number of palettes we really have */
   for (i = 1; i < curr_pal; i++)
     {     /* go through the palettes looking for duplicates */
-      for (j = 0; j < i; j++)
-        {
-          if (pal_off[i] == pal_off[j])
-            npals--;    /* if duplicate found, decrement the number of palettes */
-        }   /* end for */
+      if(pal_off[i]!=(-1))
+          for (j = 0; j < i; j++)
+            {
+              if (pal_off[i] == pal_off[j])
+                {
+                    npals--;    /* if duplicate found, decrement the number of palettes */
+                    pal_off[j]=(-1); /* mark as used, so we don't count it too... */
+                } /* end if */
+            }   /* end for */
     }     /* end for */
 
-  HDfree((VOIDP) pal_off);   /* free offsets */
+  HDfree(pal_off);   /* free offsets */
 
   if (Hclose(file_id) == FAIL)
     HGOTO_ERROR(DFE_CANTCLOSE, FAIL);
@@ -529,7 +534,7 @@ DFPrestart(void)
  USAGE
     uint16 DFPlastref(void)
  RETURNS
-    ref # on on success, FAIL on failure.
+    ref # on on success, 0 on failure.
  DESCRIPTION
     Return the last ref # read/written from a file.
  GLOBAL VARIABLES
