@@ -11,10 +11,10 @@
  ****************************************************************************/
 
 #ifdef RCSID
-static char RcsId[] = "@(#)1.34";
+static char RcsId[] = "@(#)$Revision: 1.41 $";
 #endif
 
-/* dfr8.c,v 1.34 1996/05/20 22:13:10 koziol Exp */
+/* $Id: dfr8.c,v 1.41 1997/12/18 16:50:43 koziol Exp $ */
 
 /*-----------------------------------------------------------------------------
  * File:     dfr8.c
@@ -134,7 +134,7 @@ PRIVATE intn library_terminate = FALSE;
 
 /* private functions */
 PRIVATE intn DFR8Iputimage
-            (const char *filename, VOIDP image, int32 xdim, int32 ydim, uint16 compress,
+            (const char *filename, const void * image, int32 xdim, int32 ydim, uint16 compress,
              intn append);
 
 PRIVATE int32 DFR8Iopen
@@ -205,7 +205,7 @@ DFR8setcompress(int32 type, comp_info * cinfo)
   if (type == COMP_JPEG)
   CompType = DFTAG_GREYJPEG5;
   else    /* otherwise, just use mapped tag */
-    CompType = compress_map[type];
+    CompType = (int32)compress_map[type];
   CompInfo = (*cinfo);
 
 done:
@@ -489,7 +489,7 @@ done:
  USAGE
     intn DFR8Iputimage(filename, image, xdim, ydim, compress, append)
         char *filename;         IN: name of HDF file
-        VOIDP image;            IN: ptr to buffer image is stored in
+        const void * image;            IN: ptr to buffer image is stored in
         int32 xdim,ydim;        IN: dims of space allocated by user for image
         uint16 compress;        IN: type of compression to store image with
         intn append;            IN: whether to (0) overwrite existing file, or
@@ -511,7 +511,7 @@ done:
  REVISION LOG
 --------------------------------------------------------------------------*/
 PRIVATE intn
-DFR8Iputimage(const char *filename, VOIDP image, int32 xdim, int32 ydim,
+DFR8Iputimage(const char *filename, const void * image, int32 xdim, int32 ydim,
               uint16 compress, intn append)
 {
   CONSTR(FUNC, "DFR8Iputimage");
@@ -551,10 +551,10 @@ DFR8Iputimage(const char *filename, VOIDP image, int32 xdim, int32 ydim,
   if (compress || CompressSet)
     {
       /* if a compression type has been set, check if it's the same */
-      if (CompressSet == FALSE || (compress > 1 && (int32) compress != CompType &&
-                                   !(compress == COMP_JPEG && CompType == DFTAG_GREYJPEG5)))
+      if (CompressSet == FALSE || (compress > (uint16)1 && (int32)compress != CompType &&
+                                   !(compress == (uint16)COMP_JPEG && CompType == (int32)DFTAG_GREYJPEG5)))
         {
-          if (compress > COMP_MAX_COMP || compress_map[compress] == 0)
+          if ((int32)compress > COMP_MAX_COMP || compress_map[compress] == 0)
             HGOTO_ERROR(DFE_BADSCHEME, FAIL);
           /* map JPEG compression into correct type of JPEG compression */
           if (compress == COMP_JPEG)
@@ -565,14 +565,14 @@ DFR8Iputimage(const char *filename, VOIDP image, int32 xdim, int32 ydim,
               CompInfo.jpeg.force_baseline = TRUE;
             }     /* end if */
           else    /* otherwise, just use mapped tag */
-            CompType = compress_map[compress];
+            CompType = (int32)compress_map[compress];
         }   /* end if */
       if (!Writeref)
         if ((Writeref = Hnewref(file_id)) == 0)
           HGOTO_ERROR(DFE_NOREF, FAIL);
 
-      if (DFputcomp(file_id, DFTAG_CI, Writeref, (uint8 *) image, xdim, ydim,
-                    pal, (uint8 *) newpal, (int16) CompType, &CompInfo) == FAIL)
+      if (DFputcomp(file_id, DFTAG_CI, Writeref, image, xdim, ydim,
+                    pal, newpal, (int16) CompType, &CompInfo) == FAIL)
         HGOTO_ERROR(DFE_WRITEERROR, FAIL);
       Writerig.image.tag = DFTAG_CI;
       if (CompType == DFTAG_IMC)
@@ -587,7 +587,7 @@ DFR8Iputimage(const char *filename, VOIDP image, int32 xdim, int32 ydim,
         if ((Writeref = Hnewref(file_id)) == 0)
           HGOTO_ERROR(DFE_NOREF, FAIL);
 
-      if (Hputelement(file_id, DFTAG_RI, Writeref, (uint8 *) image, xdim * ydim) == FAIL)
+      if (Hputelement(file_id, DFTAG_RI, Writeref, image, xdim * ydim) == FAIL)
         HGOTO_ERROR(DFE_PUTELEM, FAIL);
       Writerig.image.tag = DFTAG_RI;
     }     /* end else */
@@ -671,7 +671,7 @@ done:
  USAGE
     intn DFR8putimage(filename, image, xdim, ydim, compress)
         char *filename;         IN: name of HDF file
-        VOIDP image;            IN: ptr to buffer to store image in
+        const void * image;            IN: ptr to buffer to store image in
         int32 xdim,ydim;        IN: dims of space allocated by user for image
         uint16 compress;        IN: type of compression to store image with
  RETURNS
@@ -688,7 +688,7 @@ done:
  REVISION LOG
 --------------------------------------------------------------------------*/
 intn
-DFR8putimage(const char *filename, VOIDP image, int32 xdim, int32 ydim,
+DFR8putimage(const char *filename, const void * image, int32 xdim, int32 ydim,
              uint16 compress)
 {
     CONSTR(FUNC, "DFR8putimage");    /* for HERROR */
@@ -726,7 +726,7 @@ done:
  USAGE
     intn DFR8putimage(filename, image, xdim, ydim, compress)
         char *filename;         IN: name of HDF file
-        VOIDP image;            IN: ptr to buffer to store image in
+        const void * image;            IN: ptr to buffer to store image in
         int32 xdim,ydim;        IN: dims of space allocated by user for image
         uint16 compress;        IN: type of compression to store image with
  RETURNS
@@ -744,7 +744,7 @@ done:
  REVISION LOG
 --------------------------------------------------------------------------*/
 intn
-DFR8addimage(const char *filename, VOIDP image, int32 xdim, int32 ydim,
+DFR8addimage(const char *filename, const void * image, int32 xdim, int32 ydim,
              uint16 compress)
 {
     CONSTR(FUNC, "DFR8addimage");    /* for HERROR */
@@ -858,20 +858,30 @@ DFR8getrig(int32 file_id, uint16 ref, DFRrig * rig)
             }     /* end if */
           else
             {
+              DFdifree(GroupID);
               ret_value = FAIL;
               goto done;
             }
           if (rig->descimage.ncomponents != 1)
-            HGOTO_ERROR(DFE_BADCALL, FAIL);
+            {
+              DFdifree(GroupID);
+              HGOTO_ERROR(DFE_BADCALL, FAIL);
+            }
           if (rig->descimage.nt.tag == 0)
             break;  /* old RIGs */
 
           /* read NT */
           if (Hgetelement(file_id, rig->descimage.nt.tag,
                           rig->descimage.nt.ref, ntstring) == FAIL)
-            HGOTO_ERROR(DFE_GETELEM, FAIL);
+            {
+              DFdifree(GroupID);
+              HGOTO_ERROR(DFE_GETELEM, FAIL);
+            }
           if ((ntstring[2] != 8) || (ntstring[1] != DFNT_UCHAR))
-            HGOTO_ERROR(DFE_BADCALL, FAIL);
+            {
+              DFdifree(GroupID);
+              HGOTO_ERROR(DFE_BADCALL, FAIL);
+            }
           break;
 
         default:    /* ignore unknown tags */
@@ -1100,21 +1110,25 @@ DFR8nimages(const char *filename)
             {     /* just look for ID tags to get the number of components */
               if (Hgetelement(file_id, elt_tag, elt_ref, GRtbuf) != FAIL)
                 {
+                  uint16      temp16;   /* temporary holding variable */
                   int32       temp;   /* temporary holding variable */
-                  int32       ncomponents;    /* number of image components */
+                  int16       ncomponents;    /* number of image components */
                   uint8      *p;
 
                   p = GRtbuf;
                   INT32DECODE(p, temp);
                   INT32DECODE(p, temp);
-                  UINT16DECODE(p, temp);
-                  UINT16DECODE(p, temp);
+                  UINT16DECODE(p, temp16);
+                  UINT16DECODE(p, temp16);
                   INT16DECODE(p, ncomponents);
                   if (ncomponents == 1)   /* whew, all that work and we finally found an 8-bit image */
                     found_8bit = TRUE;
                 }   /* end if */
               else
-                HGOTO_ERROR(DFE_GETELEM, FAIL);
+                {
+                  DFdifree(group_id);
+                  HGOTO_ERROR(DFE_GETELEM, FAIL);
+                }
             }     /* end if */
           else
             /* check for the image tag/ref */ if (elt_tag == DFTAG_CI || elt_tag == DFTAG_RI)
@@ -1125,7 +1139,7 @@ DFR8nimages(const char *filename)
         }   /* end while */
       if (found_8bit)
         {   /* check for finding an 8-bit RIG */
-          if (rig_tag > 0 && rig_ref > 0)
+          if ((uintn)rig_tag > (uintn)0 && (uintn)rig_ref > (uintn)0)
             {     /* make certain we found an image */
               img_off[curr_image] = Hoffset(file_id, rig_tag, rig_ref);     /* store offset */
               curr_image++;
@@ -1155,11 +1169,14 @@ DFR8nimages(const char *filename)
       for (j = 0; j < i; j++)
         {
           if (img_off[i] == img_off[j])
-            nimages--;  /* if duplicate found, decrement the number of images */
+            {
+                nimages--;  /* if duplicate found, decrement the number of images */
+                img_off[j]=(-1); /* mark as used, so we don't count it too... */
+            } /* end if */
         }   /* end for */
     }     /* end for */
 
-  HDfree((VOIDP) img_off);   /* free offsets */
+  HDfree(img_off);   /* free offsets */
   if (Hclose(file_id) == FAIL)
     HGOTO_ERROR(DFE_CANTCLOSE, FAIL);
 
@@ -1351,7 +1368,7 @@ done:
  USAGE
     uint16 DFR8lastref(void)
  RETURNS
-    Ref # on success, FAIL on failure.
+    Ref # on success, 0 on failure.
  DESCRIPTION
     Returns the last ref # written to or read from.
  GLOBAL VARIABLES
@@ -1373,12 +1390,12 @@ DFR8lastref(void)
   /* Perform global, one-time initialization */
   if (library_terminate == FALSE)
       if(DFR8Istart()==FAIL)
-          HGOTO_ERROR(DFE_CANTINIT, FAIL);
+          HGOTO_ERROR(DFE_CANTINIT, 0);
 
   ret_value = Lastref;
 
 done:
-  if(ret_value == (uint16)FAIL)   
+  if(ret_value == 0)   /* 0 is invalid ref */
     { /* Error condition cleanup */
 
     } /* end if */
@@ -1657,10 +1674,13 @@ DFR8Iriginfo(int32 file_id)
       if (Hgetelement(file_id, DFTAG_ID8, Readrig.image.ref, R8tbuf) != FAIL)
         {
           uint8      *p;
+          uint16      uint16var;
 
           p = R8tbuf;
-          UINT16DECODE(p, Readrig.descimage.xdim);
-          UINT16DECODE(p, Readrig.descimage.ydim);
+          UINT16DECODE(p, uint16var);
+          Readrig.descimage.xdim=(int32)uint16var;
+          UINT16DECODE(p, uint16var);
+          Readrig.descimage.ydim=(int32)uint16var;
         }   /* end if */
       else
         HGOTO_ERROR(DFE_GETELEM, FAIL);
@@ -1757,3 +1777,4 @@ intn DFR8Pshutdown(void)
     return(SUCCEED);
 } /* end DFR8Pshutdown() */
 
+
