@@ -17,19 +17,26 @@
  *
  ****************************************************************************************/
 
-#if defined(MAC) || defined (__MWERKS__) || defined(macintosh) || defined(SYMANTEC_C)
+#if defined(MAC) || defined(macintosh) || defined(SYMANTEC_C)
+
+/*
+ * Added to HDF core library for directory reading on the Macintosh
+ *
+ * 1996/03/01 - GeorgeV.
+ */
 
 #ifdef RCSID
-static char RcsId[] = "@(#)1.2";
+static char RcsId[] = "@(#)$Revision: 1.8 $";
 #endif
 
-/* dir_mac.c,v 1.2 1996/03/01 18:59:46 georgev Exp */
+/* $Id: dir_mac.c,v 1.8 1998/02/02 21:38:42 smitchel Exp $ */
 
 #include "dir_mac.h"
 #if 0
 #include <pascal.h>
 #endif
 #include <string.h>
+#include <ctype.h>
 #include <Errors.h>
 
 OSErr	dd_errno;				/*	Global errno to check after calls to dirent routines	*/
@@ -228,6 +235,15 @@ hopendir(char *dirname, short vRefNum, long dirID)
 	HLock((char **)curh);
 	cur		= *curh;
 
+        /* Make sure the path ends in a ':' */
+        if (dirname != nil)             {
+                while ((isspace(dirname[dirname[0]])) && (dirname[0] > 0))
+                        dirname[0]--;
+
+                if ((dirname[dirname[0]] != ':') && (dirname[0]<255))
+                        dirname[++dirname[0]] = ':';
+        }
+
 	/*	If we're supposed to open anything but the current directory, set the current
 	 *	working directory to the desired directory.
 	 */
@@ -239,7 +255,13 @@ hopendir(char *dirname, short vRefNum, long dirID)
           pb.ioWDDirID				= dirID;
 		
           if (dd_errno = PBHSetVol(&pb, false))
-              goto failure_exit;
+          	;	// Don't fail. First try a local change.
+//              goto failure_exit;
+          else /* Fortner supplied fix 8/26/97 */
+            {  /* make vRefNum and dirId valid */
+                vRefNum = pb.ioVRefNum;
+                dirID = pb.ioWDDirID;
+            }
       }
 
 	cur->dd_buf	= nil;
@@ -657,4 +679,4 @@ pathdir(DIR *dirp, char *path)
 	return path;
 }
 
-#endif /* defined(MAC) || defined (__MWERKS__) || defined(macintosh) || defined(SYMANTEC_C) */
+#endif /* defined(MAC) || defined(macintosh) || defined(SYMANTEC_C) */
