@@ -11,10 +11,10 @@
  ****************************************************************************/
 
 #ifdef RCSID
-static char RcsId[] = "@(#)1.44";
+static char RcsId[] = "@(#)$Revision: 1.54 $";
 #endif
 
-/* hcomp.c,v 1.44 1996/06/17 21:14:38 koziol Exp */
+/* $Id: hcomp.c,v 1.54 1997/10/24 21:00:37 koziol Exp $ */
 
 /*
 FILE
@@ -101,11 +101,12 @@ PRIVATE int32 HCIinit_coder
              comp_info * coder_info);
 
 PRIVATE int32 HCIread_header
-            (filerec_t * file_rec, accrec_t * access_rec,
-             compinfo_t * info, comp_info * c_info, model_info * m_info);
+            (accrec_t * access_rec, compinfo_t * info,
+                comp_info * c_info, model_info * m_info);
 
 PRIVATE int32 HCIwrite_header
-            (atom_t file_id, compinfo_t * info, uint16 special_tag, uint16 ref);
+            (atom_t file_id, compinfo_t * info, uint16 special_tag, uint16 ref,
+            comp_info *c_info, model_info *m_info);
 
 PRIVATE int32 HCIinit_model
             (int16 acc_mode, comp_model_info_t * minfo, comp_model_t model_type,
@@ -125,6 +126,7 @@ funclist_t  comp_funcs =
     HCPwrite,
     HCPendaccess,
     HCPinfo,
+    NULL         /* no routine registerd */
 };
 
 /* #define TESTING */
@@ -183,6 +185,9 @@ HCIinit_coder(int16 acc_mode, comp_coder_info_t * cinfo, comp_coder_t coder_type
               break;
 
           case COMP_CODE_SKPHUFF:   /* Skipping Huffman encoding */
+              if(c_info->skphuff.skp_size<1)
+                  HRETURN_ERROR(DFE_BADCODER, FAIL)
+
               cinfo->coder_type = COMP_CODE_SKPHUFF;    /* set the coding type */
               cinfo->coder_funcs = cskphuff_funcs;  /* set the skipping huffman func. ptrs */
 
@@ -191,6 +196,9 @@ HCIinit_coder(int16 acc_mode, comp_coder_info_t * cinfo, comp_coder_t coder_type
               break;
 
           case COMP_CODE_DEFLATE:   /* gzip 'deflate' encoding */
+              if(c_info->deflate.level<1 || c_info->deflate.level>9)
+                  HRETURN_ERROR(DFE_BADCODER, FAIL)
+
               cinfo->coder_type = COMP_CODE_DEFLATE;    /* set the coding type */
               cinfo->coder_funcs = cdeflate_funcs;  /* set the gzip 'deflate' func. ptrs */
 
@@ -200,7 +208,7 @@ HCIinit_coder(int16 acc_mode, comp_coder_info_t * cinfo, comp_coder_t coder_type
               break;
 
           default:
-              HRETURN_ERROR(DFE_BADCODER, FAIL);
+              HRETURN_ERROR(DFE_BADCODER, FAIL)
       }     /* end switch */
     return (SUCCEED);
 }   /* end HCIinit_coder() */
@@ -232,6 +240,7 @@ HCIinit_model(int16 acc_mode, comp_model_info_t * minfo, comp_model_t model_type
     CONSTR(FUNC, "HCIinit_model");  /* for HERROR */
 
     /* shut compiler up */
+    acc_mode = acc_mode;
     m_info = m_info;
 
     switch (model_type)
@@ -242,12 +251,285 @@ HCIinit_model(int16 acc_mode, comp_model_info_t * minfo, comp_model_t model_type
               break;
 
           default:
-              HRETURN_ERROR(DFE_BADMODEL, FAIL);
+              HRETURN_ERROR(DFE_BADMODEL, FAIL)
       }     /* end switch */
 
     return (SUCCEED);
 }   /* end HCIinit_model() */
 
+/*--------------------------------------------------------------------------
+ NAME
+    HCPquery_encode_header -- Query the length of compression header for a memory buffer
+ USAGE
+    int32 HCPquery_encode_header(model_type, model_info, coder_type, coder_info)
+    comp_model_t model_type; IN: the type of modeling to use
+    model_info *m_info;      IN: Information needed for the modeling type chosen
+    comp_coder_t coder_type; IN: the type of encoding to use
+    coder_info *c_info;      IN: Information needed for the encoding type chosen
+
+ RETURNS
+    Return the length of the buffer needed to store the compression header on
+    success (>0) or FAIL (-1)
+ DESCRIPTION
+    Determine the compression information length in a memory block.
+
+ GLOBAL VARIABLES
+ COMMENTS, BUGS, ASSUMPTIONS
+ EXAMPLES
+ REVISION LOG
+--------------------------------------------------------------------------*/
+int32
+HCPquery_encode_header(comp_model_t model_type, model_info * m_info,
+         comp_coder_t coder_type, comp_info * c_info)
+{
+    CONSTR(FUNC, "HCPquery_encode_header");    /* for HERROR */
+    int32 coder_len=2;  /* # of bytes to encode coder information (2 minimum) */
+    int32 model_len=2;  /* # of bytes to encode model information (2 minimum) */
+    int32 ret_value=SUCCEED;
+    
+    /* clear error stack and validate args */
+    HEclear();
+    if (m_info==NULL || c_info==NULL)
+        HGOTO_ERROR(DFE_ARGS, FAIL);
+
+    /* add any additional information needed for modeling type */
+    switch (model_type)
+      {
+          default:      /* no additional information needed */
+              break;
+      }     /* end switch */
+
+    /* add any additional information needed for coding type */
+    switch (coder_type)
+      {
+          case COMP_CODE_NBIT:      /* N-bit coding needs 16 bytes of info */
+              coder_len+=16;
+              break;
+
+          case COMP_CODE_SKPHUFF:   /* Skipping Huffman coding needs 8 bytes of info */
+              coder_len+=8;
+              break;
+
+          case COMP_CODE_DEFLATE:   /* Deflation coding stores deflation level */
+              coder_len+=2;
+              break;
+
+          default:      /* no additional information needed */
+              break;
+      }     /* end switch */
+
+    ret_value=model_len+coder_len;
+
+done:
+    if(ret_value == FAIL)   
+      { /* Error condition cleanup */
+  
+      } /* end if */
+
+    /* Normal function cleanup */
+    return ret_value; 
+} /* end HCPquery_encode_header() */
+
+/*--------------------------------------------------------------------------
+ NAME
+    HCPencode_header -- Encode the compression header info to a memory buffer
+ USAGE
+    intn HCPencode_header(model_type, model_info, coder_type, coder_info)
+    void * buf;               OUT: encoded compression info header
+    comp_model_t model_type; IN: the type of modeling to use
+    model_info *m_info;      IN: Information needed for the modeling type chosen
+    comp_coder_t coder_type; IN: the type of encoding to use
+    coder_info *c_info;      IN: Information needed for the encoding type chosen
+
+ RETURNS
+    Return SUCCEED or FAIL
+ DESCRIPTION
+    Encodes the compression information to a block in memory.
+
+ GLOBAL VARIABLES
+ COMMENTS, BUGS, ASSUMPTIONS
+ EXAMPLES
+ REVISION LOG
+--------------------------------------------------------------------------*/
+intn
+HCPencode_header(uint8 *p, comp_model_t model_type, model_info * m_info,
+         comp_coder_t coder_type, comp_info * c_info)
+{
+    CONSTR(FUNC, "HCPencode_header");    /* for HERROR */
+    int32 ret_value=SUCCEED;
+    
+    /* clear error stack and validate args */
+    HEclear();
+    if (p==NULL || m_info==NULL || c_info==NULL)
+        HGOTO_ERROR(DFE_ARGS, FAIL);
+
+    UINT16ENCODE(p, (uint16) model_type);   /* specify model type */
+    UINT16ENCODE(p, (uint16) coder_type);   /* specify coder type */
+
+    /* add any additional information needed for modeling type */
+    switch (model_type)
+      {
+          default:      /* no additional information needed */
+              break;
+      }     /* end switch */
+
+    /* add any additional information needed for coding type */
+    switch (coder_type)
+      {
+          case COMP_CODE_NBIT:      /* N-bit coding needs info */
+              /* specify number-type of N-bit data */
+              INT32ENCODE(p, c_info->nbit.nt);
+              /* next is the flag to indicate whether to sign extend */
+              UINT16ENCODE(p, (uint16) c_info->nbit.sign_ext);
+              /* flag to fill with 1's or 0's */
+              UINT16ENCODE(p, (uint16) c_info->nbit.fill_one);
+              /* the offset of the bits extracted */
+              INT32ENCODE(p, (int32) c_info->nbit.start_bit);
+              /* the number of bits extracted */
+              INT32ENCODE(p, (int32) c_info->nbit.bit_len);
+              break;
+
+          case COMP_CODE_SKPHUFF:   /* Skipping Huffman coding needs info */
+              if(c_info->skphuff.skp_size<1)
+                  HRETURN_ERROR(DFE_BADCODER, FAIL)
+
+              /* specify skipping unit size */
+              UINT32ENCODE(p, (uint32) c_info->skphuff.skp_size);
+              /* specify # of bytes compressed (not used currently) */
+              UINT32ENCODE(p, (uint32) c_info->skphuff.skp_size);
+              break;
+
+          case COMP_CODE_DEFLATE:   /* Deflation coding stores deflation level */
+              if(c_info->deflate.level<1 || c_info->deflate.level>9)
+                  HRETURN_ERROR(DFE_BADCODER, FAIL)
+
+              /* specify deflation level */
+              UINT16ENCODE(p, (uint16) c_info->deflate.level);
+              break;
+
+          default:      /* no additional information needed */
+              break;
+      }     /* end switch */
+
+done:
+    if(ret_value == FAIL)   
+      { /* Error condition cleanup */
+  
+      } /* end if */
+
+    /* Normal function cleanup */
+    return ret_value; 
+} /* end HCPencode_header() */
+
+/*--------------------------------------------------------------------------
+ NAME
+    HCPdecode_header -- Decode the compression header info from a memory buffer
+ USAGE
+    intn HCPdecode_header(model_type, model_info, coder_type, coder_info)
+    void * buf;                  IN: encoded compression info header
+    comp_model_t *model_type;   OUT: the type of modeling to use
+    model_info *m_info;         OUT: Information needed for the modeling type chosen
+    comp_coder_t *coder_type;   OUT: the type of encoding to use
+    coder_info *c_info;         OUT: Information needed for the encoding type chosen
+
+ RETURNS
+    Return SUCCEED or FAIL
+ DESCRIPTION
+    Decodes the compression information from a block in memory.
+
+ GLOBAL VARIABLES
+ COMMENTS, BUGS, ASSUMPTIONS
+ EXAMPLES
+ REVISION LOG
+--------------------------------------------------------------------------*/
+intn
+HCPdecode_header(uint8 *p, comp_model_t *model_type, model_info * m_info,
+         comp_coder_t *coder_type, comp_info * c_info)
+{
+    CONSTR(FUNC, "HCPdecode_header");    /* for HERROR */
+    uint16 m_type, c_type;
+    int32 ret_value=SUCCEED;
+
+    /* clear error stack and validate args */
+    HEclear();
+    if (p==NULL || model_type==NULL || m_info==NULL || coder_type==NULL || c_info==NULL)
+        HGOTO_ERROR(DFE_ARGS, FAIL);
+
+    UINT16DECODE(p, m_type);     /* get model type */
+    *model_type=(comp_model_t)m_type;
+    UINT16DECODE(p, c_type);     /* get encoding type */
+    *coder_type=(comp_coder_t)c_type;
+
+    /* read any additional information needed for modeling type */
+    switch (*model_type)
+      {
+          default:      /* no additional information needed */
+              break;
+      }     /* end switch */
+
+    /* read any additional information needed for coding type */
+    switch (*coder_type)
+      {
+          case COMP_CODE_NBIT:      /* N-bit coding needs info */
+              {
+                  uint16      s_ext;    /* temp. var for sign extend */
+                  uint16      f_one;    /* temp. var for fill one */
+                  int32       m_off, m_len;     /* temp. var for mask offset and len */
+
+                  /* specify number-type of N-bit data */
+                  INT32DECODE(p, c_info->nbit.nt);
+                  /* next is the flag to indicate whether to sign extend */
+                  UINT16DECODE(p, s_ext);
+                  c_info->nbit.sign_ext = (intn) s_ext;
+                  /* the flag to indicate whether to fill with 1's or 0's */
+                  UINT16DECODE(p, f_one);
+                  c_info->nbit.fill_one = (intn) f_one;
+                  /* the offset of the bits extracted */
+                  INT32DECODE(p, m_off);
+                  c_info->nbit.start_bit = (intn) m_off;
+                  /* the number of bits extracted */
+                  INT32DECODE(p, m_len);
+                  c_info->nbit.bit_len = (intn) m_len;
+              }     /* end case */
+              break;
+
+          case COMP_CODE_SKPHUFF:   /* Skipping Huffman  coding needs info */
+              {
+                  uint32      skp_size,     /* size of skipping unit */
+                              comp_size;    /* # of bytes to compress */
+
+                  /* specify skipping unit size */
+                  UINT32DECODE(p, skp_size);
+                  /* specify # of bytes of skipping data to compress */
+                  UINT32DECODE(p, comp_size);   /* ignored for now */
+                  c_info->skphuff.skp_size = (intn) skp_size;
+              }     /* end case */
+              break;
+
+          case COMP_CODE_DEFLATE:   /* Deflation coding stores deflation level */
+              {
+                  uint16      level;    /* deflation level */
+
+                  /* specify deflation level */
+                  UINT16DECODE(p, level);
+                  c_info->deflate.level = (intn) level;
+              }     /* end case */
+              break;
+
+          default:      /* no additional information needed */
+              break;
+      }     /* end switch */
+
+done:
+    if(ret_value == FAIL)   
+      { /* Error condition cleanup */
+  
+      } /* end if */
+
+    /* Normal function cleanup */
+    return ret_value; 
+} /* end HCPdecode_header() */
+
 /*--------------------------------------------------------------------------
  NAME
     HCIwrite_header -- Write the compression header info to a file
@@ -256,6 +538,8 @@ HCIinit_model(int16 acc_mode, comp_model_info_t * minfo, comp_model_t model_type
     atom_t file_id;         IN: File ID of the file to write the header to
     compinfo_t *info;       IN: ptr the compression information
     uint16 special_tag,ref; IN: the tag/ref of the compressed element
+    coder_info *c_info;     IN: Information needed for the encoding type chosen
+    model_info *m_info;     IN: Information needed for the modeling type chosen
 
  RETURNS
     Return SUCCEED or FAIL
@@ -268,12 +552,13 @@ HCIinit_model(int16 acc_mode, comp_model_info_t * minfo, comp_model_t model_type
  REVISION LOG
 --------------------------------------------------------------------------*/
 PRIVATE int32
-HCIwrite_header(atom_t file_id, compinfo_t * info, uint16 special_tag, uint16 ref)
+HCIwrite_header(atom_t file_id, compinfo_t * info, uint16 special_tag, uint16 ref, comp_info *c_info, model_info *m_info)
 {
     CONSTR(FUNC, "HCIwrite_header");    /* for HERROR */
     int32       dd_aid;         /* AID for writing the special info */
     uint8      *p;              /* pointer to the temporary buffer */
     uint8       local_ptbuf[32];
+    int32       header_len;     /* how many bytes the header is */
     int32       ret_value=SUCCEED;
 
     /* write special element info to the file */
@@ -282,6 +567,7 @@ HCIwrite_header(atom_t file_id, compinfo_t * info, uint16 special_tag, uint16 re
     UINT16ENCODE(p, COMP_HEADER_VERSION);   /* specify header version */
     INT32ENCODE(p, info->length);   /* write length of un-comp. data */
     UINT16ENCODE(p, (uint16) info->comp_ref);   /* specify ref # of comp. data */
+#ifdef OLD_WAY
     UINT16ENCODE(p, (uint16) info->minfo.model_type);   /* specify model type */
     UINT16ENCODE(p, (uint16) info->cinfo.coder_type);   /* specify coder type */
 
@@ -318,7 +604,15 @@ HCIwrite_header(atom_t file_id, compinfo_t * info, uint16 special_tag, uint16 re
           default:      /* no additional information needed */
               break;
       }     /* end switch */
-
+#else /* OLD_WAY */
+    if((header_len=HCPquery_encode_header(info->minfo.model_type,
+            m_info,info->cinfo.coder_type,c_info))==FAIL)
+        HGOTO_ERROR(DFE_INTERNAL, FAIL);
+    if(HCPencode_header(p,info->minfo.model_type,m_info,
+            info->cinfo.coder_type,c_info)==FAIL)
+        HGOTO_ERROR(DFE_INTERNAL, FAIL);
+    p+=header_len;
+#endif /* OLD_WAY */
     
     /* write the special info structure to fill */
     if((dd_aid=Hstartaccess(file_id,special_tag,ref,DFACC_ALL))==FAIL)
@@ -343,7 +637,6 @@ done:
     HCIread_header -- Read the compression header info from a file
  USAGE
     int32 HCIread_header(file_rec,access_rec,info,comp_info,model_info)
-    filerec_t *file_rec;    IN: ptr to the file record for the HDF file
     accrec_t *access_rec;   IN: ptr to the access element record
     compinfo_t *info;       IN: ptr the compression information
     comp_info *comp_info;   IN/OUT: ptr to encoding info
@@ -359,90 +652,44 @@ done:
  REVISION LOG
 --------------------------------------------------------------------------*/
 PRIVATE int32
-HCIread_header(filerec_t * file_rec, accrec_t * access_rec,
+HCIread_header(accrec_t * access_rec,
                compinfo_t * info, comp_info * c_info, model_info * m_info)
 {
     CONSTR(FUNC, "HCIread_header");     /* for HERROR */
     uint16      header_version; /* version of the compression header */
-    uint16      mtype, ctype;   /* temporary variables for model and coder type */
-    int32       data_off;		/* offset of the data we are checking */
+    uint16      ctag, cref;     /* tag/ref for the object */
+    int32       temp_aid;       /* temporary AID for header info */
+    int32       data_len;		/* offset of the data we are checking */
     uint8      *p;              /* pointer to the temporary buffer */
-    uint8       local_ptbuf[32];
+    uint8      *local_ptbuf;
     int32       ret_value=SUCCEED;
 
     /* shut compiler up */
     m_info = m_info;
 
     /* get the info for the dataset */
-    if(HTPinquire(access_rec->ddid,NULL,NULL,&data_off,NULL)==FAIL)
+    if(HTPinquire(access_rec->ddid,&ctag,&cref,NULL,&data_len)==FAIL)
         HGOTO_ERROR(DFE_INTERNAL, FAIL);
-    if (HPseek(file_rec, data_off + 2) == FAIL)
-        HGOTO_ERROR(DFE_SEEKERROR, FAIL);
-    if (HP_read(file_rec, local_ptbuf, (COMP_HEADER_LENGTH - 2)) == FAIL)
-        HGOTO_ERROR(DFE_READERROR, FAIL);
+    if((local_ptbuf=(uint8 *)HDmalloc(data_len))==NULL)
+        HGOTO_ERROR(DFE_NOSPACE, FAIL);
 
-    p = local_ptbuf;
+    /* Get the compression header */
+    if ((temp_aid=Hstartaccess(access_rec->file_id,MKSPECIALTAG(ctag),cref,DFACC_READ)) == FAIL)
+        HGOTO_ERROR(DFE_BADAID, FAIL);
+    if (Hread(temp_aid,0,local_ptbuf) == FAIL)
+        HGOTO_ERROR(DFE_READERROR, FAIL);
+    if(Hendaccess(temp_aid)==FAIL)
+        HGOTO_ERROR(DFE_CANTENDACCESS, FAIL);
+
+    p = local_ptbuf+2;
     UINT16DECODE(p, header_version);    /* get header length */
     INT32DECODE(p, info->length);   /* get _uncompressed_ data length */
     UINT16DECODE(p, info->comp_ref);    /* get ref # of comp. data */
-    UINT16DECODE(p, mtype);     /* get model type */
-    info->minfo.model_type = (comp_model_t) mtype;
-    UINT16DECODE(p, ctype);     /* get encoding type */
-    info->cinfo.coder_type = (comp_coder_t) ctype;
 
-    /* read any additional information needed for modeling type */
-    switch (info->minfo.model_type)
-      {
-          default:      /* no additional information needed */
-              break;
-      }     /* end switch */
-
-    /* read any additional information needed for coding type */
-    switch (info->cinfo.coder_type)
-      {
-          case COMP_CODE_NBIT:      /* N-bit coding needs info */
-              {
-                  uint16      s_ext;    /* temp. var for sign extend */
-                  uint16      f_one;    /* temp. var for fill one */
-                  int32       m_off, m_len;     /* temp. var for mask offset and len */
-
-                  if (HP_read(file_rec, p, 16) == FAIL)
-                      HGOTO_ERROR(DFE_READERROR, FAIL);
-                  /* specify number-type of N-bit data */
-                  INT32DECODE(p, c_info->nbit.nt);
-                  /* next is the flag to indicate whether to sign extend */
-                  UINT16DECODE(p, s_ext);
-                  c_info->nbit.sign_ext = (intn) s_ext;
-                  /* the flag to indicate whether to fill with 1's or 0's */
-                  UINT16DECODE(p, f_one);
-                  c_info->nbit.fill_one = (intn) f_one;
-                  /* the offset of the bits extracted */
-                  INT32DECODE(p, m_off);
-                  c_info->nbit.start_bit = (intn) m_off;
-                  /* the number of bits extracted */
-                  INT32DECODE(p, m_len);
-                  c_info->nbit.bit_len = (intn) m_len;
-              }     /* end case */
-              break;
-
-          case COMP_CODE_SKPHUFF:   /* Skipping Huffman  coding needs info */
-              {
-                  uint32      skp_size,     /* size of skipping unit */
-                              comp_size;    /* # of bytes to compress */
-
-                  if (HP_read(file_rec, p, 8) == FAIL)
-                      HGOTO_ERROR(DFE_READERROR, FAIL);
-                  /* specify skipping unit size */
-                  UINT32DECODE(p, skp_size);
-                  /* specify # of bytes of skipping data to compress */
-                  UINT32DECODE(p, comp_size);   /* ignored for now */
-                  c_info->skphuff.skp_size = (intn) skp_size;
-              }     /* end case */
-              break;
-
-          default:      /* no additional information needed */
-              break;
-      }     /* end switch */
+    /* Decode the compression header */
+    if(HCPdecode_header(p,&(info->minfo.model_type),m_info,&(info->cinfo.coder_type),c_info)==FAIL)
+        HGOTO_ERROR(DFE_INTERNAL, FAIL);
+    HDfree(local_ptbuf);
 
 done:
     if(ret_value == FAIL)   
@@ -489,7 +736,7 @@ HCcreate(int32 file_id, uint16 tag, uint16 ref, comp_model_t model_type,
     atom_t      data_id=FAIL;   /* dd ID of existing regular element */
     int32       data_len;		/* length of the data we are checking */
     uint16      special_tag;    /* special version of tag */
-    VOIDP       buf = NULL;      /* temporary buffer */
+    void *       buf = NULL;      /* temporary buffer */
     int32       ret_value=SUCCEED;
 
     /* clear error stack and validate args */
@@ -525,11 +772,20 @@ HCcreate(int32 file_id, uint16 tag, uint16 ref, comp_model_t model_type,
                     HGOTO_ERROR(DFE_CANTFLUSH, FAIL);
                 HGOTO_ERROR(DFE_INTERNAL, FAIL);
             } /* end if */
+
+          if ((buf = HDmalloc((uint32) data_len)) == NULL)
+              HGOTO_ERROR(DFE_NOSPACE, FAIL);
+          if (Hgetelement(file_id, tag, ref, buf) == FAIL)
+              HGOTO_ERROR(DFE_READERROR, FAIL);
+          /* Delete the old DD from the file and memory hash table */
+          if (FAIL == HTPdelete(data_id))
+              HGOTO_ERROR(DFE_CANTDELDD, FAIL);
+
       } /* end if */
 
     /* set up the special element information and write it to file */
     info = (compinfo_t *) HDmalloc(sizeof(compinfo_t));
-    access_rec->special_info = (VOIDP) info;
+    access_rec->special_info = info;
     if (info == NULL)
           HGOTO_ERROR(DFE_NOSPACE, FAIL);
 
@@ -538,10 +794,12 @@ HCcreate(int32 file_id, uint16 tag, uint16 ref, comp_model_t model_type,
     /* set up compressed special info structure */
     info->attached = 1;
     info->comp_ref = Htagnewref(file_id,DFTAG_COMPRESSED);  /* get the new reference # */
-    HCIinit_model(DFACC_RDWR, &(info->minfo), model_type, m_info);
-    HCIinit_coder(DFACC_RDWR, &(info->cinfo), coder_type, c_info);
+    if(HCIinit_model(DFACC_RDWR, &(info->minfo), model_type, m_info)==FAIL)
+        HGOTO_ERROR(DFE_MINIT,FAIL);
+    if(HCIinit_coder(DFACC_RDWR, &(info->cinfo), coder_type, c_info)==FAIL)
+        HGOTO_ERROR(DFE_CINIT,FAIL);
 
-    if (HCIwrite_header(file_id, info, special_tag, ref) == FAIL)
+    if (HCIwrite_header(file_id, info, special_tag, ref, c_info, m_info) == FAIL)
           HGOTO_ERROR(DFE_WRITEERROR, FAIL);
 
     /* update access record and file record */
@@ -563,11 +821,6 @@ HCcreate(int32 file_id, uint16 tag, uint16 ref, comp_model_t model_type,
     if (data_id != FAIL)
       {
 
-          if ((buf = (VOIDP) HDmalloc((uint32) data_len)) == NULL)
-              HGOTO_ERROR(DFE_NOSPACE, FAIL);
-          if (Hgetelement(file_id, tag, ref, buf) == FAIL)
-              HGOTO_ERROR(DFE_READERROR, FAIL);
-
           /* write the data through to the compression layer */
           if (HCPwrite(access_rec, data_len, buf) == FAIL)
               HGOTO_ERROR(DFE_MODEL, FAIL);
@@ -575,10 +828,6 @@ HCcreate(int32 file_id, uint16 tag, uint16 ref, comp_model_t model_type,
           /* seek back to the beginning of the data through to the compression layer */
           if (HCPseek(access_rec, 0, DF_START) == FAIL)
               HGOTO_ERROR(DFE_MODEL, FAIL);
-
-          /* Delete the old DD from the file and memory hash table */
-          if (FAIL == HTPdelete(data_id))
-              HGOTO_ERROR(DFE_CANTDELDD, FAIL);
       }     /* end if */
 
     ret_value=HAregister_atom(AIDGROUP,access_rec);
@@ -587,14 +836,14 @@ done:
     if(ret_value == FAIL)   
       { /* Error condition cleanup */
         if(access_rec!=NULL)
-            HDfree(access_rec);
+            HIrelease_accrec_node(access_rec);
         if(info!=NULL)
-            HDfree((VOIDP) info);
+            HDfree(info);
       } /* end if */
 
     /* Normal function cleanup */
     if (buf != NULL)
-        HDfree((VOIDP) buf);
+        HDfree(buf);
 
     return ret_value; 
 }   /* end HCcreate() */
@@ -634,15 +883,15 @@ HCIstaccess(accrec_t * access_rec, int16 acc_mode)
     /* intialize the access record */
     access_rec->special = SPECIAL_COMP;
     access_rec->posn = 0;
-    access_rec->access = acc_mode|DFACC_READ;
+    access_rec->access = (uint32)(acc_mode|DFACC_READ);
 
     /* get the special info record */
-    access_rec->special_info = (VOIDP) HDmalloc(sizeof(compinfo_t));
+    access_rec->special_info = HDmalloc(sizeof(compinfo_t));
     info = (compinfo_t *) access_rec->special_info;
     if (info == NULL)
         HGOTO_ERROR(DFE_NOSPACE, FAIL);
 
-    if (HCIread_header(file_rec, access_rec, info, &c_info, &m_info) == FAIL)
+    if (HCIread_header(access_rec, info, &c_info, &m_info) == FAIL)
           HGOTO_ERROR(DFE_COMPINFO, FAIL);
     info->attached = 1;
     if (HCIinit_model(acc_mode,&(info->minfo), info->minfo.model_type, &m_info) == FAIL)
@@ -658,7 +907,7 @@ done:
     if(ret_value == FAIL)   
       { /* Error condition cleanup */
         if(info!=NULL)
-            HDfree((VOIDP) info);
+            HDfree(info);
       } /* end if */
 
     /* Normal function cleanup */
@@ -800,7 +1049,7 @@ done:
     int32 HCPread(access_rec,length,data)
     accrec_t *access_rec;   IN: the access record of the data element
     int32 length;           IN: the number of bytes to read
-    VOIDP data;             OUT: the buffer to place the bytes read
+    void * data;             OUT: the buffer to place the bytes read
  RETURNS
     Returns the number of bytes read or FAIL
  DESCRIPTION
@@ -812,7 +1061,7 @@ done:
  REVISION LOG
 --------------------------------------------------------------------------*/
 int32
-HCPread(accrec_t * access_rec, int32 length, VOIDP data)
+HCPread(accrec_t * access_rec, int32 length, void * data)
 {
     CONSTR(FUNC, "HCPread");    /* for HERROR */
     compinfo_t *info;           /* information on the special element */
@@ -855,7 +1104,7 @@ done:
     int32 HCPwrite(access_rec,length,data)
     accrec_t *access_rec;   IN: the access record of the data element
     int32 length;           IN: the number of bytes to write
-    VOIDP data;             IN: the buffer to retrieve the bytes written
+    void * data;             IN: the buffer to retrieve the bytes written
  RETURNS
     Returns the number of bytes written or FAIL
  DESCRIPTION
@@ -867,7 +1116,7 @@ done:
  REVISION LOG
 --------------------------------------------------------------------------*/
 int32
-HCPwrite(accrec_t * access_rec, int32 length, const VOIDP data)
+HCPwrite(accrec_t * access_rec, int32 length, const void * data)
 {
     CONSTR(FUNC, "HCPwrite");   /* for HERROR */
     compinfo_t *info;           /* information on the special element */
@@ -973,7 +1222,7 @@ HCPinquire(accrec_t * access_rec, int32 *pfile_id, uint16 *ptag,
     if (paccess != NULL)
         *paccess = (int16)access_rec->access;
     if (pspecial != NULL)
-        *pspecial = access_rec->special;
+        *pspecial = (int16)access_rec->special;
 
     return (SUCCEED);
 }   /* end HCPinquire() */
@@ -1016,7 +1265,7 @@ HCPendaccess(accrec_t * access_rec)
     file_rec->attach--;
 
     /* free the access record */
-    HDfree(access_rec);
+    HIrelease_accrec_node(access_rec);
 
 done:
   if(ret_value == FAIL)   
@@ -1050,7 +1299,7 @@ HCPcloseAID(accrec_t * access_rec)
 {
     CONSTR(FUNC, "HCPcloseAID");    /* for HERROR */
     compinfo_t *info;           /* special information record */
-    int32       ret;
+    int32       ret=SUCCEED;
 
     info = (compinfo_t *) access_rec->special_info;
     if ((ret = (*(info->minfo.model_funcs.endaccess)) (access_rec)) == FAIL)
@@ -1060,7 +1309,7 @@ HCPcloseAID(accrec_t * access_rec)
     if (--(info->attached) == 0)
         HDfree(info);
 
-    return (SUCCEED);
+    return (ret);
 }   /* end HCPcloseAID() */
 
 /* ------------------------------- HCPinfo -------------------------------- */
@@ -1092,8 +1341,8 @@ HCPinfo(accrec_t * access_rec, sp_info_block_t * info_block)
     /* fill in the info_block */
     info_block->key = SPECIAL_COMP;
 
-    info_block->comp_type = info->cinfo.coder_type;
-    info_block->model_type = info->minfo.model_type;
+    info_block->comp_type = (int32)info->cinfo.coder_type;
+    info_block->model_type = (int32)info->minfo.model_type;
     info_block->comp_size = Hlength(access_rec->file_id, DFTAG_COMPRESSED, info->comp_ref);
 
     return SUCCEED;
