@@ -10,7 +10,7 @@
  *                                                                          *
  ****************************************************************************/
 
-/* hdfi.h,v 1.111 1996/07/03 20:56:00 acheng Exp */
+/* $Id: hdfi.h,v 1.125 1998/02/23 16:29:59 acheng Exp $ */
 
 #ifndef HDFI_H
 #define HDFI_H
@@ -21,15 +21,31 @@
 
 /*--------------------------------------------------------------------------*/
 /*                              MT/NT constants                             */
-/*  four MT nibbles represent double, float, int, uchar (from most          */
-/*  significant to least significant).                                      */
+/*  Four MT nibbles represent double, float, int, uchar (from most          */
+/*      significant to least significant).                                  */
+/*  Each "column" in the "table" below is essentially independant of the    */
+/*      other "columns", for example the CONVEXNATIVE entry means that the  */
+/*      floating point formats are in Convex native format but the integers */
+/*      are big-endian and standard sizes                                   */
+/*  If you add another value to this "table", you need to add another       */
+/*      DFNTF_xxx entry in hntdefs.h                                        */
 /*  The values for each nibble are:                                         */
 /*      1 - Big Endian                                                      */
+/*          (i.e. Big-Endian, 32-bit architecture w/IEEE Floats)            */
 /*      2 - VAX                                                             */
+/*          (i.e. Middle-Endian, 32-bit architecture w/VAX Floats)          */
 /*      3 - Cray                                                            */
+/*          (i.e. Big-Endian, all 64-bit architecture w/Cray Floats)        */
 /*      4 - Little Endian                                                   */
+/*          (i.e. Little-Endian, 32-bit architecture w/IEEE Floats)         */
 /*      5 - Convex                                                          */
+/*          (i.e. Big-Endian, 32-bit architecture w/Convex Native Floats)   */
 /*      6 - Fujitsu VP                                                      */
+/*          (i.e. Big-Endian, 32-bit architecture w/Fujitsu Native Floats)  */
+/*      7 - Cray MPP                                                        */
+/*          (i.e. Big-Endian, 32-bit architecture w/IEEE Floats, but no 16-bit type)            */
+/*      8 - Cray IEEE                                                       */
+/*          (i.e. Big-Endian, all 64-bit architecture w/IEEE Floats)        */
 /*--------------------------------------------------------------------------*/
 #define     DFMT_SUN            0x1111
 #define     DFMT_ALLIANT        0x1111
@@ -40,6 +56,7 @@
 #define     DFMT_CONVEXNATIVE   0x5511
 #define     DFMT_CONVEX         0x1111
 #define     DFMT_UNICOS         0x3331
+#define     DFMT_UNICOSIEEE     0x1831
 #define     DFMT_CTSS           0x3331
 #define     DFMT_VAX            0x2221
 #define     DFMT_MIPSEL         0x4441
@@ -57,10 +74,10 @@
 #define UNIXUNBUFIO 1
 #define UNIXBUFIO   2
 #define MACIO       3
-#define PCIO        4    /* 16-bit MS-DOS File I/O */
-#define WINIO       5    /* 16-bit Windows File I/O */
+#define PCIO        4    /* 16-bit MS-DOS File I/O (deprecated) */
+#define WINIO       5    /* 16-bit Windows File I/O (deprecated) */
 #define PAGEBUFIO   6    /* page buffering - fmpool */
-#define WINNTIO     7    /* 32-bit Windows File I/O */
+#define WINNTIO     7    /* 32-bit Windows File I/O (deprecated, WinNT now uses UNIXBUFIO) */ 
 
 /* IBM RS6000 AIX hack */
 #if defined(IBM6000) || defined(_AIX)
@@ -83,7 +100,15 @@
  * Define options for each platform
  *-------------------------------------------------------------------------*/
 
-#if (defined(SUN) || defined(sun) || defined(__sun__)) & !defined(__i386)
+/*
+ * Meaning of each defined macros (not completed yet)
+ *
+ * BIG_LONGS--Define when long is not "equal" to int32.  True in cases
+ *      where (int32 *) is not compatible with (long *).  Should
+ *      be renamed as LONGNEINT32.
+ */
+
+#if (defined(SUN) || defined(sun) || defined(__sun__) || defined(__SUNPRO_C)) & !defined(__i386)
 #ifdef __STDC__
 #define ANSISUN
 #else /* __STDC__ */
@@ -103,9 +128,6 @@ Please check your Makefile.
 #endif
 #define GOT_MACHINE
 
-#ifdef QAK
-#include <ctype.h>
-#endif /* QAK */
 #include <unistd.h>                 /* for some file I/O stuff */
 #include <sys/time.h>
 #include <sys/file.h>               /* for unbuffered i/o stuff */
@@ -127,6 +149,7 @@ typedef unsigned int      uintn;
 typedef int               intf;     /* size of INTEGERs in Fortran compiler */
 typedef float             float32;
 typedef double            float64;
+typedef int               hdf_pint_t;   /* an integer the same size as a pointer */
 #define FNAME_POST_UNDERSCORE
 #define _fcdtocp(desc) (desc)
 #ifdef  HAVE_FMPOOL
@@ -183,6 +206,7 @@ typedef unsigned int      uintn;
 typedef int               intf;     /* size of INTEGERs in Fortran compiler */
 typedef float             float32;
 typedef double            float64;
+typedef int               hdf_pint_t;   /* an integer the same size as a pointer */
 #define FNAME_POST_UNDERSCORE
 #define _fcdtocp(desc) (desc)
 #ifdef  HAVE_FMPOOL
@@ -242,6 +266,7 @@ typedef unsigned int      uintn;
 typedef float             float32;
 typedef double            float64;
 typedef int               intf;     /* size of INTEGERs in Fortran compiler */
+typedef int               hdf_pint_t;   /* an integer the same size as a pointer */
 #define _fcdtocp(desc) (desc)
 #ifdef  HAVE_FMPOOL
 #define FILELIB PAGEBUFIO  /* enable page buffering */
@@ -261,7 +286,7 @@ typedef int               intf;     /* size of INTEGERs in Fortran compiler */
 
 #endif /* IBM6000 */
 
-#if defined(HP9000) || defined(hpux) || defined(__hpux)
+#if defined(HP9000) || (!defined(__convexc__) && (defined(hpux) || defined(__hpux)))
 
 #ifndef HP9000
 #define HP9000
@@ -298,6 +323,7 @@ typedef unsigned int      uintn;
 typedef float             float32;
 typedef double            float64;
 typedef int               intf;     /* size of INTEGERs in Fortran compiler */
+typedef int               hdf_pint_t;   /* an integer the same size as a pointer */
 #define _fcdtocp(desc) (desc)
 #ifdef  HAVE_FMPOOL
 #define FILELIB PAGEBUFIO  /* enable page buffering */
@@ -321,6 +347,14 @@ typedef int               intf;     /* size of INTEGERs in Fortran compiler */
 #define IRIX
 #endif
 
+#if (_MIPS_SZLONG == 64)
+/* IRIX 64 bits objects.  It is nearly the same as the conventional
+ * 32 bits objects.  Let them share IRIX definitions for now.
+ */
+#define IRIX64
+#endif
+
+
 #ifdef GOT_MACHINE
 If you get an error on this line more than one machine type has been defined.
 Please check your Makefile.
@@ -350,6 +384,7 @@ typedef unsigned int       uintn;
 typedef float              float32;
 typedef double             float64;
 typedef int                intf;     /* size of INTEGERs in Fortran compiler */
+typedef long               hdf_pint_t;   /* an integer the same size as a pointer */
 #define FNAME_POST_UNDERSCORE
 #define _fcdtocp(desc) (desc)
 #ifdef  HAVE_FMPOOL
@@ -357,6 +392,10 @@ typedef int                intf;     /* size of INTEGERs in Fortran compiler */
 #else
 #define FILELIB UNIXBUFIO
 #endif
+#ifdef IRIX64
+#define BIG_LONGS
+#endif
+
 
 /* JPEG #define's - Look in the JPEG docs before changing - (Q) */
 
@@ -390,7 +429,11 @@ Please check your Makefile.
 #include <sys/stat.h>
 #endif /*O_RDONLY*/
 
+#ifdef _CRAYIEEE
+#define DF_MT   DFMT_UNICOSIEEE
+#else
 #define DF_MT   DFMT_UNICOS
+#endif
 typedef void            VOID;
 typedef void            *VOIDP;
 #ifdef OLD_WAY /* May need to be included on other machines than the C-90 */
@@ -409,6 +452,7 @@ typedef unsigned int    uintn;
 typedef float           float32;
 typedef double          float64;
 typedef int             intf;     /* size of INTEGERs in Fortran compiler */
+typedef int               hdf_pint_t;   /* an integer the same size as a pointer */
 
 #define DF_CAPFNAMES            /* fortran names are in all caps */
 #ifdef  HAVE_FMPOOL
@@ -469,6 +513,7 @@ typedef unsigned int    uintn;
 typedef float           float32;
 typedef double          float64;
 typedef int             intf;     /* size of INTEGERs in Fortran compiler */
+typedef int               hdf_pint_t;   /* an integer the same size as a pointer */
 
 #define _HUGE              /* This should only be defined to a value on the PC */
 #define DF_CAPFNAMES            /* fortran names are in all caps */
@@ -511,9 +556,11 @@ typedef unsigned short int uint16;
 #ifdef __alpha
 typedef int                int32;
 typedef unsigned int       uint32;
+typedef long              hdf_pint_t;   /* an integer the same size as a pointer */
 #else
 typedef long int           int32;
 typedef unsigned long int  uint32;
+typedef int               hdf_pint_t;   /* an integer the same size as a pointer */
 #endif
 typedef int                intn;
 typedef unsigned int       uintn;
@@ -540,7 +587,7 @@ typedef int                intf;     /* size of INTEGERs in Fortran compiler */
 
 #endif /* VMS */
 
-#if defined(CONVEX) || defined(CONVEXNATIVE) || defined(__convex__)
+#if defined(CONVEX) || defined(CONVEXNATIVE) || defined(__convexc__)
 
 #ifndef CONVEX
 #define CONVEX
@@ -576,7 +623,7 @@ typedef unsigned int      uintn;
 typedef float             float32;
 typedef double            float64;
 typedef int               intf;     /* size of INTEGERs in Fortran compiler */
-#define FNAME_POST_UNDERSCORE
+typedef int               hdf_pint_t;   /* an integer the same size as a pointer */
 #define _fcdtocp(desc) (desc)
 #ifdef  HAVE_FMPOOL
 #define FILELIB PAGEBUFIO  /* enable page buffering */
@@ -633,6 +680,7 @@ typedef unsigned int    uintn;
 typedef float           float32;
 typedef double          float64;
 typedef int             intf;     /* size of INTEGERs in Fortran compiler */
+typedef int               hdf_pint_t;   /* an integer the same size as a pointer */
 #define _fcdtocp(desc) (desc)
 #define FNAME_POST_UNDERSCORE
 #ifdef  HAVE_FMPOOL
@@ -650,7 +698,7 @@ typedef int             intf;     /* size of INTEGERs in Fortran compiler */
 
 #endif /* MIPSEL */
 
-#if defined(MAC) || defined(macintosh) || defined(__MWERKS__) || defined (SYMANTEC_C)
+#if defined(MAC) || defined(macintosh) || defined (SYMANTEC_C)
 
 #ifdef GOT_MACHINE
 If you get an error on this line more than one machine type has been defined.
@@ -660,11 +708,11 @@ Please check your Makefile.
 
 #include <memory.h>             /* malloc stuff for MPW */
 #include <fcntl.h>              /* unbuffered I/O stuff for MPW */
-#ifdef __MWERKS__  /* Metrowerks */
+#ifdef __MWERKS__				/* Metrowerks */
 #include <sioux.h>
 #include <console.h>
 #endif
-#ifdef SYMANTEC_C                  /* for SYMANTEC C */
+#ifdef SYMANTEC_C				/* for SYMANTEC C */
 #include <unix.h>
 #define isascii(c)  (isprint(c) || iscntrl(c))
 #else  /* MPW, possibly others */
@@ -692,6 +740,7 @@ typedef unsigned int      uintn;
 typedef float             float32;
 typedef double            float64;
 typedef int               intf;     /* size of INTEGERs in Fortran compiler */
+typedef int               hdf_pint_t;   /* an integer the same size as a pointer */
 #define _fcdtocp(desc) (desc)
 void exit(int status);
 
@@ -706,8 +755,8 @@ void exit(int status);
 
 #endif /*MAC*/
 
-/* Metrowerks compilier defines some PC stuff so need to exclude this on the MAC */
-#if !(defined(__MWERKS__) || defined(MAC))
+/* Metrowerks Mac compiler defines some PC stuff so need to exclude this on the Mac */
+#if !(defined(macintosh) || defined(MAC))
 
 #if defined INTEL86 || defined M_I86 || defined M_I386 || defined DOS386 || defined __i386 || defined UNIX386
 #ifndef INTEL86
@@ -780,6 +829,7 @@ typedef unsigned int      uintn;
 typedef float             float32;
 typedef double            float64;
 typedef long              intf;     /* size of INTEGERs in Fortran compiler */
+typedef int               hdf_pint_t;   /* an integer the same size as a pointer */
 
 #if defined UNIX386
 #define FNAME_POST_UNDERSCORE
@@ -788,15 +838,11 @@ typedef long              intf;     /* size of INTEGERs in Fortran compiler */
 #endif
 #define _fcdtocp(desc) (desc)
 
-#if defined WIN386
-#define FILELIB WINNTIO
-#else
 #ifdef  HAVE_FMPOOL
 #define FILELIB PAGEBUFIO  /* enable page buffering */
 #else
 #define FILELIB UNIXBUFIO
 #endif
-#endif /* WIN386 */
 
 /* JPEG #define's - Look in the JPEG docs before changing - (Q) */
 
@@ -808,7 +854,7 @@ typedef long              intf;     /* size of INTEGERs in Fortran compiler */
 #define INCLUDES_ARE_ANSI
 
 #endif /* INTEL86 */
-#endif /* !(defined(__MWERKS__) || defined(MAC)) */
+#endif /* !(defined(macintosh) || defined(MAC)) */
 
 #if defined(NEXT) || defined(NeXT)
 
@@ -845,6 +891,7 @@ typedef unsigned int      uintn;
 typedef int               intf;     /* size of INTEGERs in Fortran compiler */
 typedef float             float32;
 typedef double            float64;
+typedef int               hdf_pint_t;   /* an integer the same size as a pointer */
 #define FNAME_POST_UNDERSCORE
 #define _fcdtocp(desc) (desc)
 #ifdef  HAVE_FMPOOL
@@ -899,6 +946,7 @@ typedef unsigned int      uintn;
 typedef int               intf;     /* size of INTEGERs in Fortran compiler */
 typedef float             float32;
 typedef double            float64;
+typedef int               hdf_pint_t;   /* an integer the same size as a pointer */
 #define FNAME_POST_UNDERSCORE
 #define _fcdtocp(desc) (desc)
 #define FILELIB UNIXBUFIO
@@ -945,6 +993,7 @@ typedef unsigned int      uintn;
 typedef int               intf;     /* size of INTEGERs in Fortran compiler */
 typedef float             float32;
 typedef double            float64;
+typedef long              hdf_pint_t;   /* an integer the same size as a pointer */
 #define FNAME_POST_UNDERSCORE
 #define _fcdtocp(desc) (desc)
 #ifdef  HAVE_FMPOOL
@@ -999,6 +1048,7 @@ typedef unsigned int       uintn;
 typedef int                intf;     /* size of INTEGERs in Fortran compiler */
 typedef float              float32;
 typedef double             float64;
+typedef int               hdf_pint_t;   /* an integer the same size as a pointer */
 #define FNAME_POST_UNDERSCORE
 #define _fcdtocp(desc) (desc)
 #define FILELIB UNIXBUFIO
@@ -1045,6 +1095,7 @@ typedef unsigned int    uintn;
 typedef float           float32;
 typedef double          float64;
 typedef int             intf;     /* size of INTEGERs in Fortran compiler */
+typedef int               hdf_pint_t;   /* an integer the same size as a pointer */
 #define _fcdtocp(desc) (desc)
 #define FNAME_POST_UNDERSCORE
 #define FILELIB UNIXBUFIO
@@ -1069,11 +1120,11 @@ correctly.
 /*-----------------------------------------------------*/
 
 #   define INT16ENCODE(p, i) \
-{ *(p) = (uint8)(((uint16)(i) >> 8) & 0xff); (p)++; \
-        *(p) = (uint8)((uint16)(i) & 0xff); (p)++; }
+{ *(p) = (uint8)(((uintn)(i) >> 8) & 0xff); (p)++; \
+        *(p) = (uint8)((uintn)(i) & 0xff); (p)++; }
 
 #   define UINT16ENCODE(p, i) \
-{ *(p) = (uint8)(((uint16)(i) >> 8) & 0xff); (p)++; *(p) = (uint8)((i) & 0xff); (p)++; }
+{ *(p) = (uint8)(((uintn)(i) >> 8) & 0xff); (p)++; *(p) = (uint8)((i) & 0xff); (p)++; }
 
 #   define INT32ENCODE(p, i) \
 { *(p) = (uint8)(((uint32)(i) >> 24) & 0xff); (p)++; \
@@ -1090,8 +1141,16 @@ correctly.
 #   define NBYTEENCODE(d, s, n) \
 {   HDmemcpy(d,s,n); p+=n }
 
+/* DECODE converts big endian bytes pointed by p to integer values and store
+ * it in i.  For signed values, need to do sign-extension when converting
+ * the 1st byte which carries the sign bit.
+ * The macros does not require i be of a certain byte sizes.  It just requires
+ * i be big enough to hold the intended value range.  E.g. INT16DECODE works
+ * correctly even if i is actually a 64bit int like in a Cray.
+ */
+
 #   define INT16DECODE(p, i) \
-{ (i) = (int16)((*(p) & 0xff) << 8); (p)++; \
+{ (i) = ((*(p) & 0x80) ? ~0xffff : 0x00) | ((int16)(*(p) & 0xff) << 8); (p)++; \
         (i) |= (int16)((*(p) & 0xff)); (p)++; }
 
 #   define UINT16DECODE(p, i) \
@@ -1099,7 +1158,7 @@ correctly.
         (i) |= (uint16)(*(p) & 0xff); (p)++; }
 
 #   define INT32DECODE(p, i) \
-{ (i) = ((int32)(*(p) & 0xff) << 24); (p)++; \
+{ (i) = ((*(p) & 0x80) ? ~0xffffffff : 0x00) | ((int32)(*(p) & 0xff) << 24); (p)++; \
         (i) |= ((int32)(*(p) & 0xff) << 16); (p)++; \
         (i) |= ((int32)(*(p) & 0xff) << 8); (p)++; \
         (i) |= (*(p) & 0xff); (p)++; }
@@ -1108,7 +1167,7 @@ correctly.
 { (i) = ((uint32)(*(p) & 0xff) << 24); (p)++; \
         (i) |= ((uint32)(*(p) & 0xff) << 16); (p)++; \
         (i) |= ((uint32)(*(p) & 0xff) << 8); (p)++; \
-        (i) |= (*(p) & 0xff); (p)++; }
+        (i) |= (uint32)(*(p) & 0xff); (p)++; }
 
 /* Note! the NBYTEDECODE macro is backwards from the memcpy() routine, */
 /*      in the spirit of the other DECODE macros */
@@ -1119,9 +1178,9 @@ correctly.
 *                   Conversion Routine Pointers
 ***************************************************************************/
 #    ifndef DFKMASTER
-extern int (*DFKnumin)(void * source, void * dest, uint32 num_elm,
+extern int (*DFKnumin)(const void * source, void * dest, uint32 num_elm,
             uint32 source_stride,uint32 dest_stride);
-extern int (*DFKnumout)(void * source, void * dest, uint32 num_elm,
+extern int (*DFKnumout)(const void * source, void * dest, uint32 num_elm,
             uint32 source_stride,uint32 dest_stride);
 #     endif /* DFKMASTER */
 
@@ -1231,8 +1290,8 @@ extern int (*DFKnumout)(void * source, void * dest, uint32 num_elm,
 #  define HDstrchr(s,c)         (strchr((s),(c)))
 #  define HDstrrchr(s,c)        (strrchr((s),(c)))
 #  define HDstrtol(s,e,b)       (strtol((s),(e),(b)))
-/* non-standard function, not defined on the following mahcines - */
-#if !(defined VMS || defined macintosh || defined MAC || defined __MWERKS__ || defined SYMANTEC_C || defined MIPSEL || defined NEXT || defined CONVEX || defined IBM6000)
+/* non-standard function, not defined on the following machines - */
+#if !(defined VMS || defined macintosh || defined MAC || defined SYMANTEC_C || defined MIPSEL || defined NEXT || defined CONVEX || defined IBM6000 || defined ANSISUN || defined IRIX)
 #  define HDstrdup(s)      ((char *)strdup((const char *)(s)))
 #endif /* !(VMS | etc..) */
 
@@ -1249,7 +1308,7 @@ extern int (*DFKnumout)(void * source, void * dest, uint32 num_elm,
 /**************************************************************************
 *  Misc. functions
 **************************************************************************/
-#if defined (MAC) || defined (macintosh) || defined(__MWERKS__) || defined (SYMANTEC_C)
+#if defined (MAC) || defined (macintosh) || defined (SYMANTEC_C)
 #define HDstat(path, result)	(mstat(path))
 #else /* !macintosh */
 #define HDstat(path, result)	(stat(path, result))
