@@ -11,10 +11,10 @@
  ****************************************************************************/
 
 #ifdef RCSID
-static char RcsId[] = "@(#)1.74";
+static char RcsId[] = "@(#)$Revision: 1.79 $";
 #endif
 
-/* hextelt.c,v 1.74 1996/06/12 21:13:10 sxu Exp */
+/* $Id: hextelt.c,v 1.79 1998/02/02 21:38:48 smitchel Exp $ */
 
 /*LINTLIBRARY */
 /* ------------------------------ hextelt.c -------------------------------
@@ -86,7 +86,7 @@ EXPORTED ROUTINES
 /* Directory seperator definitions relating to a path. 
  * Note this does not provide a universal way to recognize
  * different path name conventions and translate between them */
-#if defined (MAC) || defined (macintosh) || defined(__MWERKS__) || defined (SYMANTEC_C) 
+#if defined (MAC) || defined (macintosh) || defined (SYMANTEC_C) 
 #define DIR_SEPC  58  /* Integer value of ':' */
 #define DIR_SEPS  ":"
 #else /* not Macintosh */
@@ -206,7 +206,7 @@ HXcreate(int32 file_id, uint16 tag, uint16 ref, const char *extern_file_name, in
     uint16      special_tag;    /* special version of tag */
     uint8       local_ptbuf[20 + MAX_PATH_LEN];     /* temp working buffer */
     char	   *fname=NULL;    /* filename built from external filename */
-    VOIDP       buf = NULL;      /* temporary buffer */
+    void *       buf = NULL;      /* temporary buffer */
     int32       ret_value = SUCCEED;
 
 #ifdef HAVE_PABLO
@@ -280,20 +280,20 @@ HXcreate(int32 file_id, uint16 tag, uint16 ref, const char *extern_file_name, in
     HDfree(fname);
 
     /* set up the special element information and write it to file */
-    access_rec->special_info = (VOIDP) HDmalloc((uint32) sizeof(extinfo_t));
+    access_rec->special_info = HDmalloc((uint32) sizeof(extinfo_t));
     info = (extinfo_t *) access_rec->special_info;
     if (!info)
         HGOTO_ERROR(DFE_NOSPACE, FAIL);
 
     if (data_id!=FAIL && data_len>0)
       {
-          if ((buf = (VOIDP) HDmalloc((uint32) data_len)) == NULL)
+          if ((buf = HDmalloc((uint32) data_len)) == NULL)
               HGOTO_ERROR(DFE_NOSPACE, FAIL);
-          if (Hgetelement(file_id, tag, ref, (VOIDP)buf) == FAIL)
+          if (Hgetelement(file_id, tag, ref, buf) == FAIL)
                 HGOTO_ERROR(DFE_READERROR, FAIL);
           if (HI_SEEK(file_external, offset) == FAIL)
                 HGOTO_ERROR(DFE_SEEKERROR, FAIL);
-          if (HI_WRITE(file_external, buf, data_len) == FAIL)
+          if (HI_WRITE(file_external, buf, (int)data_len) == FAIL)
                 HGOTO_ERROR(DFE_WRITEERROR, FAIL);
           info->length = data_len;
       }
@@ -309,7 +309,7 @@ HXcreate(int32 file_id, uint16 tag, uint16 ref, const char *extern_file_name, in
       HGOTO_ERROR(DFE_NOSPACE, FAIL);
 
     /* Getting ready to write out special info struct */
-    info->length_file_name = HDstrlen(extern_file_name);
+    info->length_file_name = (int32)HDstrlen(extern_file_name);
     {
         uint8      *p = local_ptbuf;
     
@@ -348,18 +348,18 @@ done:
   if(ret_value == FAIL)   
     { /* Error condition cleanup */
         if(access_rec!=NULL)
-            HDfree(access_rec);
+            HIrelease_accrec_node(access_rec);
         if(info!=NULL)
-            HDfree((VOIDP) info);
+            HDfree(info);
         if(fname!=NULL)
-            HDfree((VOIDP) fname);
+            HDfree(fname);
         if(data_id!=FAIL)
             HTPendaccess(data_id);
     } /* end if */
 
   /* Normal function cleanup */
   if (buf != NULL)
-      HDfree((VOIDP) buf);
+      HDfree(buf);
 
 #ifdef HAVE_PABLO
     TRACE_OFF(H_mask, ID_HXcreate);
@@ -384,9 +384,6 @@ HXPsetaccesstype(accrec_t * access_rec)
 {
     CONSTR(FUNC, "HXPsetaccesstype");
     hdf_file_t  file_external;  /* external file descriptor */
-#ifdef CM5
-    int32       para_extfile_id;    /* parallel external file id */
-#endif /* CM5 */
     extinfo_t  *info;           /* special element information */
     char	*fname=NULL;
     intn       ret_value = SUCCEED;
@@ -420,20 +417,6 @@ HXPsetaccesstype(accrec_t * access_rec)
               info->file_external = file_external;
               break;
               
-#ifdef CM5
-          case DFACC_PARALLEL:
-              para_extfile_id = CM_OPEN(fname, DFACC_WRITE);
-              if (para_extfile_id == FAIL)
-                {
-                    para_extfile_id = CM_CREATE(fname);
-                    if (para_extfile_id == FAIL)
-                        HGOTO_ERROR(DFE_BADOPEN, FAIL);
-                }
-              HDfree(fname);
-              info->para_extfile_id = para_extfile_id;
-              break;
-
-#endif /* CM5 */
           default:
               HGOTO_ERROR(DFE_BADOPEN, FAIL);
       }
@@ -487,7 +470,7 @@ HXIstaccess(accrec_t * access_rec, int16 acc_mode)
     /* intialize the access record */
     access_rec->special = SPECIAL_EXT;
     access_rec->posn = 0;
-    access_rec->access = acc_mode|DFACC_READ;
+    access_rec->access = (uint32)(acc_mode|DFACC_READ);
 
     /* Get the data's offset & length */
     if(HTPinquire(access_rec->ddid,NULL,NULL,&data_off,NULL)==FAIL)
@@ -507,7 +490,7 @@ HXIstaccess(accrec_t * access_rec, int16 acc_mode)
           if (HP_read(file_rec, local_ptbuf, 12) == FAIL)
               HGOTO_ERROR(DFE_READERROR, FAIL);
 
-          access_rec->special_info = (VOIDP) HDmalloc((uint32) sizeof(extinfo_t));
+          access_rec->special_info = HDmalloc((uint32) sizeof(extinfo_t));
           info = (extinfo_t *) access_rec->special_info;
           if (info==NULL)
             HGOTO_ERROR(DFE_NOSPACE, FAIL);
@@ -538,7 +521,7 @@ done:
   if(ret_value == FAIL)   
     { /* Error condition cleanup */
         if(access_rec!=NULL)
-            HDfree(access_rec);
+            HIrelease_accrec_node(access_rec);
         if(info !=NULL)
           {   /* free file name first */
               if (info->extern_file_name != NULL)
@@ -651,7 +634,7 @@ USAGE
    int32 HXPseek(access_rec, length, data)
    access_t * access_rec;      IN: access record to mess with
    int32      length;          IN: number of bytes to read
-   VOIDP      data;            IN: buffer for data
+   void *      data;            IN: buffer for data
 RETURNS
    The number of bytes read or FAIL on error
 DESCRIPTION
@@ -665,7 +648,7 @@ DESCRIPTION
 
 ---------------------------------------------------------------------------*/
 int32
-HXPread(accrec_t * access_rec, int32 length, VOIDP data)
+HXPread(accrec_t * access_rec, int32 length, void * data)
 {
     CONSTR(FUNC, "HXPread");    /* for HERROR */
     extinfo_t  *info =          /* information on the special element */
@@ -703,17 +686,6 @@ HXPread(accrec_t * access_rec, int32 length, VOIDP data)
       }
 
     /* read it in from the file */
-#ifdef CM5
-    if (access_rec->access_type == DFACC_PARALLEL)
-      {
-          /* parallel access handling */
-          if (CM_SEEK(info->para_extfile_id, access_rec->posn + info->extern_offset) == FAIL)
-              HGOTO_ERROR(DFE_SEEKERROR, FAIL);
-          if (CM_READ(info->para_extfile_id, data, length) == FAIL)
-              HGOTO_ERROR(DFE_READERROR, FAIL);
-      }
-    else
-#endif
       {
           if (HI_SEEK(info->file_external, access_rec->posn + info->extern_offset) == FAIL)
               HGOTO_ERROR(DFE_SEEKERROR, FAIL);
@@ -745,7 +717,7 @@ USAGE
    int32 HXPwrite(access_rec, length, data)
    access_t * access_rec;      IN: access record to mess with
    int32      length;          IN: number of bytes to read
-   VOIDP      data;            IN: buffer of data
+   void *      data;            IN: buffer of data
 RETURNS
    The number of bytes written or FAIL on error
 DESCRIPTION
@@ -756,7 +728,7 @@ DESCRIPTION
 
 ---------------------------------------------------------------------------*/
 int32
-HXPwrite(accrec_t * access_rec, int32 length, const VOIDP data)
+HXPwrite(accrec_t * access_rec, int32 length, const void * data)
 {
     uint8       local_ptbuf[4]; /* temp buffer */
     CONSTR(FUNC, "HXPwrite");   /* for HERROR */
@@ -792,18 +764,6 @@ HXPwrite(accrec_t * access_rec, int32 length, const VOIDP data)
     }
 
     /* write the data onto file */
-#ifdef CM5
-    if (access_rec->access_type == DFACC_PARALLEL)
-      {
-          /* parallel access handling */
-          if (CM_SEEK(info->para_extfile_id,
-                      access_rec->posn + info->extern_offset) == FAIL)
-              HGOTO_ERROR(DFE_SEEKERROR, FAIL);
-          if (CM_WRITE(info->para_extfile_id, data, length) == FAIL)
-                HGOTO_ERROR(DFE_DENIED, FAIL);
-      }
-    else
-#endif /* CM5 */
       {
           if (HI_SEEK(info->file_external, access_rec->posn + info->extern_offset) == FAIL)
               HGOTO_ERROR(DFE_SEEKERROR, FAIL);
@@ -912,7 +872,7 @@ HXPinquire(accrec_t * access_rec, int32 *pfile_id, uint16 *ptag,
     if (paccess)
         *paccess = (int16)access_rec->access;
     if (pspecial)
-        *pspecial = access_rec->special;
+        *pspecial = (int16)access_rec->special;
 
 done:
   if(ret_value == FAIL)   
@@ -961,13 +921,13 @@ HXPendaccess(accrec_t * access_rec)
     file_rec->attach--;
 
     /* free the access record */
-    HDfree(access_rec);
+    HIrelease_accrec_node(access_rec);
 
 done:
   if(ret_value == FAIL)   
     { /* Error condition cleanup */
       if(access_rec!=NULL)
-          HDfree(access_rec);
+          HIrelease_accrec_node(access_rec);
 
     } /* end if */
 
@@ -1012,8 +972,8 @@ HXPcloseAID(accrec_t * access_rec)
       {
           if (info->file_open)
               HI_CLOSE(info->file_external);
-          HDfree((VOIDP) info->extern_file_name);
-          HDfree((VOIDP) info);
+          HDfree(info->extern_file_name);
+          HDfree(info);
           access_rec->special_info=NULL;
       }
 
@@ -1125,7 +1085,7 @@ HXPreset(accrec_t * access_rec, sp_info_block_t * info_block)
     info->extern_file_name = (char *) HDstrdup(info_block->path);
     if (!info->extern_file_name)
         HGOTO_ERROR(DFE_NOSPACE, FAIL);
-    info->length_file_name = HDstrlen(info->extern_file_name);
+    info->length_file_name = (int32)HDstrlen(info->extern_file_name);
 
     /*
      * delete the existing tag / ref object
@@ -1312,8 +1272,8 @@ HXIbuildfilename(const char *ext_fname, const intn acc_mode)
     static int	firstinvoked = 1;	/* true if invoked the first time */
 
     char	*finalpath = NULL;	/* Final pathname to return */
-    char	*fname = NULL;
-#if !(defined (MAC) || defined (macintosh) || defined(__MWERKS__) || defined (SYMANTEC_C))
+    const char	*fname = NULL;
+#if !(defined (MAC) || defined (macintosh) || defined (SYMANTEC_C))
     struct	stat filestat;	/* for checking pathname existence */
 #endif
     char        *ret_value = NULL; /* FAIL */
@@ -1327,13 +1287,13 @@ HXIbuildfilename(const char *ext_fname, const intn acc_mode)
 
     if (!ext_fname)
         HGOTO_ERROR(DFE_ARGS, NULL);
-    fname = (char *) ext_fname;
+    fname = ext_fname;
 
     /* get the space for the final pathname */
     if (!(finalpath=HDmalloc(MAX_PATH_LEN)))
         HGOTO_ERROR(DFE_NOSPACE, NULL);
 
-    fname_len = HDstrlen(fname);
+    fname_len = (int)HDstrlen(fname);
     
     switch (acc_mode){
     case DFACC_CREATE: {			/* Creating a new external element */
@@ -1345,7 +1305,7 @@ HXIbuildfilename(const char *ext_fname, const intn acc_mode)
 
             /* try function variable */
             if (extcreatedir) {
-                path_len = HDstrlen(extcreatedir);
+                path_len = (int)HDstrlen(extcreatedir);
 
                 if (fname_len + 1 + path_len + 1 > MAX_PATH_LEN )
                     HGOTO_ERROR(DFE_NOSPACE, NULL);
@@ -1355,7 +1315,7 @@ HXIbuildfilename(const char *ext_fname, const intn acc_mode)
 
             /* try Envrironment Variable */
             if (HDFEXTCREATEDIR) {
-                path_len = HDstrlen(HDFEXTCREATEDIR);
+                path_len = (int)HDstrlen(HDFEXTCREATEDIR);
 
                 if (fname_len + 1 + path_len + 1 > MAX_PATH_LEN )
                     HGOTO_ERROR(DFE_NOSPACE, NULL);
@@ -1382,9 +1342,9 @@ HXIbuildfilename(const char *ext_fname, const intn acc_mode)
             else if (!extdir && !HDFEXTDIR) {
                 HGOTO_ERROR(DFE_FNF, NULL);
             }
-            /* stripe the pathname component */
+            /* strip the pathname component */
             fname = HDstrrchr(fname, DIR_SEPC) + 1;
-            fname_len = HDstrlen(fname);
+            fname_len = (int)HDstrlen(fname);
 
             /* continue to Relative Pathname */
         }
