@@ -10,7 +10,7 @@
  *                                                                          *
  ****************************************************************************/
 
-/* hfile.h,v 1.88 1996/06/28 19:30:51 bljones Exp */
+/* $Id: hfile.h,v 1.105 1998/02/19 16:27:16 bljones Exp $ */
 
 /*+ hfile.h
    *** Header for hfile.c, routines for low level data element I/O
@@ -47,9 +47,9 @@
 /* Library version numbers */
 
 #define LIBVER_MAJOR    4
-#define LIBVER_MINOR    0
-#define LIBVER_RELEASE  2
-#define LIBVER_STRING   "NCSA HDF Version 4.0 Release 2, July 19, 1996"
+#define LIBVER_MINOR    1 
+#define LIBVER_RELEASE  2 
+#define LIBVER_STRING   "NCSA HDF Version 4.1 Release 2, March 1998"
 #define LIBVSTR_LEN    80   /* length of version string  */
 #define LIBVER_LEN  92      /* 4+4+4+80 = 92 */
 /* end of version tags */
@@ -83,9 +83,9 @@ typedef FILE *hdf_file_t;
 #   define HI_CREATE(p)        (fopen((p), "wb+"))
 #endif /* !SUN w/ GNU CC */
 #endif /* VMS */
-#   define HI_READ(f, b, n)    (((n) == fread((b), 1, (size_t)(n), (f))) ? \
+#   define HI_READ(f, b, n)    (((size_t)(n) == (size_t)fread((b), 1, (size_t)(n), (f))) ? \
                                 SUCCEED : FAIL)
-#   define HI_WRITE(f, b, n)   (((n) == fwrite((b), 1, (size_t)(n), (f))) ? \
+#   define HI_WRITE(f, b, n)   (((size_t)(n) == (size_t)fwrite((b), 1, (size_t)(n), (f))) ? \
                                 SUCCEED : FAIL)
 #   define HI_CLOSE(f)   (fclose(f))
 #   define HI_FLUSH(f)   (fflush(f)==0 ? SUCCEED : FAIL)
@@ -101,7 +101,7 @@ typedef FILE *hdf_file_t;
 typedef int hdf_file_t;
 #   define HI_OPEN(p, a)       (((a) & DFACC_WRITE) ? \
                                     open((p), O_RDWR) : open((p), O_RDONLY))
-#   define HI_CREATE(p)         (open((p), O_RDWR | O_CREAT | O_TRUNC))
+#   define HI_CREATE(p)         (open((p), O_RDWR | O_CREAT | O_TRUNC, 0666))
 #   define HI_CLOSE(f)          (close(f))
 #   define HI_FLUSH(f)          (SUCCEED)
 #   define HI_READ(f, b, n)     (((n)==read((f), (char *)(b), (n))) ? SUCCEED : FAIL)
@@ -128,6 +128,7 @@ typedef short hdf_file_t;
 #   define OPENERR(f)           (f < 0)
 #endif /* FILELIB == MACIO */
 
+#ifdef NOT_USED     /* Deprecated routines, not supported any more */
 #if (FILELIB == PCIO)
 /* using special PC functions to enable reading/writing large chunks */
 typedef FILE *hdf_file_t;
@@ -185,6 +186,7 @@ typedef HFILE hdf_file_t;
 #   define HI_TELL(f)  (_llseek((f),0l,1))
 #   define OPENERR(f)  ((f) == (HFILE)HFILE_ERROR)
 #endif /* FILELIB == WINNTIO */
+#endif /* NOT_USED */
 
 #if (FILELIB == PAGEBUFIO)
 #include "fmpio.h"
@@ -229,8 +231,18 @@ typedef MPFILE *hdf_file_t;
    
    Tag  -- identifies the type of data, 16 bit unsigned integer whose
            value ranges from 1 - 65535. Tags are assigned by NCSA.
+           The HDF tag space is divided as follows based on the 2 highest bits:
+
+              00: NCSA reserved ordinary tags
+              01: NCSA reserved special tags(i.e. regular tags made into 
+                                                  linked-block, external, 
+                                                  compressed or chunked.)
+              10, 11: User tags.
+
            Current tag assingments are:
            00001 - 32767  - reserved for NCSA use
+                            00001 - 16383 - NCSA regular tags
+                            16384 - 32767 - NCSA special tags
            32768 - 64999  - user definable
            65000 - 65535  - reserved for expansion of format
 
@@ -361,7 +373,15 @@ typedef struct filerec_t
       struct ddblock_t *ddnull; /* location of last ddblock with a DFTAG_NULL */
       int32       ddnull_idx;   /* offset of the last location with DFTAG_NULL */
 
+      /* tag tree for file */
       TBBT_TREE *tag_tree;      /* TBBT of the tags in the file */
+
+      /* annotation stuff for file */
+      intn       an_num[4];   /* Holds number of annotations found of each type */
+      TBBT_TREE *an_tree[4];  /* tbbt trees for each type of annotation in file 
+                               * i.e. file/data labels and descriptions.
+                               * This is done for faster searching of annotations
+                               * of a particular type. */
   }
 filerec_t;
 
@@ -385,11 +405,17 @@ typedef struct accrec_t
       int32       file_id;      /* id of attached file */
       atom_t      ddid;         /* DD id for the DD attached to */
       int32       posn;         /* seek position with respect to start of element */
-      VOIDP       special_info; /* special element info? */
+      void *       special_info; /* special element info? */
       struct funclist_t *special_func;  /* ptr to special function? */
+      struct accrec_t *next;    /* for free-list linking */
   }
 accrec_t;
 
+#ifdef HFILE_MASTER
+/* Pointer to the access record node free list */
+static accrec_t *accrec_free_list=NULL;
+#endif /* HFILE_MASTER */
+
 /* this type is returned to applications programs or other special
    interfaces when they need to know information about a given
    special element.  This is all information that would not be returned
@@ -415,6 +441,10 @@ typedef struct sp_info_block_t
       /* variable-length linked blocks */
       int32       min_block;    /* the minimum block size */
 
+    /* Chunked elements */
+      int32       chunk_size;   /* logical size of chunks */
+      int32       ndims;        /* number of dimensions */
+      int32       *cdims;       /* array of chunk lengths for each dimension */
   }
 sp_info_block_t;
 
@@ -430,8 +460,8 @@ typedef struct funclist_t
                                  uint16 *ptag, uint16 *pref, int32 *plength,
                                int32 *poffset, int32 *pposn, int16 *paccess,
                                      int16 *pspecial);
-      int32       (*read) (accrec_t * access_rec, int32 length, VOIDP data);
-      int32       (*write) (accrec_t * access_rec, int32 length, const VOIDP data);
+      int32       (*read) (accrec_t * access_rec, int32 length, void * data);
+      int32       (*write) (accrec_t * access_rec, int32 length, const void * data);
       intn        (*endaccess) (accrec_t * access_rec);
       int32       (*info) (accrec_t * access_rec, sp_info_block_t * info);
       int32       (*reset) (accrec_t * access_rec, sp_info_block_t * info);
@@ -505,14 +535,16 @@ extern      "C"
     extern accrec_t *HIget_access_rec
                 (void);
 
-    extern VOIDP HIgetspinfo
+    extern void HIrelease_accrec_node(accrec_t *acc);
+
+    extern void * HIgetspinfo
                 (accrec_t * access_rec);
 
     extern intn HPcompare_filerec_path
-                (const VOIDP obj, const VOIDP key);
+                (const void * obj, const void * key);
 
     extern intn HPcompare_accrec_tagref
-                (const VOIDP rec1, const VOIDP rec2);
+                (const void * rec1, const void * rec2);
 
     extern int32 HPgetdiskblock
                 (filerec_t * file_rec, int32 block_size, intn moveto);
@@ -524,19 +556,19 @@ extern      "C"
                 (int32 access_id, sp_info_block_t * info_block);
 
     extern intn HP_read
-                (filerec_t *file_rec,VOIDP buf,int32 bytes);
+                (filerec_t *file_rec,void * buf,int32 bytes);
 
     extern intn HPseek
                 (filerec_t *file_rec,int32 offset);
 
     extern intn HP_write
-                (filerec_t *file_rec,const VOIDP buf,int32 bytes);
+                (filerec_t *file_rec,const void * buf,int32 bytes);
 
     extern intn tagcompare
-                (VOIDP k1, VOIDP k2, intn cmparg);
+                (void * k1, void * k2, intn cmparg);
 
     extern VOID tagdestroynode
-                (VOIDP n);
+                (void * n);
 
 /*
    ** from hblocks.c
@@ -551,10 +583,10 @@ extern      "C"
                 (accrec_t * access_rec, int32 offset, int origin);
 
     extern int32 HLPread
-                (accrec_t * access_rec, int32 length, VOIDP data);
+                (accrec_t * access_rec, int32 length, void * data);
 
     extern int32 HLPwrite
-                (accrec_t * access_rec, int32 length, const VOIDP data);
+                (accrec_t * access_rec, int32 length, const void * data);
 
     extern int32 HLPinquire
                 (accrec_t * access_rec, int32 *pfile_id, uint16 *ptag, uint16 *pref,
@@ -583,10 +615,10 @@ extern      "C"
                 (accrec_t * access_rec, int32 offset, int origin);
 
     extern int32 HXPread
-                (accrec_t * access_rec, int32 length, VOIDP data);
+                (accrec_t * access_rec, int32 length, void * data);
 
     extern int32 HXPwrite
-                (accrec_t * access_rec, int32 length, const VOIDP data);
+                (accrec_t * access_rec, int32 length, const void * data);
 
     extern int32 HXPinquire
                 (accrec_t * access_rec, int32 *pfile_id, uint16 *ptag, uint16 *pref,
@@ -630,10 +662,10 @@ extern      "C"
                  int16 *pspecial);
 
     extern int32 HCPread
-                (accrec_t * access_rec, int32 length, VOIDP data);
+                (accrec_t * access_rec, int32 length, void * data);
 
     extern int32 HCPwrite
-                (accrec_t * access_rec, int32 length, const VOIDP data);
+                (accrec_t * access_rec, int32 length, const void * data);
 
     extern intn HCPendaccess
                 (accrec_t * access_rec);
@@ -663,10 +695,10 @@ extern      "C"
                  int16 *pspecial);
 
     extern int32 HVPread
-                (accrec_t * access_rec, int32 length, VOIDP data);
+                (accrec_t * access_rec, int32 length, void * data);
 
     extern int32 HVPwrite
-                (accrec_t * access_rec, int32 length, const VOIDP data);
+                (accrec_t * access_rec, int32 length, const void * data);
 
     extern intn HVPendaccess
                 (accrec_t * access_rec);
@@ -674,7 +706,12 @@ extern      "C"
     extern int32 HVPinfo
                 (accrec_t * access_rec, sp_info_block_t * info_block);
 
-#if defined (MAC) || defined (macintosh) || defined(__MWERKS__) || defined (SYMANTEC_C)
+/*
+   ** from hchunks.c - should be the same as found in 'hchunks.h'
+ */
+#include "hchunks.h"
+
+#if defined (MAC) || defined (macintosh) || defined (SYMANTEC_C)
     extern hdf_file_t mopen
                 (char *filename, intn access);
 
