@@ -11,10 +11,10 @@
  ****************************************************************************/
 
 #ifdef RCSID
-static char RcsId[] = "1.8";
+static char RcsId[] = "$Revision: 1.8 $";
 #endif
 
-/* hvblocks.c,v 1.8 1996/05/13 17:00:53 georgev Exp */
+/* $Id: hvblocks.c,v 1.8 1996/05/13 17:00:53 georgev Exp $ */
 
 /*LINTLIBRARY */
 /* ------------------------------ hvblocks.c -------------------------------
