@@ -11,10 +11,28 @@
  ****************************************************************************/
 
 #ifdef RCSID
-static char RcsId[] = "@(#)1.29";
+static char RcsId[] = "@(#)$Revision: 1.40 $";
 #endif
 
-/* vgf.c,v 1.29 1996/06/21 05:14:21 acheng Exp */
+/* $Id: vgf.c,v 1.40 1998/02/09 21:54:13 epourmal Exp $ */
+
+/*-----------------------------------------------------------------------------
+  FUNCTION NAMEING CONVENTION:(For the most part true)
+  ---------------------------------------------------
+  This file contains the HDF-style C stubs for the Vxxx interfaces i.e.
+  both Vgroups and Vdatas. They call the corresponding C-functions for 
+  the Vxxx interfaces.
+
+  The basic routines called by fortran will be of the form vfxxx.
+
+  If only a C stub is needed it will be named nvfxxx and have the FNAME()
+  function applied to it. There are a few exceptions where C stub is 
+  named for example ndfixxx or ndfxxx, oh well.
+
+  If a Fortran stub is also required the fortran stub will be called
+  vfxxx(vgff.f) and the one in here will be nvfxxxc and again be FNAME()ed
+
+-----------------------------------------------------------------------------*/
 
 /*
    *
@@ -26,7 +44,8 @@ static char RcsId[] = "@(#)1.29";
    *
    *********************************************************************** */
 
-#include "vg.h"
+#define VSET_INTERFACE
+#include "hdf.h"
 
 /*
    **  remove trailing blanks from a string. input argument is a  string
@@ -40,7 +59,7 @@ trimendblanks(char *ss)
 {
     int32       i, n;
 
-    n = HDstrlen(ss);
+    n = (int32)HDstrlen(ss);
     for (i = n - 1; i >= 0; i--)
       {
           if (ss[i] != ' ')
@@ -93,7 +112,7 @@ ndfivopn(_fcd name, intf * acc_mode, intf * defdds, intf * namelen)
 FRETVAL(intf)
 ndfvclos(intf * file_id)
 {
-    return (Vclose(*file_id));
+    return (Vclose((int32)*file_id));
 }   /* ndfvclos() */
 
 /*
@@ -102,7 +121,7 @@ ndfvclos(intf * file_id)
  */
 
 FRETVAL(intf)
-nvatchc(HFILEID * f, intf * vgid, _fcd accesstype)
+nvatchc(intf * f, intf * vgid, _fcd accesstype)
 {
     int32       vkey;
     char       *acc;
@@ -110,10 +129,10 @@ nvatchc(HFILEID * f, intf * vgid, _fcd accesstype)
     acc = HDf2cstring(accesstype, 1);
     if (!acc) return(FAIL);
 
-    vkey = Vattach(*f, *vgid, acc);
+    vkey = Vattach((int32)*f, *vgid, acc);
     HDfree(acc);
 
-    return (vkey);
+    return ((intf)vkey);
 }
 
 /* ------------------------------------------------------------------ */
@@ -184,9 +203,9 @@ nvdelete(intf * f, intf * vkey)
  */
 
 FRETVAL(intf)
-nvgidc(HFILEID * f, intf * vgid)
+nvgidc(intf * f, intf * vgid)
 {
-    return ((intf) Vgetid(*f, *vgid));
+    return ((intf) Vgetid((int32)*f, *vgid));
 }
 
 /* ------------------------------------------------------------------ */
@@ -273,9 +292,9 @@ nvisvgc(intf * vkey, intf * id)
  */
 
 FRETVAL(intf)
-nvfstart(HFILEID * f)
+nvfstart(intf * f)
 {
-    return (Vstart(*f));
+    return (Vstart((int32) *f));
 }   /* nvfstart */
 
 /* ------------------------------------------------------------------ */
@@ -284,9 +303,9 @@ nvfstart(HFILEID * f)
  */
 
 FRETVAL(intf)
-nvfend(HFILEID * f)
+nvfend(intf * f)
 {
-    return ((intf) Vend(*f));
+    return ((intf) Vend((int32) *f));
 }   /* nvfend */
 
 /* ------------------------------------------------------------------ */
@@ -311,17 +330,10 @@ nvisvsc(intf * vkey, intf * id)
  */
 
 FRETVAL(intf)
-nvsatchc(HFILEID * f, intf * vsid, _fcd accesstype)
+nvsatchc(intf * f, intf * vsid, _fcd accesstype)
 {
-    intf        vkey;
-    char       *acc;
-
-    acc = HDf2cstring(accesstype, 1);   /* 'r' or 'w' only */
-    if (!acc) return(FAIL);
-    vkey = VSattach(*f, *vsid, acc);
-    HDfree(acc);
-
-    return (vkey);
+    /* need not HDf2cstring since only first char is accessed. */
+    return(VSattach((int32) *f, *vsid, _fcdtocp(accesstype)));
 }
 
 /* ------------------------------------------------------------------ */
@@ -360,7 +372,7 @@ nvsqtag(intf * vkey)
     return ((intf)VSQuerytag((int32)*vkey));
 }
 
-/* ------------------------------------------------------------------ */
+/* ----------------------------------------------------------------- */
 /*
    **  get the version # of a vdata
    **  related: VSgetversion--vsgver--
@@ -391,9 +403,24 @@ nvsseekc(intf * vkey, intf * eltpos)
  */
 
 FRETVAL(intf)
-nvsgnamc(intf * vkey, _fcd vsname)
+nvsgnamc(intf * vkey, _fcd vsname, intf *vsnamelen)
 {
-    return (VSgetname(*vkey, _fcdtocp(vsname)));
+    CONSTR(FUNC, "vsgnamc");
+    char	*tvsname = NULL;
+    intn	status;
+
+    /* Allocate space for fortran strings */
+    tvsname = (char *) HDmalloc(*vsnamelen + 1);
+    if (!tvsname)
+        HRETURN_ERROR(DFE_NOSPACE, FAIL);
+
+    status = VSgetname(*vkey, tvsname);
+
+    /* convert C-string results back to Fortran strings */
+    HDpackFstring(tvsname, _fcdtocp(vsname), (intn) *vsnamelen);
+    HDfree(tvsname);
+
+    return(status);
 }   /* VSGNAMC */
 
 /* ------------------------------------------------------------------ */
@@ -403,9 +430,24 @@ nvsgnamc(intf * vkey, _fcd vsname)
  */
 
 FRETVAL(intf)
-nvsgclsc(intf * vkey, _fcd vsclass)
+nvsgclsc(intf * vkey, _fcd vsclass, intf *vsclasslen)
 {
-    return (VSgetclass(*vkey, _fcdtocp(vsclass)));
+    CONSTR(FUNC, "vsgclsc");
+    char	*tvsclass = NULL;
+    intn	status;
+
+    /* Allocate space for fortran strings */
+    tvsclass = (char *) HDmalloc(*vsclasslen + 1);
+    if (!tvsclass)
+        HRETURN_ERROR(DFE_NOSPACE, FAIL);
+
+    status = VSgetclass(*vkey, tvsclass);
+
+    /* convert C-string results back to Fortran strings */
+    HDpackFstring(tvsclass, _fcdtocp(vsclass), (intn) *vsclasslen);
+    HDfree(tvsclass);
+
+    return(status);
 }   /* VSGCLSC */
 
 /* ------------------------------------------------------------------ */
@@ -416,10 +458,39 @@ nvsgclsc(intf * vkey, _fcd vsclass)
 
 FRETVAL(intf)
 nvsinqc(intf * vkey, intf * nelt, intf * interlace, _fcd fields, intf * eltsize,
-        _fcd vsname)
+        _fcd vsname, intf *fieldslen, intf *vsnamelen)
 {
-    return ((intf) VSinquire(*vkey, (int32 *) nelt, (int32 *) interlace,
-                  _fcdtocp(fields), (int32 *) eltsize, _fcdtocp(vsname)));
+    CONSTR(FUNC, "vsinqc");
+    char	*tfields = NULL;
+    char	*tvsname = NULL;
+    intn	status;
+    int32	tnelt, til, teltsz;
+
+    /* Allocate space for fortran strings */
+    tfields = (char *) HDmalloc(*fieldslen + 1);
+    if (!tfields)
+        HRETURN_ERROR(DFE_NOSPACE, FAIL);
+    tvsname = (char *) HDmalloc(*vsnamelen + 1);
+    if (!tvsname){
+        HDfree(tfields);
+        HRETURN_ERROR(DFE_NOSPACE, FAIL)
+    }
+    
+    /* the following contains error for nelt, interlace and eltsize */
+    /* if int32 and intf are different in size. */
+    status = VSinquire(*vkey, &tnelt, &til, tfields, &teltsz, tvsname);
+    if (status != FAIL){
+	*nelt = tnelt;
+	*interlace = til;
+	*eltsize = teltsz;
+	/* convert C-string results back to Fortran strings */
+	HDpackFstring(tfields, _fcdtocp(fields), (intn) *fieldslen);
+	HDpackFstring(tvsname, _fcdtocp(vsname), (intn) *vsnamelen);
+    }
+    HDfree(tfields);
+    HDfree(tvsname);
+
+    return status;
 }   /* VSINQC */
 
 /* ------------------------------------------------------------------ */
@@ -450,7 +521,7 @@ nvsfexc(intf * vkey, _fcd fields, intf * fieldslen)
  */
 
 FRETVAL(intf)
-nvsfndc(HFILEID * f, _fcd name, intf * namelen)
+nvsfndc(intf * f, _fcd name, intf * namelen)
 {
     intf        ret;
     char       *cname;
@@ -471,7 +542,7 @@ nvsfndc(HFILEID * f, _fcd name, intf * namelen)
  */
 
 FRETVAL(intf)
-nvsgidc(HFILEID * f, intf * vsid)
+nvsgidc(intf * f, intf * vsid)
 {
     return ((intf) VSgetid(*f, *vsid));
 }
@@ -483,7 +554,7 @@ nvsgidc(HFILEID * f, intf * vsid)
  */
 
 FRETVAL(intf)
-nvsdltc(HFILEID * f, intf * vsid)
+nvsdltc(intf * f, intf * vsid)
 {
     return ((intf) VSdelete(*f, *vsid));
 }
@@ -840,7 +911,7 @@ nvssizc(intf * vkey, _fcd fields, intf * fieldslen)
  */
 
 FRETVAL(intf)
-nventsc(HFILEID * f, intf * vgid)
+nventsc(intf * f, intf * vgid)
 {
     return ((intf) Ventries(*f, *vgid));
 }
@@ -852,7 +923,7 @@ nventsc(HFILEID * f, intf * vgid)
  */
 
 FRETVAL(intf)
-nvlonec(HFILEID * f, intf * idarray, intf * asize)
+nvlonec(intf * f, intf * idarray, intf * asize)
 {
     return ((intf) Vlone(*f, (int32 *)idarray, (int32) *asize));
 }
@@ -864,7 +935,7 @@ nvlonec(HFILEID * f, intf * idarray, intf * asize)
  */
 
 FRETVAL(intf)
-nvslonec(HFILEID * f, intf * idarray, intf * asize)
+nvslonec(intf * f, intf * idarray, intf * asize)
 {
     return (VSlone(*f, (int32 *)idarray, (int32) *asize));
 }
@@ -876,7 +947,7 @@ nvslonec(HFILEID * f, intf * idarray, intf * asize)
  */
 
 FRETVAL(intf)
-nvfindc(HFILEID * f, _fcd name, intf * namelen)
+nvfindc(intf * f, _fcd name, intf * namelen)
 {
     char *tmp_name;
     intf ret;
@@ -897,7 +968,7 @@ nvfindc(HFILEID * f, _fcd name, intf * namelen)
  */
 
 FRETVAL(intf)
-nvfndclsc(HFILEID * f, _fcd vgclass, intf * classlen)
+nvfndclsc(intf * f, _fcd vgclass, intf * classlen)
 {
     char *t_class;
     intf ret;
@@ -923,7 +994,7 @@ nvfndclsc(HFILEID * f, _fcd vgclass, intf * classlen)
  */
 
 FRETVAL(intf)
-nvhsdc(HFILEID * f, _fcd field, uint8 *buf, intf * n, intf * datatype, _fcd vsname,
+nvhsdc(intf * f, _fcd field, uint8 *buf, intf * n, intf * datatype, _fcd vsname,
        _fcd vsclass, intf * fieldlen, intf * vsnamelen, intf * vsclasslen)
 {
     char       *fld, *name, *tclass;
@@ -957,7 +1028,7 @@ nvhsdc(HFILEID * f, _fcd field, uint8 *buf, intf * n, intf * datatype, _fcd vsna
  */
 
 FRETVAL(intf)
-nvhscdc(HFILEID * f, _fcd field, _fcd cbuf, intf * n, intf * datatype, _fcd vsname,
+nvhscdc(intf * f, _fcd field, _fcd cbuf, intf * n, intf * datatype, _fcd vsname,
        _fcd vsclass, intf * fieldlen, intf * vsnamelen, intf * vsclasslen)
 {
     if ((*datatype != DFNT_CHAR) && (*datatype != DFNT_UCHAR))
@@ -973,7 +1044,7 @@ nvhscdc(HFILEID * f, _fcd field, _fcd cbuf, intf * n, intf * datatype, _fcd vsna
  */
 
 FRETVAL(intf)
-nvhscdmc(HFILEID * f, _fcd field, _fcd cbuf, intf * n, intf * datatype,
+nvhscdmc(intf * f, _fcd field, _fcd cbuf, intf * n, intf * datatype,
         _fcd vsname, _fcd vsclass, intf * order, intf * fieldlen,
         intf * vsnamelen, intf * vsclasslen)
 {
@@ -990,7 +1061,7 @@ nvhscdmc(HFILEID * f, _fcd field, _fcd cbuf, intf * n, intf * datatype,
  */
 
 FRETVAL(intf)
-nvhsdmc(HFILEID * f, _fcd field, uint8 *buf, intf * n, intf * datatype,
+nvhsdmc(intf * f, _fcd field, uint8 *buf, intf * n, intf * datatype,
         _fcd vsname, _fcd vsclass, intf * order, intf * fieldlen,
         intf * vsnamelen, intf * vsclasslen)
 {
@@ -1028,7 +1099,7 @@ nvhsdmc(HFILEID * f, _fcd field, uint8 *buf, intf * n, intf * datatype,
  */
 
 FRETVAL(intf)
-nvhmkgpc(HFILEID * f, intf * tagarray, intf * refarray, intf * n, _fcd vgname,
+nvhmkgpc(intf * f, intf * tagarray, intf * refarray, intf * n, _fcd vgname,
          _fcd vgclass, intf * vgnamelen, intf * vgclasslen)
 {
     char       *gname, *gclass;
@@ -1176,7 +1247,7 @@ nvadtrc(intf * vkey, intf * tag, intf * ref)
 FRETVAL(intf)
 nvsqfnelt(intf * vkey, intf * nelt)
 {
-    int32       ret_nelt;
+    int32       ret_nelt=0;
     intn        stat;
 
     stat = VSQuerycount((int32) *vkey, &ret_nelt);
@@ -1194,7 +1265,7 @@ nvsqfnelt(intf * vkey, intf * nelt)
 FRETVAL(intf)
 nvsqfintr(intf * vkey, intf * interlace)
 {
-    int32       ret_inter;
+    int32       ret_inter=0;
     intn        stat;
 
     stat = VSQueryinterlace((int32) *vkey, &ret_inter);
@@ -1232,7 +1303,7 @@ nvsqfldsc(intf * vkey, _fcd fields, intf *fieldslen)
 FRETVAL(intf)
 nvsqfvsiz(intf * vkey, intf * size)
 {
-    int32       ret_size;
+    int32       ret_size=0;
     intn        stat;
 
     stat = VSQueryvsize((int32) *vkey, &ret_size);
@@ -1334,3 +1405,49 @@ nvsfncpk(intf *vs, intf *packtype, _fcd buflds, intf *buf, intf *bufsz,
     if (afield) HDfree(afield);
     return(ret);
 }
+
+/*
+   **  C-stub for deleting a tag/ref pair in a vgroup.
+   **  related: called by vfdtr() and calls Vdeletetagref(), 
+   **  
+ */
+
+FRETVAL(intf)
+nvdtrc(intf * vkey, intf * tag, intf * ref)
+{
+    return ((intf) Vdeletetagref(*vkey, *tag, *ref));
+}
+
+/*------------------------------------------------------------------------
+ *       Name:      vscfcls 
+ *       Purpose:   calls VSfindclass 
+ *       Inputs:    id    -  file ID
+ *                  name  -  class of vdata to find  
+ *       Returns:   returns 0 if not found, or error. Otherwise, returns
+ *                  the vdata's ref number (a positive integer)
+ *       Related functions: vffcls, VSfindclass 
+ *       Users:     HDF Fortran programmers
+ ---------------------------------------------------------------------*/
+     FRETVAL (intf)
+#ifdef PROTOTYPE
+       nvscfcls( intf *id, _fcd name, intf *namelen )
+#else
+       nvscfcls (id, name, namelen)
+               intf   *id;
+              _fcd    name;
+               intf   *namelen;
+#endif /* PROTOTYPE */
+
+{
+       intf  fi_id;
+       intf  ret;
+       char  *class_name;
+
+       fi_id = *id;
+       class_name = HDf2cstring(name, (intn) *namelen);
+       if (!class_name) return(FAIL);
+
+       ret = VSfindclass( fi_id, class_name);
+       HDfree(class_name);
+       return(ret);
+} 
