@@ -11,10 +11,10 @@
  ****************************************************************************/
 
 #ifdef RCSID
-static char RcsId[] = "@(#)1.39";
+static char RcsId[] = "@(#)$Revision: 1.48 $";
 #endif
 
-/* vrw.c,v 1.39 1996/05/20 20:51:41 koziol Exp */
+/* $Id: vrw.c,v 1.48 1998/02/07 22:29:01 koziol Exp $ */
 /***********************************************************************
 *
 * vrw.c
@@ -23,7 +23,8 @@ static char RcsId[] = "@(#)1.39";
 *
 
 LOCAL ROUTINES
- None
+ VSPshutdown  --  Free the Vtbuf buffer.
+
 EXPORTED ROUTINES
  VSseek  -- Seeks to an element boundary within a vdata i.e. 2nd element.
  VSread  -- Reads a specified number of elements' worth of data from a vdata.
@@ -32,101 +33,109 @@ EXPORTED ROUTINES
 	     You must specify how your data in your buffer is interlaced.
              Creates an aid, and writes it out if this is the first time.
 
+ NOTE: Another pass needs to made through this file to update some of
+       the comments about certain sections of the code. -GV 9/8/97
+
 ************************************************************************/
 
-#include "vg.h"
+#define VSET_INTERFACE
+#include "hdf.h"
 
 #ifndef MIN
 #define MIN(a,b)     ((a) < (b) ? (a) : (b))
 #endif /* MIN */
 
-#if 0
-/*
- * Private conversion buffer stuff
- * VDATA_BUFFER_MAX is the largest buffer that can be allocated for
- *   writing (haven't implemented reading yet).
- * Vtbuf is the buffer
- * Vtbufsize is the buffer size in bytes at any given time.
- * Vtbuf is increased in size as need be
- * BUG: the final Vtbuf never gets freed
- */
-#define VDATA_BUFFER_MAX 1000000
-#endif /* if 0 */
-
 PRIVATE uint32 Vtbufsize = 0;
 PRIVATE uint8 *Vtbuf = NULL;
 
-/*--------------------------------------------------------------------------
+/*******************************************************************************
  NAME
-    VSPshutdown
- PURPOSE
-    Free the Vtbuf buffer.
- USAGE
-    intn VSPshutdown()
- RETURNS
-    Returns SUCCEED/FAIL
+    VSPshutdown  --  Free the Vtbuf buffer.
+
  DESCRIPTION
     For completeness, when the VSet interface is shut-down, free the Vtbuf.
- GLOBAL VARIABLES
- COMMENTS, BUGS, ASSUMPTIONS
+
     Should only ever be called by the "atexit" function HDFend
- EXAMPLES
- REVISION LOG
---------------------------------------------------------------------------*/
-intn VSPshutdown(void)
+
+ RETURNS
+    Returns SUCCEED/FAIL
+
+*******************************************************************************/
+intn 
+VSPshutdown(void)
 {
   intn  ret_value = SUCCEED;
 
-  if(Vtbuf!=NULL)
+  /* free global buffers */
+  if(Vtbuf != NULL)
     {
       HDfree(Vtbuf);
-      Vtbuf=NULL;
+      Vtbuf = NULL;
       Vtbufsize = 0;
     } /* end if */
 
+  /* Clear the local buffers in vio.c */
+  ret_value = VSPhshutdown();
+
   return ret_value;
 } /* end VSPshutdown() */
 
-/* --------------------------- VSseek -------------------------------------- */
-
-/*
+/*******************************************************************************
+NAME
    VSseek
 
+DESCRIPTION
    Seeks to an element boundary within a vdata
    Vdata must be attached with "r" or "w" access.
    Specify eltpos = 0 for 1st element, 1 for 2nd element etc.
+
+   (eg  returns 5 if seek to the 6th element, etc)
+
+RETURNS
    RETURNS FAIL on error
    RETURNS position of element seeked to (0 or a +ve integer)
-   (eg  returns 5 if seek to the 6th element, etc)
- */
+
+*******************************************************************************/
 int32
-VSseek(int32 vkey, int32 eltpos)
+VSseek(int32 vkey,   /* IN: vdata key */
+       int32 eltpos  /* IN: element position in vdata */)
 {
-    int32       ret, offset;
-    vsinstance_t *w;
-    VDATA      *vs;
-    int32      ret_value = SUCCEED;
+    int32        ret;
+    int32        offset;
+    vsinstance_t *w = NULL;
+    VDATA        *vs = NULL;
+    int32        ret_value = SUCCEED;
     CONSTR(FUNC, "VSseek");
 
 #ifdef HAVE_PABLO
     TRACE_ON(VS_mask, ID_VSseek);
 #endif /* HAVE_PABLO */
 
+    /* clear error stack */
+    HEclear();
+
+    /* check if vdata is part of vdata group */
     if (HAatom_group(vkey)!=VSIDGROUP)
         HGOTO_ERROR(DFE_ARGS, FAIL);
 
-    /* locate vs's index in vstab */
+    /* get vdata instance */
     if (NULL == (w = (vsinstance_t *) HAatom_object(vkey)))
         HGOTO_ERROR(DFE_NOVS, FAIL);
 
+    /* get vdata itself and check it. Check element postion also. */
     vs = w->vs;
     if ((vs == NULL) || (eltpos < 0))
         HGOTO_ERROR(DFE_ARGS, FAIL);
 
+    /* Don't allow seeks in 0-field vdatas */
+    if (vs->wlist.n<=0)
+        HGOTO_ERROR(DFE_BADFIELDS, FAIL);
+
+    /* calculate offset of element in vdata */
     offset = eltpos * vs->wlist.ivsize;
 
-    ret = Hseek(vs->aid, offset, DF_START);
-    if (ret == FAIL)
+    /* seek to element */
+    if (( ret = Hseek(vs->aid, offset, DF_START)) == FAIL)
         HGOTO_ERROR(DFE_BADSEEK, FAIL);
 
     ret_value = (eltpos);
@@ -145,29 +154,41 @@ done:
   return ret_value;
 }	/* VSseek */
 
-/* ------------------------------------------------------------------------ */
-/*
+/*******************************************************************************
+NAME
    VSread
+
+DESCRIPTION
    Reads a specified number of elements' worth of data from a vdata.
    Data will be returned to you interlaced in the way you specified.
+
+RETURNS
    RETURNS FAIL if error
    RETURNS the number of elements read (0 or a +ve integer).
- */
+
+*******************************************************************************/
 int32 
-VSread(int32 vkey, uint8 buf[], int32 nelt, int32 interlace)
+VSread(int32 vkey,       /* IN: vdata key */
+       uint8 buf[],      /* IN/OUT: space to put elements in */
+       int32 nelt,       /* IN: number of elements to read */
+       int32 interlace   /* IN: interlace to return elements in 'buf' */)
 {
     intn isize = 0;
     intn order = 0;
     intn index = 0;
-    uint16 esize = 0;
-    uint16 hsize = 0;
-    uint8 *b1, *b2;
-    int32  i, j, nv, offset, type;
-    DYN_VWRITELIST *w;
-    DYN_VREADLIST  *r;
-    int32       uvsize;         /* size of "element" as NEEDED by user */
-    vsinstance_t *wi;
-    VDATA      *vs;
+    intn esize = 0;
+    intn hsize = 0;
+    uint8 *b1 = NULL;
+    uint8 *b2 = NULL;
+    int32  i, j;
+    int32  nv;
+    int32  offset;
+    int32  type;
+    int32  uvsize;         /* size of "element" as NEEDED by user */
+    DYN_VWRITELIST *w = NULL;
+    DYN_VREADLIST  *r = NULL;
+    vsinstance_t   *wi = NULL;
+    VDATA          *vs = NULL;
     int32      ret_value = SUCCEED;
     CONSTR(FUNC, "VSread");
 
@@ -175,36 +196,49 @@ VSread(int32 vkey, uint8 buf[], int32 nelt, int32 interlace)
     TRACE_ON(VS_mask, ID_VSread);
 #endif /* HAVE_PABLO */
 
-    if (HAatom_group(vkey)!=VSIDGROUP)
+    /* clear error stack */
+    HEclear();
+
+    /* check if vdata is part of vdata group */
+    if (HAatom_group(vkey) != VSIDGROUP)
         HGOTO_ERROR(DFE_ARGS, FAIL);
 
-    /* locate vs's index in vstab */
+    /* get vdata instance */
     if (NULL == (wi = (vsinstance_t *) HAatom_object(vkey)))
         HGOTO_ERROR(DFE_NOVS, FAIL);
 
+    /* get vdata itself and check it */
     vs = wi->vs;
     if (vs == NULL)
         HGOTO_ERROR(DFE_ARGS, FAIL);
 
+    /* check access id and number of vertices in vdata */
     if ((vs->aid == 0) || (vs->nvertices == 0))
         HGOTO_ERROR(DFE_ARGS, FAIL);
 
+    /* Don't allow reads in 0-field vdatas */
+    if (vs->wlist.n<=0)
+        HGOTO_ERROR(DFE_BADFIELDS, FAIL);
+
+    /* check if vdata exists in file */
     if (vexistvs(vs->f, vs->oref) == FAIL)
         HGOTO_ERROR(DFE_NOVS, FAIL);
 
+    /* check interlace parameter */
     if (interlace != FULL_INTERLACE && interlace != NO_INTERLACE)
         HGOTO_ERROR(DFE_ARGS, FAIL);
 
+    /* read/write lists */
     w = &(vs->wlist);
     r = &(vs->rlist);
-    hsize = vs->wlist.ivsize;   /* size as stored in HDF */
+    hsize = (intn)vs->wlist.ivsize;   /* size as stored in HDF */
 
     /* alloc space (Vtbuf) for reading in the raw data from vdata */
-    if (Vtbufsize < nelt * (uint32) hsize)
+    if (Vtbufsize < (size_t)nelt * (size_t) hsize)
       {
-          Vtbufsize = nelt * (uint32) hsize;
+          Vtbufsize = (size_t)nelt * (size_t) hsize;
           if (Vtbuf)
-              HDfree((VOIDP) Vtbuf);
+              HDfree(Vtbuf);
           if ((Vtbuf = (uint8 *) HDmalloc(Vtbufsize)) == NULL)
               HGOTO_ERROR(DFE_NOSPACE, FAIL);
       }
@@ -245,9 +279,9 @@ VSread(int32 vkey, uint8 buf[], int32 nelt, int32 interlace)
       {
           b1 = buf;
           b2 = Vtbuf;
-          DFKsetNT(w->type[0]);
+          DFKsetNT((int32)w->type[0]);
 
-          DFKnumin(b2, b1, (uint32) w->order[0] * nelt, 0, 0);
+          DFKnumin(b2, b1, (uint32) w->order[0] * (uint32)nelt, 0, 0);
 
           HGOTO_DONE(nelt);
       }     /* case (e) */
@@ -260,11 +294,11 @@ VSread(int32 vkey, uint8 buf[], int32 nelt, int32 interlace)
           for (j = 0; j < r->n; j++)
             {
                 i = r->item[j];
-                b2 = Vtbuf + w->off[i];
-                type = w->type[i];
-                isize = w->isize[i];
-                esize = w->esize[i];
-                order = w->order[i];
+                b2 = Vtbuf + (size_t)w->off[i];
+                type = (int32)w->type[i];
+                isize = (intn)w->isize[i];
+                esize = (intn)w->esize[i];
+                order = (intn)w->order[i];
 
                 DFKsetNT(type);
                 for (index = 0; index < order; index++)
@@ -285,11 +319,11 @@ VSread(int32 vkey, uint8 buf[], int32 nelt, int32 interlace)
           for (j = 0; j < r->n; j++)
             {
                 i = r->item[j];
-                b2 = Vtbuf + w->off[i] * nelt;
-                type = w->type[i];
-                esize = w->esize[i];
-                isize = w->isize[i];
-                order = w->order[i];
+                b2 = Vtbuf + (size_t)w->off[i] * (size_t)nelt;
+                type = (int32)w->type[i];
+                esize = (intn)w->esize[i];
+                isize = (intn)w->isize[i];
+                order = (intn)w->order[i];
 
                 DFKsetNT(type);
                 for (index = 0; index < order; index++)
@@ -314,11 +348,11 @@ VSread(int32 vkey, uint8 buf[], int32 nelt, int32 interlace)
             {
                 i = r->item[j];
                 b1 = buf + offset;
-                b2 = Vtbuf + w->off[i];
-                type = w->type[r->item[j]];
-                esize = w->esize[i];
-                isize = w->isize[i];
-                order = w->order[i];
+                b2 = Vtbuf + (size_t)w->off[i];
+                type = (int32)w->type[r->item[j]];
+                esize = (intn)w->esize[i];
+                isize = (intn)w->isize[i];
+                order = (intn)w->order[i];
 
                 DFKsetNT(type);
                 for (index = 0; index < order; index++)
@@ -344,11 +378,11 @@ VSread(int32 vkey, uint8 buf[], int32 nelt, int32 interlace)
             {
                 i = r->item[j];
                 b1 = buf + offset;
-                b2 = Vtbuf + w->off[i] * nelt;
-                type = w->type[i];
-                isize = w->isize[i];
-                esize = w->esize[i];
-                order = w->order[i];
+                b2 = Vtbuf + (size_t)w->off[i] * (size_t)nelt;
+                type = (int32)w->type[i];
+                isize = (intn)w->isize[i];
+                esize = (intn)w->esize[i];
+                order = (intn)w->order[i];
 
                 DFKsetNT(type);
                 for (index = 0; index < order; index++)
@@ -377,38 +411,50 @@ done:
   return ret_value;
 }   /* VSread */
 
-/* ------------------------------------------------------------------ */
-/*
+/*******************************************************************************
+NAME
    VSwrite
+
+DESCRIPTION
    Writes a specified number of elements' worth of data to a vdata.
    You must specify how your data in your buffer is interlaced.
-   RETURNS FAIL if error
-   RETURNS the number of elements written (0 or a +ve integer).
 
    NEW
    create an aid, and write out if this is the first time.
    (otherwise) subsequent writes result in link-blocks.
- */
+
+RETURNS
+   RETURNS FAIL if error
+   RETURNS the number of elements written (0 or a +ve integer).
+
+*******************************************************************************/
 int32 
-VSwrite(int32 vkey, uint8 buf[], int32 nelt, int32 interlace)
+VSwrite(int32 vkey,         /* IN: vdata key */
+        const uint8 buf[],  /* IN: elements to write to vdata */
+        int32 nelt,         /* IN: number of elements */
+        int32 interlace     /* IN: interlace of elements 'buf' */)
 {
     intn isize = 0;
     intn order = 0;
     intn index = 0;
-    uint16 esize = 0;
-    uint8 *src, *dest, *Src;
-
-    int32       j, type, offset;
-    int32       position, new_size;
+    intn esize = 0;
+    uint8 *dest = NULL;
+    const uint8 *src, *Src;
+    int32       j;
+    int32       type;
+    int32       offset;
+    int32       position = 0;
+    int32       new_size;
     int32       status;
     int32       total_bytes;    /* total number of bytes that need to be written out */
-    DYN_VWRITELIST *w;
+    DYN_VWRITELIST *w = NULL;
     int32       int_size;       /* size of "element" as needed by user in memory */
     intn        hdf_size = 0;   /* size of record in HDF file */
-    vsinstance_t *wi;
-    VDATA      *vs;
+    vsinstance_t *wi = NULL;
+    VDATA        *vs = NULL;
     int32       bytes;          /* number of elements / bytes to write next time */
-    int32       chunk, done;    /* number of records to do / done */
+    int32       chunk;
+    int32       done;    /* number of records to do / done */
     int32       ret_value = SUCCEED;
     CONSTR(FUNC, "VSwrite");
 
@@ -416,24 +462,31 @@ VSwrite(int32 vkey, uint8 buf[], int32 nelt, int32 interlace)
     TRACE_ON(VS_mask, ID_VSwrite);
 #endif /* HAVE_PABLO */
 
-    if (HAatom_group(vkey)!=VSIDGROUP)
+    /* clear error stack */
+    HEclear();
+
+    /* check if vdata is part of vdata group */
+    if (HAatom_group(vkey) != VSIDGROUP)
         HGOTO_ERROR(DFE_ARGS, FAIL);
 
-    /* locate vs's index in vstab */
+    /* get vdata instance */
     if (NULL == (wi = (vsinstance_t *) HAatom_object(vkey)))
         HGOTO_ERROR(DFE_NOVS, FAIL);
 
+    /* get vdata itself and check it. Also check number of elements */
     vs = wi->vs;
-
     if ((nelt <= 0) || (vs == NULL))
         HGOTO_ERROR(DFE_ARGS, FAIL);
 
+    /* check if write access to vdata */
     if (vs->access != 'w')
         HGOTO_ERROR(DFE_BADACC, FAIL);
 
-    if (-1L == vexistvs(vs->f, vs->oref))
+    /* check if vdata exists in the file */
+    if (FAIL == vexistvs(vs->f, vs->oref))
         HGOTO_ERROR(DFE_NOVS, FAIL);
 
+    /* get write list */
     w = & vs->wlist;
     if (w->n == 0)
       {
@@ -442,10 +495,11 @@ VSwrite(int32 vkey, uint8 buf[], int32 nelt, int32 interlace)
           HGOTO_DONE(FAIL);
       }
 
+    /* check interlace of input buffer */
     if (interlace != NO_INTERLACE && interlace != FULL_INTERLACE)
         HGOTO_ERROR(DFE_ARGS, FAIL);
 
-    hdf_size = w->ivsize;   /* as stored in HDF file */
+    hdf_size = (intn)w->ivsize;   /* as stored in HDF file */
     total_bytes = hdf_size * nelt;
 
     /* make sure we have a valid AID */
@@ -465,30 +519,7 @@ VSwrite(int32 vkey, uint8 buf[], int32 nelt, int32 interlace)
      *  AND we are incresing its size
      */
     HQueryposition(vs->aid, &position);
-    new_size = (position / vs->wlist.ivsize) + nelt;
-
-#ifdef OLD_WAY
-    if ((vs->nvertices > 0) && (new_size > vs->nvertices))
-      {
-          int16       special;
-
-          HQueryspecial(vs->aid, &special);
-          if (!special)
-            {
-
-                int32       blk_size;
-
-                blk_size = ((new_size*vs->wlist.ivsize) > VDEFAULTBLKSIZE ? (new_size*vs->wlist.ivsize) : VDEFAULTBLKSIZE);
-
-                Hendaccess(vs->aid);
-                vs->aid = HLcreate(vs->f, VSDATATAG, vs->oref,
-                                   blk_size, VDEFAULTNBLKS);
-
-                /* seek back to correct point */
-                j = Hseek(vs->aid, position, DF_START);
-            }
-      }
-#endif /* OLD_WAY */
+    new_size = (position / (intn)vs->wlist.ivsize) + nelt;
 
     /* this should really be cached in the Vdata structure */
     for (int_size = 0, j = 0; j < w->n; j++)
@@ -544,9 +575,9 @@ VSwrite(int32 vkey, uint8 buf[], int32 nelt, int32 interlace)
                 chunk = buf_size / hdf_size + 1;
 
                 /* get a buffer big enough to hold the values */
-                Vtbufsize = chunk * hdf_size;
+                Vtbufsize = (size_t)chunk * (size_t)hdf_size;
                 if (Vtbuf)
-                    HDfree((VOIDP) Vtbuf);
+                    HDfree(Vtbuf);
                 if ((Vtbuf = (uint8 *) HDmalloc(Vtbufsize)) == NULL)
                     HGOTO_ERROR(DFE_NOSPACE, FAIL);
             }
@@ -575,11 +606,11 @@ VSwrite(int32 vkey, uint8 buf[], int32 nelt, int32 interlace)
                 for (j = 0; j < w->n; j++)
                   {
                       src = Src + offset;
-                      dest = Vtbuf + w->off[j];
-                      type = w->type[j];
-                      esize = w->esize[j];
-                      isize = w->isize[j];
-                      order = w->order[j];
+                      dest = Vtbuf + (size_t)w->off[j];
+                      type = (int32)w->type[j];
+                      esize = (intn)w->esize[j];
+                      isize = (intn)w->isize[j];
+                      order = (intn)w->order[j];
 
                       DFKsetNT(type);
                       for (index = 0; index < order; index++)
@@ -616,9 +647,9 @@ VSwrite(int32 vkey, uint8 buf[], int32 nelt, int32 interlace)
 	  /* alloc space (Vtbuf) for writing out the data */
 	  if (Vtbufsize < (uint32) total_bytes)
 	    {
-          Vtbufsize = total_bytes;
+          Vtbufsize = (uint32)total_bytes;
           if (Vtbuf)
-              HDfree((VOIDP) Vtbuf);
+              HDfree(Vtbuf);
           if ((Vtbuf = (uint8 *) HDmalloc(Vtbufsize)) == NULL)
               HGOTO_ERROR(DFE_NOSPACE, FAIL);
 	    }
@@ -631,11 +662,11 @@ VSwrite(int32 vkey, uint8 buf[], int32 nelt, int32 interlace)
 		src = buf;
 		for (j = 0; j < w->n; j++)
 		  {
-		      dest = Vtbuf + w->off[j];
-		      type = w->type[j];
-		      esize = w->esize[j];
-		      isize = w->isize[j];
-		      order = w->order[j];
+		      dest = Vtbuf + (size_t)w->off[j];
+		      type = (int32)w->type[j];
+		      esize = (intn)w->esize[j];
+		      isize = (intn)w->isize[j];
+		      order = (intn)w->order[j];
 
 		      DFKsetNT(type);
 		      for (index = 0; index < order; index++)
@@ -658,10 +689,10 @@ VSwrite(int32 vkey, uint8 buf[], int32 nelt, int32 interlace)
 		for (j = 0; j < w->n; j++)
 		  {
 		      dest = Vtbuf + w->off[j] * nelt;
-		      type = w->type[j];
-		      esize = w->esize[j];
-		      isize = w->isize[j];
-		      order = w->order[j];
+		      type = (int32)w->type[j];
+		      esize = (intn)w->esize[j];
+		      isize = (intn)w->isize[j];
+		      order = (intn)w->order[j];
 
 		      DFKsetNT(type);
 		      for (index = 0; index < order; index++)
@@ -684,10 +715,10 @@ VSwrite(int32 vkey, uint8 buf[], int32 nelt, int32 interlace)
 		  {
 		      src = buf + offset;
 		      dest = Vtbuf + w->off[j] * nelt;
-		      type = w->type[j];
-		      isize = w->isize[j];
-		      esize = w->esize[j];
-		      order = w->order[j];
+		      type = (int32)w->type[j];
+		      isize = (intn)w->isize[j];
+		      esize = (intn)w->esize[j];
+		      order = (intn)w->order[j];
 
 		      DFKsetNT(type);
 		      for (index = 0; index < order; index++)
@@ -727,4 +758,3 @@ done:
   return ret_value;
 }	/* VSwrite */
 
-/* ------------------------------------------------------------------ */
