@@ -11,10 +11,10 @@
  ****************************************************************************/
 
 #ifdef RCSID
-static char RcsId[] = "@(#)1.54";
+static char RcsId[] = "@(#)$Revision: 1.64 $";
 #endif
 
-/* vsfld.c,v 1.54 1996/06/21 05:14:22 acheng Exp */
+/* $Id: vsfld.c,v 1.64 1998/02/07 22:29:03 koziol Exp $ */
 
 /*****************************************************************************
 * vsetf.c
@@ -41,7 +41,8 @@ EXPORTED ROUTINES
 
 ************************************************************************/
 
-#include "vg.h"
+#define VSET_INTERFACE
+#include "hdf.h"
 #include <stdarg.h>
 
 /*
@@ -50,7 +51,7 @@ EXPORTED ROUTINES
    ** ==================================================================
    * */
 
-PRIVATE const SYMDEF rstab[] =
+static const SYMDEF rstab[] =
 {
     {"PX", DFNT_FLOAT32, SIZE_FLOAT32, 1},
     {"PY", DFNT_FLOAT32, SIZE_FLOAT32, 1},
@@ -116,137 +117,114 @@ VSsetfields(int32 vkey, const char *fields)
      * write to an empty vdata : set the write list but do not set the
      *   read list cuz there is nothing there to read yet...
      */
-    if (vs->access == 'w' && vs->nvertices == 0)
-      {
+    if ((vs->access == 'w') && (vs->nvertices == 0))
+       {
           wlist=&(vs->wlist);
-          wlist->ivsize = 0;
-          wlist->n = 0;
+          if (wlist->n == 0) /* fields not set yet, Sept. 96. */
+            /* do not re-set fields if they were already set. */
+          {
+              wlist->ivsize = 0;
+              wlist->n = 0;
 
           /* allocate space for the internal WRITELIST structures */
-          if((wlist->type=HDmalloc(sizeof(int16)*ac))==NULL)
-              HGOTO_ERROR(DFE_NOSPACE,FAIL);
-          if((wlist->isize=HDmalloc(sizeof(uint16)*ac))==NULL)
-            {
-              HDfree(wlist->type);
-              HGOTO_ERROR(DFE_NOSPACE,FAIL);
-            } /* end if */
-          if((wlist->off=HDmalloc(sizeof(uint16)*ac))==NULL)
-            {
-              HDfree(wlist->isize);
-              HDfree(wlist->type);
-              HGOTO_ERROR(DFE_NOSPACE,FAIL);
-            } /* end if */
-          if((wlist->order=HDmalloc(sizeof(uint16)*ac))==NULL)
-            {
-              HDfree(wlist->off);
-              HDfree(wlist->isize);
-              HDfree(wlist->type);
-              HGOTO_ERROR(DFE_NOSPACE,FAIL);
-            } /* end if */
-          if((wlist->name=HDmalloc(sizeof(char *)*ac))==NULL)
-            {
-              HDfree(wlist->order);
-              HDfree(wlist->off);
-              HDfree(wlist->isize);
-              HDfree(wlist->type);
-              HGOTO_ERROR(DFE_NOSPACE,FAIL);
-            } /* end if */
-          if((wlist->esize=HDmalloc(sizeof(uint16)*ac))==NULL)
-            {
-              HDfree(wlist->name);
-              HDfree(wlist->order);
-              HDfree(wlist->off);
-              HDfree(wlist->isize);
-              HDfree(wlist->type);
-              HGOTO_ERROR(DFE_NOSPACE,FAIL);
-            } /* end if */
+              /* Allocate buffer to hold all the int16/uint16 arrays */
+              if((wlist->bptr=HDmalloc(sizeof(uint16)*(size_t)(ac*5)))==NULL)
+                  HGOTO_ERROR(DFE_NOSPACE,FAIL);
 
-          for (i = 0; i < ac; i++)
-            {
-                found = FALSE;
+              /* Use buffer to support the other arrays */
+              wlist->type=(int16 *)wlist->bptr;
+              wlist->off=(uint16 *)wlist->type+ac;
+              wlist->isize=wlist->off+ac;
+              wlist->order=wlist->isize+ac;
+              wlist->esize=wlist->order+ac;
+              if((wlist->name=HDmalloc(sizeof(char *)*(size_t)ac))==NULL)
+                {
+                  HDfree(wlist->bptr);
+                  HGOTO_ERROR(DFE_NOSPACE,FAIL);
+                } /* end if */
+
+              for (i = 0; i < ac; i++)
+                {
+                    found = FALSE;
                 /* --- first look in the user's symbol table --- */
-                for (j = 0; j < vs->nusym; j++)
-                    if (!HDstrcmp(av[i], vs->usym[j].name))
-                      {
-                          found = TRUE;
-
+                    for (j = 0; j < vs->nusym; j++)
+                        if (!HDstrcmp(av[i], vs->usym[j].name))
+                          {
+                              found = TRUE;
                             
-                          if((wlist->name[wlist->n]=HDstrdup(vs->usym[j].name))==NULL)
-                            {
-                              HDfree(wlist->esize);
-                              HDfree(wlist->name);
-                              HDfree(wlist->order);
-                              HDfree(wlist->off);
-                              HDfree(wlist->isize);
-                              HDfree(wlist->type);
-                              HGOTO_ERROR(DFE_NOSPACE,FAIL);
-                            } /* end if */
-                          order = vs->usym[j].order;
-                          wlist->type[wlist->n] = vs->usym[j].type;
-                          wlist->order[wlist->n] = order;
+                              if((wlist->name[wlist->n]=HDstrdup(vs->usym[j].name))==NULL)
+                               {
+                                  HDfree(wlist->name);
+                                  HDfree(wlist->bptr);
+                                  HGOTO_ERROR(DFE_NOSPACE,FAIL);
+                                } /* end if */
+                              order = vs->usym[j].order;
+                              wlist->type[wlist->n] = vs->usym[j].type;
+                              wlist->order[wlist->n] = order;
 
-                          value = order * DFKNTsize(vs->usym[j].type | DFNT_NATIVE);
-/* should be ok if field size in memory > MAX_FIELD_SIZE. ( 65535 after 40r1p1)  
-                          if (value > MAX_FIELD_SIZE)
-                              HGOTO_ERROR(DFE_BADFIELDS, FAIL);
-*/
-                          wlist->esize[wlist->n] = (uint16) value;
+                              value = order * DFKNTsize(vs->usym[j].type | DFNT_NATIVE);
+                              if (value == FAIL)
+                                  HGOTO_ERROR(DFE_BADFIELDS, FAIL);
+                              wlist->esize[wlist->n] = (uint16) value;
 
-                          value = order * vs->usym[j].isize;
-                          if (value > MAX_FIELD_SIZE)
-                              HGOTO_ERROR(DFE_BADFIELDS, FAIL);
-                          wlist->isize[wlist->n] = (uint16) value;
+                              value = order * vs->usym[j].isize;
+                              if (value > MAX_FIELD_SIZE)
+                                  HGOTO_ERROR(DFE_BADFIELDS, FAIL);
+                              wlist->isize[wlist->n] = (uint16) value;
 
-                          value = (int32) wlist->ivsize + (int32) (wlist->isize[wlist->n]);
-                          if (value > MAX_FIELD_SIZE)
-                              HGOTO_ERROR(DFE_BADFIELDS, FAIL);
-                          wlist->ivsize = (uint16) value;
+                              value = (int32) wlist->ivsize + (int32) (wlist->isize[wlist->n]);
+                              if (value > MAX_FIELD_SIZE)
+                                  HGOTO_ERROR(DFE_BADFIELDS, FAIL);
+                              wlist->ivsize = (uint16) value;
 
-                          wlist->n++;
-                          break;
-                      }
+                              wlist->n++;
+                              break;
+                          }
 
                 /* --- now look in the reserved symbol table --- */
-                if (!found)
-                  {
-                      for (j = 0; j < NRESERVED; j++)
-                          if (!HDstrcmp(av[i], rstab[j].name))
-                            {
-                                found = TRUE;
+                    if (!found)
+                      {
+                          for (j = 0; j < (intn)NRESERVED; j++)
+                              if (!HDstrcmp(av[i], rstab[j].name))
+                                {
+                                    found = TRUE;
 
-                                if((wlist->name[wlist->n]=HDstrdup(rstab[j].name))==NULL)
-                                  {
-                                    HDfree(wlist->esize);
-                                    HDfree(wlist->name);
-                                    HDfree(wlist->order);
-                                    HDfree(wlist->off);
-                                    HDfree(wlist->isize);
-                                    HDfree(wlist->type);
-                                    HGOTO_ERROR(DFE_NOSPACE,FAIL);
-                                  } /* end if */
-                                order = rstab[j].order;
-                                wlist->type[wlist->n] = rstab[j].type;
-                                wlist->order[wlist->n] = order;
-                                wlist->esize[wlist->n] = (uint16) (order * DFKNTsize((int32) rstab[j].type | DFNT_NATIVE));
-                                wlist->isize[wlist->n] = (uint16) (order * rstab[j].isize);
-                                wlist->ivsize += (uint16) (wlist->isize[wlist->n]);
-                                wlist->n++;
-                                break;
-                            }
-                  }
-                if (!found)     /* field is not a defined field - error  */
-                    HGOTO_ERROR(DFE_BADFIELDS, FAIL);
-            }
+                                    if((wlist->name[wlist->n]=HDstrdup(rstab[j].name))==NULL)
+                                      {
+                                        HDfree(wlist->name);
+                                        HDfree(wlist->bptr);
+                                        HGOTO_ERROR(DFE_NOSPACE,FAIL);
+                                      } /* end if */
+                                    order = rstab[j].order;
+                                    wlist->type[wlist->n] = rstab[j].type;
+                                    wlist->order[wlist->n] = order;
+                                    value = order * DFKNTsize(rstab[j].type | DFNT_NATIVE);
+                                    if (value == FAIL)
+                                      HGOTO_ERROR(DFE_BADFIELDS, FAIL);
+                                    wlist->esize[wlist->n] = (uint16) value;
+                                    wlist->isize[wlist->n] = (uint16) (order * rstab[j].isize);
+                                    wlist->ivsize += (uint16) (wlist->isize[wlist->n]);
+                                    wlist->n++;
+                                    break;
+                                }
+                      }
+                    if (!found)     /* field is not a defined field - error  */
+                        HGOTO_ERROR(DFE_BADFIELDS, FAIL);
+                }
 
           /* *********************************************************** */
           /* compute and save the fields' offsets */
-          for (uj = 0, i = 0; i < wlist->n; i++)
-            {
-                wlist->off[i] = (uint16) uj;
-                uj += wlist->isize[i];
-            }
+               for (uj = 0, i = 0; i < wlist->n; i++)
+                {
+                    wlist->off[i] = (uint16) uj;
+                    uj += wlist->isize[i];
+                }
 
-          HGOTO_DONE(SUCCEED); /* OK */
+              vs->marked = TRUE; /* mark vdata as being modified */
+              vs->new_h_sz = TRUE; /* mark vdata header size being changed */
+
+              HGOTO_DONE(SUCCEED); /* OK */
+          } /* if wlist->n == 0 */
       }     /* writing to empty vdata */
 
     /*
@@ -262,7 +240,7 @@ VSsetfields(int32 vkey, const char *fields)
           rlist->item=NULL;
 
           /* Allocate enough space for the read list */
-          if((rlist->item=(intn *)HDmalloc(sizeof(intn)*(ac)))==NULL)
+          if((rlist->item=(intn *)HDmalloc(sizeof(intn)*(size_t)(ac)))==NULL)
               HGOTO_ERROR(DFE_NOSPACE, FAIL);
           for (i = 0; i < ac; i++)
             {
@@ -307,7 +285,8 @@ VSfdefine(int32 vkey, const char *field, int32 localtype, int32 order)
 {
     char      **av;
     int32       ac;
-    int16       isize, usymid, replacesym;
+    int16       isize, replacesym;
+    intn        usymid;
     intn j;
     vsinstance_t *w;
     VDATA      *vs;
@@ -333,7 +312,7 @@ VSfdefine(int32 vkey, const char *field, int32 localtype, int32 order)
     if (order < 1 || order > MAX_ORDER)
         HGOTO_ERROR(DFE_BADORDER, FAIL);
     /* don't forget to check for field size limit */
-    isize = DFKNTsize(localtype);
+    isize = (int16)DFKNTsize(localtype);
     if ( (isize == FAIL) || (isize * order) > MAX_FIELD_SIZE )
         HGOTO_ERROR(DFE_BADFIELDS, FAIL);
 
@@ -363,21 +342,21 @@ VSfdefine(int32 vkey, const char *field, int32 localtype, int32 order)
           }
 
     if (replacesym)
-        usymid = (int16) j;     /* new definition will replace old at this index */
+        usymid = j;     /* new definition will replace old at this index */
     else
       {
           SYMDEF *tmp_sym=vs->usym;  /* temp. pointer to the new symdef list */
 
-          usymid = vs->nusym;
+          usymid = (intn)vs->nusym;
           /* use temporary pointer in case we run out of memory, so we don't loose original list */
           if (tmp_sym == NULL)
            {
-             if((tmp_sym=(SYMDEF *) HDmalloc(sizeof(SYMDEF)*(usymid+1)))==NULL)
+             if((tmp_sym=(SYMDEF *) HDmalloc(sizeof(SYMDEF)*(size_t)(usymid+1)))==NULL)
               HGOTO_ERROR(DFE_NOSPACE,FAIL);
             }
           else
             {
-              if((tmp_sym=(SYMDEF *) HDrealloc((VOIDP)tmp_sym,sizeof(SYMDEF)*(usymid+1)))==NULL)
+              if((tmp_sym=(SYMDEF *) HDrealloc(tmp_sym,sizeof(SYMDEF)*(size_t)(usymid+1)))==NULL)
                   HGOTO_ERROR(DFE_NOSPACE,FAIL);
             }
           vs->usym=tmp_sym;
@@ -485,6 +464,9 @@ VFfieldname(int32 vkey, int32 index)
     if ((vs == NULL) || (vs->otag != VSDESCTAG))
       HGOTO_ERROR(DFE_ARGS,NULL);
 
+    if (vs->wlist.n == 0)
+      HGOTO_ERROR(DFE_BADFIELDS,NULL);
+
     ret_value = ((char *) vs->wlist.name[index]);
 
 done:
@@ -530,6 +512,9 @@ VFfieldtype(int32 vkey, int32 index)
     if ((vs == NULL) || (vs->otag != VSDESCTAG))
       HGOTO_ERROR(DFE_ARGS,FAIL);
 
+    if (vs->wlist.n == 0)
+      HGOTO_ERROR(DFE_BADFIELDS,FAIL);
+
     ret_value = ((int32) vs->wlist.type[index]);
 
 done:
@@ -576,6 +561,9 @@ VFfieldisize(int32 vkey, int32 index)
     if ((vs == NULL) || (vs->otag != VSDESCTAG))
       HGOTO_ERROR(DFE_ARGS,FAIL);
 
+    if (vs->wlist.n == 0)
+      HGOTO_ERROR(DFE_BADFIELDS,FAIL);
+
     ret_value = ((int32) vs->wlist.isize[index]);
 
 done:
@@ -622,6 +610,9 @@ VFfieldesize(int32 vkey, int32 index)
     if ((vs == NULL) || (vs->otag != VSDESCTAG))
       HGOTO_ERROR(DFE_ARGS,FAIL);
 
+    if (vs->wlist.n == 0)
+      HGOTO_ERROR(DFE_BADFIELDS,FAIL);
+
     ret_value = ((int32) vs->wlist.esize[index]);
 
 done:
@@ -667,6 +658,9 @@ VFfieldorder(int32 vkey, int32 index)
     if ((vs == NULL) || (vs->otag != VSDESCTAG))
       HGOTO_ERROR(DFE_ARGS,FAIL);
 
+    if (vs->wlist.n == 0)
+      HGOTO_ERROR(DFE_BADFIELDS,FAIL);
+
     ret_value = ((int32) vs->wlist.order[index]);
 
 done:
@@ -733,7 +727,7 @@ intn VSsetexternalfile(int32 vkey, const char *filename, int32 offset)
     if (vs->access != 'w')
         HGOTO_ERROR(DFE_BADACC, FAIL);
 
-    if (-1L == vexistvs(vs->f, vs->oref))
+    if (FAIL == vexistvs(vs->f, vs->oref))
         HGOTO_ERROR(DFE_NOVS, FAIL);
 
     if(!w->ref)
@@ -763,13 +757,14 @@ done:
 
     return ret_value;
 } /* VSsetexternalfile */
+
 /*----------------------------------------------------------------- 
 NAME
     VSfpack -- pack into or unpack from a buf the values of fully
               interlaced fields.
 USAGE
     intn VSfpack(int32 vsid, intn packtype, char *fields_in_buf,
-         VOIDP buf, intn bufsz, intn n_records, char *fields, VOIDP fldbufpt[])
+         void * buf, intn bufsz, intn n_records, char *fields, void * fldbufpt[])
     int32 vsid; IN: vdata id.
     intn packtype; IN: 
          _HDF_VSPACK(0) -- pack field values into vdata buf;
@@ -777,14 +772,14 @@ USAGE
     char *fields_in_buf; IN: 
          fields in buf to write to or read from vdata. NULL 
          stands for all fields in the vdata.
-    VOIDP buf; IN: buffer for vdata values.
+    void * buf; IN: buffer for vdata values.
     intn bufsz; IN: buf size in byte.
     intn n_records; IN: number of records to pack or unpack.
     char *fields; IN: 
          names of the fields to be pack/unpack. It may be a 
          subset of the fields_in_buf. NULL stands for all 
          fields in buf. 
-    VOIDP fldbufpt[]; IN: array of pointers to field buffers.
+    void * fldbufpt[]; IN: array of pointers to field buffers.
 RETURNS
     SUCCEED(0) on success; FIAL(-1) otherwise.
 DESCRIPTION
@@ -802,12 +797,11 @@ DESCRIPTION
 
 /*---------------------------------------------------------*/
 
-intn VSfpack(int32 vsid, intn packtype, char *fields_in_buf,
-         VOIDP buf, intn bufsz, intn n_records, char *fields, VOIDP fldbufpt[])
+intn VSfpack(int32 vsid, intn packtype, const char *fields_in_buf,
+         void * buf, intn bufsz, intn n_records, const char *fields, void * fldbufpt[])
 {
     CONSTR(FUNC, "VSfpack");
 
-    va_list ap;
     int32 ac;
     char **av, *s;
     uint8 *bufp = (uint8 *)buf;
@@ -846,8 +840,8 @@ intn VSfpack(int32 vsid, intn packtype, char *fields_in_buf,
             HGOTO_ERROR(DFE_ARGS, FAIL);
     } 
     blist.n = ac;
-    blist.idx = (int32 *)HDmalloc(ac * sizeof(int32));
-    blist.offs = (int32 *)HDmalloc(ac * sizeof(int32));
+    blist.idx = (int32 *)HDmalloc((size_t)ac * sizeof(int32));
+    blist.offs = (int32 *)HDmalloc((size_t)ac * sizeof(int32));
     if ((blist.idx == NULL) || (blist.offs == NULL))
          HGOTO_ERROR(DFE_NOSPACE, FAIL);
       /* fill arrays blist.msizes and blist.offs; calculate
@@ -902,11 +896,11 @@ intn VSfpack(int32 vsid, intn packtype, char *fields_in_buf,
     else 
         ac = blist.n;
        /* fill array of fmsizes, foffs, fbufps */
-    if ((fmsizes=(int32 *)HDmalloc(ac*sizeof(int32))) == NULL) 
+    if ((fmsizes=(int32 *)HDmalloc((size_t)ac*sizeof(int32))) == NULL) 
          HGOTO_ERROR(DFE_NOSPACE, FAIL);
-    if ((foffs = (int32 *)HDmalloc(ac*sizeof(int32))) == NULL) 
+    if ((foffs = (int32 *)HDmalloc((size_t)ac*sizeof(int32))) == NULL) 
          HGOTO_ERROR(DFE_NOSPACE, FAIL);
-    if ((fbufps=(uint8 **)HDmalloc(ac*sizeof(uint8 *))) == NULL) 
+    if ((fbufps=(uint8 **)HDmalloc((size_t)ac*sizeof(uint8 *))) == NULL) 
          HGOTO_ERROR(DFE_NOSPACE, FAIL);
     if (fields != NULL)  { /* a subset of buf fields */
         for (i=0; i<ac; i++) {
@@ -930,7 +924,7 @@ intn VSfpack(int32 vsid, intn packtype, char *fields_in_buf,
             }     /* for */
             if (!found)
                 HGOTO_ERROR(DFE_BADFIELDS, FAIL); 
-            fmsizes[i] = w->esize[blist.idx[j]];
+            fmsizes[i] = (int32)w->esize[blist.idx[j]];
             foffs[i] = blist.offs[j];
 	    fbufps[i] = fldbufpt[i];
             if (fbufps[i] == NULL)  
@@ -940,7 +934,7 @@ intn VSfpack(int32 vsid, intn packtype, char *fields_in_buf,
     else
     {     /* all buf fields */
         for (i=0; i < ac; i++)   {
-            fmsizes[i] = w->esize[blist.idx[i]];
+            fmsizes[i] = (int32)w->esize[blist.idx[i]];
             foffs[i] = blist.offs[i];
 	    fbufps[i] = fldbufpt[i];
             if (fbufps[i] == NULL)  
@@ -982,7 +976,7 @@ done:
        HDfree(fbufps);
 
 #ifdef HAVE_PABLO
-  TRACE_OFF(VS_mask, ID_VSfpact)
+  TRACE_OFF(VS_mask, ID_VSfpack);
 #endif /* HAVE_PABLO */
 
   return ret_value;
