@@ -10,10 +10,11 @@
  *                                                                          *
  ****************************************************************************/
 
-/* fortest.h,v 1.2 1996/06/22 23:07:26 acheng Exp */
+/* $Id: fortest.h,v 1.3 1997/05/22 23:36:55 sxu Exp $ */
 
 #ifndef __FORTEST_H
 #define __FORTEST_H
+#include "hdf.h"
 
 /* Verbosity Environment Variable */
 #define FOR_VERB    "HDF_FOR_VERBOSITY"
