@@ -11,7 +11,7 @@
  ****************************************************************************/
 
 #ifdef RCSID
-static char RcsId[] = "slab.c,v 1.14 1996/05/16 18:42:59 koziol Exp";
+static char RcsId[] = "$Id: slab.c,v 1.18 1997/10/25 00:56:23 koziol Exp $";
 #endif
 
 /* $Id */
@@ -24,8 +24,7 @@ static char RcsId[] = "slab.c,v 1.14 1996/05/16 18:42:59 koziol Exp";
 ***************************************************************************/
 /* slabwin & slabwuin do not work for all machines */
 
-#include "hdf.h"
-#include "tutils.h"
+#include "tproto.h"
 
 /* Internal variables */
 static int32  rank = 3;
@@ -38,15 +37,13 @@ static float32  maxf32 = (float32) 123.0;
 static float32  minf32 = (float32) -1.0;
 static float32  fillf32 = (float32) 1.0;
 
-#if !(defined CRAYMPP)
-static intn  maxin = 123;
-static intn  minin = -1;
-static intn  fillin = 1;
+static int32  maxin = 123;
+static int32  minin = -1;
+static int32  fillin = 1;
 
-static uintn  maxuin = 123;
-static uintn  minuin = 2;
-static uintn  filluin = 1;
-#endif
+static uint32  maxuin = 123;
+static uint32  minuin = 2;
+static uint32  filluin = 1;
 
 static int32  maxi32 = 123;
 static int32  mini32 = -1;
@@ -108,21 +105,19 @@ static float32  scrowf32[3] =
 static float32  sccolf32[4] =
 {(float32) 0.0, (float32) 1.0, (float32) 2.0, (float32) 3.0};
 
-#if !(defined CRAYMPP)
-static intn  scplnin[2] =
+static int32  scplnin[2] =
 {0, 100};
-static intn  scrowin[3] =
+static int32  scrowin[3] =
 {0, 10, 20};
-static intn  sccolin[4] =
+static int32  sccolin[4] =
 {0, 1, 2, 3};
 
-static uintn  scplnuin[2] =
+static uint32  scplnuin[2] =
 {0, 100};
-static uintn  scrowuin[3] =
+static uint32  scrowuin[3] =
 {0, 10, 20};
-static uintn  sccoluin[4] =
+static uint32  sccoluin[4] =
 {0, 1, 2, 3};
-#endif
 
 static int32  scplni32[2] =
 {0, 100};
@@ -227,27 +222,26 @@ static float64  slabw5f64[2][3][1] =
         {113.0},
         {123.0}}};
 
-#if !(defined CRAYMPP)
-static intn  slabw1in[1][1][3] =
+static int32  slabw1in[1][1][3] =
 {
     {
         {110, 111, 112}}};
-static intn  slabw2in[2][1][3] =
+static int32  slabw2in[2][1][3] =
 {
     {
         {20, 21, 22}},
     {
         {120, 121, 122}}};
-static intn  slabw3in[1][2][3] =
+static int32  slabw3in[1][2][3] =
 {
     {
         {0, 1, 2},
         {10, 11, 12}}};
-static intn  slabw4in[1][1][3] =
+static int32  slabw4in[1][1][3] =
 {
     {
         {100, 101, 102}}};
-static intn  slabw5in[2][3][1] =
+static int32  slabw5in[2][3][1] =
 {
     {
         {3},
@@ -258,26 +252,26 @@ static intn  slabw5in[2][3][1] =
         {113},
         {123}}};
 
-static uintn  slabw1uin[1][1][3] =
+static uint32  slabw1uin[1][1][3] =
 {
     {
         {110, 111, 112}}};
-static uintn  slabw2uin[2][1][3] =
+static uint32  slabw2uin[2][1][3] =
 {
     {
         {20, 21, 22}},
     {
         {120, 121, 122}}};
-static uintn  slabw3uin[1][2][3] =
+static uint32  slabw3uin[1][2][3] =
 {
     {
         {0, 1, 2},
         {10, 11, 12}}};
-static uintn  slabw4uin[1][1][3] =
+static uint32  slabw4uin[1][1][3] =
 {
     {
         {100, 101, 102}}};
-static uintn  slabw5uin[2][3][1] =
+static uint32  slabw5uin[2][3][1] =
 {
     {
         {3},
@@ -287,7 +281,6 @@ static uintn  slabw5uin[2][3][1] =
         {103},
         {113},
         {123}}};
-#endif
 
 static int32  slabw1i32[1][1][3] =
 {
@@ -587,8 +580,7 @@ static float64  f64data[2][3][4] =
         {100.0, 101.0, 102.0, 103.0},
         {110.0, 111.0, 112.0, 113.0},
         {120.0, 121.0, 122.0, 123.0}}};
-#if !(defined CRAYMPP)
-static intn  indata[2][3][4] =
+static int32  indata[2][3][4] =
 {
     {
         {0, 1, 2, 3},
@@ -598,7 +590,7 @@ static intn  indata[2][3][4] =
         {100, 101, 102, 103},
         {110, 111, 112, 113},
         {120, 121, 122, 123}}};
-static uintn  uindata[2][3][4] =
+static uint32  uindata[2][3][4] =
 {
     {
         {0, 1, 2, 3},
@@ -608,7 +600,6 @@ static uintn  uindata[2][3][4] =
         {100, 101, 102, 103},
         {110, 111, 112, 113},
         {120, 121, 122, 123}}};
-#endif
 static int32  i32data[2][3][4] =
 {
     {
@@ -686,10 +677,8 @@ static const char  *sw4 = "s4w.hdf";
 
 static int slabwf32(void);
 static int slabwf64(void);
-#if !(defined CRAYMPP)
 static int slabwin(void);
 static int slabwuin(void);
-#endif
 static int slabwi32(void);
 static int slabwui32(void);
 static int slabwi16(void);
@@ -711,7 +700,7 @@ slabwf32(void)
     int32       ret = 0;
     int32       num_err = 0;
     float32     sdata[2][3][4]; /* Data array read from from file */
-    float32     lfill;
+    float32     lfill=(float32)0.0;
 
     MESSAGE(10, printf("\n slabwf32:  Writing 5 slabs to slabwf32.hdf \n");
         );
@@ -720,7 +709,7 @@ slabwf32(void)
     CHECK(ret, FAIL, "DFSDclear");
 
     /* First set dimensions */
-    ret = DFSDsetdims((int16) rank, size_dims);
+    ret = DFSDsetdims((intn) rank, size_dims);
     CHECK(ret, FAIL, "DFSDsetdims");
 
     /* Set dimension strings */
@@ -865,7 +854,7 @@ slabwf64(void)
     int32       ret = 0;
     int32       num_err = 0;
     float64     sdata[2][3][4]; /* Data array read from from file */
-    float64     lfill;
+    float64     lfill=(float64)0.0;
 
     MESSAGE(10, printf("\n slabwf64:  Writing 5 slabs to slabwf64.hdf \n");
         );
@@ -877,7 +866,7 @@ slabwf64(void)
     CHECK(ret, FAIL, "DFSDsetNT");
 
     /* First set dimensions */
-    ret = DFSDsetdims((int16) rank, size_dims);
+    ret = DFSDsetdims((intn) rank, size_dims);
     CHECK(ret, FAIL, "DFSDsetdims");
 
     /* Set dimension strings */
@@ -1012,9 +1001,8 @@ slabwf64(void)
     return (int) num_err;
 }
 
-#if !(defined CRAYMPP)
 /*
-   ** Write intn data set to slabwin.hdf as 5 hyperslabs.
+   ** Write int32 data set as native data type to slabwin.hdf as 5 hyperslabs.
  */
 static int
 slabwin(void)
@@ -1022,8 +1010,8 @@ slabwin(void)
     int32       i, j, k;
     int32       ret = 0;
     int32       num_err = 0;
-    intn        sdata[2][3][4]; /* Data array read from from file */
-    intn        lfill;
+    int32        sdata[2][3][4]; /* Data array read from from file */
+    int32        lfill=0;
 
     MESSAGE(10, printf("\n slabwin:  Writing 5 slabs to slabwin.hdf \n");
         );
@@ -1035,7 +1023,7 @@ slabwin(void)
     CHECK(ret, FAIL, "DFSDsetNT");
 
     /* First set dimensions */
-    ret = DFSDsetdims((int16) rank, size_dims);
+    ret = DFSDsetdims((intn) rank, size_dims);
     CHECK(ret, FAIL, "DFSDsetdims");
 
     /* Set dimension strings */
@@ -1140,7 +1128,7 @@ slabwin(void)
     CHECK(ret, FAIL, "DFSDgetfillvalue");
     if (lfill != fillin)
         num_err += 1;
-    MESSAGE(10, printf("\n       fill value =: %d \n", lfill);
+    MESSAGE(10, printf("\n       fill value =: %d \n", (int)lfill);
         );
 
     if (num_err != 0)
@@ -1157,7 +1145,7 @@ slabwin(void)
               {
                   if (sdata[i][j][k] != indata[i][j][k])
                       num_err++;
-                  MESSAGE(10, printf("%u, ", sdata[i][j][k]);
+                  MESSAGE(10, printf("%u, ", (unsigned)sdata[i][j][k]);
                       );
               }
     if (num_err == 0)
@@ -1171,7 +1159,7 @@ slabwin(void)
 }
 
 /*
-   ** Write uintn data set to slabwuin.hdf as 5 hyperslabs.
+   ** Write uint32 data set as native data type to slabwuin.hdf as 5 hyperslabs.
  */
 static int
 slabwuin(void)
@@ -1179,8 +1167,8 @@ slabwuin(void)
     int32       i, j, k;
     int32       ret = 0;
     int32       num_err = 0;
-    uintn       sdata[2][3][4]; /* Data array read from from file */
-    uintn       lfill;
+    uint32       sdata[2][3][4]; /* Data array read from from file */
+    uint32       lfill=0;
 
     MESSAGE(10, printf("\n slabwuin:  Writing 5 slabs to slabwuin.hdf \n");
         );
@@ -1192,7 +1180,7 @@ slabwuin(void)
     CHECK(ret, FAIL, "DFSDsetNT");
 
     /* First set dimensions */
-    ret = DFSDsetdims((int16) rank, size_dims);
+    ret = DFSDsetdims((intn) rank, size_dims);
     CHECK(ret, FAIL, "DFSDsetdims");
 
     /* Set dimension strings */
@@ -1297,7 +1285,7 @@ slabwuin(void)
     CHECK(ret, FAIL, "DFSDgetfillvalue");
     if (lfill != filluin)
         num_err += 1;
-    MESSAGE(10, printf("\n       fill value =: %u \n", lfill);
+    MESSAGE(10, printf("\n       fill value =: %u \n", (unsigned)lfill);
         );
 
     if (num_err != 0)
@@ -1314,7 +1302,7 @@ slabwuin(void)
               {
                   if (sdata[i][j][k] != uindata[i][j][k])
                       num_err++;
-                  MESSAGE(10, printf("%u, ", sdata[i][j][k]);
+                  MESSAGE(10, printf("%u, ", (unsigned)sdata[i][j][k]);
                       );
               }
     if (num_err == 0)
@@ -1326,7 +1314,6 @@ slabwuin(void)
 
     return (int) num_err;
 }
-#endif
 
 /*
    ** Write int32 data set to slabwi32.hdf as 5 hyperslabs.
@@ -1338,7 +1325,7 @@ slabwi32(void)
     int32       ret = 0;
     int32       num_err = 0;
     int32       sdata[2][3][4]; /* Data array read from from file */
-    int32       lfill;
+    int32       lfill=(int32)0;
 
     MESSAGE(10, printf("\n slabwi32:  Writing 5 slabs to slabwi32.hdf \n");
         );
@@ -1350,7 +1337,7 @@ slabwi32(void)
     CHECK(ret, FAIL, "DFSDsetNT");
 
     /* First set dimensions */
-    ret = DFSDsetdims((int16) rank, size_dims);
+    ret = DFSDsetdims((intn) rank, size_dims);
     CHECK(ret, FAIL, "DFSDsetdims");
 
     /* Set dimension strings */
@@ -1495,7 +1482,7 @@ slabwui32(void)
     int32       ret = 0;
     int32       num_err = 0;
     uint32      sdata[2][3][4]; /* Data array read from from file */
-    uint32      lfill;
+    uint32      lfill=(uint32)0;
 
     MESSAGE(10, printf("\n slabwui32:  Writing 5 slabs to slabwui32.hdf \n");
         );
@@ -1507,7 +1494,7 @@ slabwui32(void)
     CHECK(ret, FAIL, "DFSDsetNT");
 
     /* First set dimensions */
-    ret = DFSDsetdims((int16) rank, size_dims);
+    ret = DFSDsetdims((intn) rank, size_dims);
     CHECK(ret, FAIL, "DFSDsetdims");
 
     /* Set dimension strings */
@@ -1652,7 +1639,7 @@ slabwi16(void)
     int32       ret = 0;
     int32       num_err = 0;
     int16       sdata[2][3][4]; /* Data array read from from file */
-    int16       lfill;
+    int16       lfill=(int16)0;
 
     MESSAGE(10, printf("\n slabwi16:  Writing 5 slabs to slabwi16.hdf \n");
         );
@@ -1664,7 +1651,7 @@ slabwi16(void)
     CHECK(ret, FAIL, "DFSDsetNT");
 
     /* First set dimensions */
-    ret = DFSDsetdims((int16) rank, size_dims);
+    ret = DFSDsetdims((intn) rank, size_dims);
     CHECK(ret, FAIL, "DFSDsetdims");
 
     /* Set dimension strings */
@@ -1809,7 +1796,7 @@ slabwui16(void)
     int32       ret = 0;
     int32       num_err = 0;
     uint16      sdata[2][3][4]; /* Data array read from from file */
-    uint16      lfill;
+    uint16      lfill=(uint16)0;
 
     MESSAGE(10, printf("\n slabwui16:  Writing 5 slabs to slabwui16.hdf \n");
         );
@@ -1821,7 +1808,7 @@ slabwui16(void)
     CHECK(ret, FAIL, "DFSDsetNT");
 
     /* First set dimensions */
-    ret = DFSDsetdims((int16) rank, size_dims);
+    ret = DFSDsetdims((intn) rank, size_dims);
     CHECK(ret, FAIL, "DFSDsetdims");
 
     /* Set dimension strings */
@@ -1966,7 +1953,7 @@ slabwi8(void)
     int32       ret = 0;
     int32       num_err = 0;
     int8        sdata[2][3][4]; /* Data array read from from file */
-    int8        lfill;
+    int8        lfill=(int8)0;
 
     MESSAGE(10, printf("\n slabwi8:  Writing 5 slabs to slabwi8.hdf \n");
         );
@@ -1978,7 +1965,7 @@ slabwi8(void)
     CHECK(ret, FAIL, "DFSDsetNT");
 
     /* First set dimensions */
-    ret = DFSDsetdims((int16) rank, size_dims);
+    ret = DFSDsetdims((intn) rank, size_dims);
     CHECK(ret, FAIL, "DFSDsetdims");
 
     /* Set dimension strings */
@@ -2123,7 +2110,7 @@ slabwui8(void)
     int32       ret = 0;
     int32       num_err = 0;
     uint8       sdata[2][3][4]; /* Data array read from from file */
-    uint8       lfill;
+    uint8       lfill=(uint8)0;
 
     MESSAGE(10, printf("\n slabwui8:  Writing 5 slabs to slabwui8.hdf \n");
         );
@@ -2135,7 +2122,7 @@ slabwui8(void)
     CHECK(ret, FAIL, "DFSDsetNT");
 
     /* First set dimensions */
-    ret = DFSDsetdims((int16) rank, size_dims);
+    ret = DFSDsetdims((intn) rank, size_dims);
     CHECK(ret, FAIL, "DFSDsetdims");
 
     /* Set dimension strings */
@@ -2284,7 +2271,7 @@ slab1w(void)
     CHECK(ret, FAIL, "DFSDclear");
 
     /* First set dimensions */
-    ret = DFSDsetdims((int16) rank, size_dims);
+    ret = DFSDsetdims((intn) rank, size_dims);
     CHECK(ret, FAIL, "DFSDsetdims");
 
     /* Set dimension strings */
@@ -2371,7 +2358,7 @@ slab2w(void)
     int32       ret = 0;
     int32       num_err = 0;
     float32     sdata[2][3][4]; /* Data array read from from file */
-    float32     lfill;
+    float32     lfill=(float32)0.0;
     intn        trank;
 
     MESSAGE(10, printf("\n slab2w:  Writing the last 2 of 5 slabs to slab1w.hdf \n");
@@ -2474,7 +2461,7 @@ slab3w(void)
     CHECK(ret, FAIL, "DFSDclear");
 
     /* First set dimensions */
-    ret = DFSDsetdims((int16) rank, size_dims);
+    ret = DFSDsetdims((intn) rank, size_dims);
     CHECK(ret, FAIL, "DFSDsetdims");
 
     /* Set dimension strings */
@@ -2783,7 +2770,7 @@ slab4w(void)
     CHECK(ret, FAIL, "DFSDclear");
 
     /* First set dimensions */
-    ret = DFSDsetdims((int16) rank, size_dims);
+    ret = DFSDsetdims((intn) rank, size_dims);
     CHECK(ret, FAIL, "DFSDsetdims");
 
     /* Set dimension strings */
@@ -2871,13 +2858,8 @@ test_slab(void)
 {
     num_errs += slabwf32();
     num_errs += slabwf64();
-#if !(defined CRAYMPP)
     num_errs += slabwin();
     num_errs += slabwuin();
-#else
-    printf("    Skipping slabwin\n");
-    printf("    Skipping slabuwin\n");
-#endif
     num_errs += slabwi32();
     num_errs += slabwui32();
     num_errs += slabwi16();
