@@ -11,35 +11,36 @@
  ****************************************************************************/
 
 #ifdef RCSID
-static char RcsId[] = "@(#)1.26";
+static char RcsId[] = "@(#)$Revision: 1.31 $";
 #endif
 
-/* he_file.c,v 1.26 1996/06/24 21:37:29 acheng Exp */
+/* $Id: he_file.c,v 1.31 1998/02/02 21:46:09 smitchel Exp $ */
 
 /* --- he-file.c  --- file and annotation manipulation routines */
-#if defined _POSIX_SOURCE
-#include <unistd.h>
-#endif
-
 #include "he.h"
 #ifdef VMS
 #   include <descrip.h>
 #   include <processes.h>
 #   include <unixlib.h>
+/*
 #   include <tpudef.h>
+*/
 #   include <tpu$routines>
 #endif
 
 /* get the prototype for the wait() func. */
-#if defined SUN | defined HP9000
+#if defined SUN | defined HP9000 | defined IRIX | defined UNIX386
 #include <sys/wait.h>
 #endif /* SUN | HP9000 */
+#if defined _POSIX_SOURCE | defined IRIX
+#include <unistd.h>
+#endif
 
 int
 HEannotate(HE_CMD * cmd)
 {
     int i;
-    char       *editor = NULL;
+    const char       *editor = NULL;
     int         ann = HE_LABEL;
 
     for (i = 1; i < cmd->argc; i++)
@@ -84,10 +85,10 @@ HEannotate(HE_CMD * cmd)
 /* Edit annontations (labels and descriptions) for the
  * current data element */
 int
-annotate(char *editor, int ann)
+annotate(const char *editor, int ann)
 {
 
-#if !defined MAC && !defined WIN386 && !defined(__MWERKS__)
+#if !defined MAC && !defined WIN386 && !defined(macintosh)
 
     int32       len;            /* length of annotation */
     char       *buf;            /* annotation buffer */
@@ -105,19 +106,8 @@ annotate(char *editor, int ann)
 
     /* Get the annotation from hdf file
      */
-    len = getAnn(ann, currTag(), he_desc[he_currDesc].ref, &buf);
+    len = getAnn(ann, currTag, he_desc[he_currDesc].ref, &buf);
 
-    /* make sure some editor will be used
-     * defaults to /usr/bin/ex
-     * but should be made a compile time option
-     */
-/*    if (editor == NULL)
-   {
-   char *getenv();
-   editor = getenv("EDITOR");
-   if (editor == NULL) editor = "/usr/bin/ex";
-   }
- */
     (void) getTmpName(&file);
 
     /* if there is prior annotation, put it into the tmp file
@@ -198,7 +188,7 @@ annotate(char *editor, int ann)
     if (ann == HE_LABEL)
       {
           /* take out control characters from the end */
-          for (i = len; i >= 0 && !isgraph(buf[i]); i--)
+          for (i = len; i >= 0 && !isgraph((int)buf[i]); i--)
               ;
           buf[i + 1] = '\0';
       }
@@ -209,7 +199,7 @@ annotate(char *editor, int ann)
 
     /* write annotation to the hdf file
      */
-    ret = putAnn(ann, currTag(), he_desc[he_currDesc].ref, buf, len);
+    ret = putAnn(ann, currTag, he_desc[he_currDesc].ref, buf, len);
     updateDesc();
 
     /* clean up
@@ -224,7 +214,7 @@ annotate(char *editor, int ann)
 
     return 1;
 
-#endif /* not def MAC & WIN386 & __MWERKS__ */
+#endif /* not def MAC & WIN386 & macintosh */
 
 }
 
@@ -423,9 +413,9 @@ writ(char *file, uint16 tag, uint16 ref)
       }
 
     /* handle special cases */
-    if (isAnnot(currTag()))
+    if (isAnnot(currTag))
         return writeAnnot(file, tag, ref);
-    if (isGrp(currTag()))
+    if (isGrp(currTag))
         return writeGrp(file);
 
     if (getNewRef(file, &ref1) < 0)
@@ -512,7 +502,7 @@ HEput(HE_CMD * cmd)
 {
     int i;
     int         verbose = NO;
-    char       *template = "elt#.@";
+    char       *template = (char *)"elt#.@";
 
     for (i = 1; i < cmd->argc; i++)
         if (cmd->argv[i][0] == '-')
@@ -626,7 +616,7 @@ putR8(char *image, char *pal, int verbose)
           noFile();
           return HE_FAIL;
       }
-    if (!isRig(currTag()))
+    if (!isRig(currTag))
       {
           fprintf(stderr, "Current element not an image group.");
           return HE_FAIL;
