@@ -11,10 +11,10 @@
  ****************************************************************************/
 
 #ifdef RCSID
-static char *RcsId[] = "@(#)1.46";
+static char *RcsId[] = "@(#)$Revision: 1.57 $";
 #endif
 
-/* vshow.c,v 1.46 1996/05/28 18:53:25 koziol Exp */
+/* $Id: vshow.c,v 1.57 1998/02/23 16:49:09 acheng Exp $ */
 
 /*****************************************************************************
 *
@@ -35,32 +35,38 @@ static char *RcsId[] = "@(#)1.46";
 #include <console.h>
 #endif
 
-#include "vg.h"
+#define VSET_INTERFACE
+#include "hdf.h"
 
 static int  condensed;
 
-int32       vsdumpfull
+static int32 vsdumpfull
             (int32 vs);
 
-int32 fmtbyte
+static int32 fmtbyte
             (char *x);
-int32 fmtchar
+static int32 fmtchar
             (char *x);
 
-int32 fmtint
+#ifdef UNUSED
+static int32 fmtint
+            (char *x);
+#endif /* UNUSED */
+
+static int32 fmtfloat
             (char *x);
 
-int32 fmtfloat
+static int32 fmtlong
             (char *x);
 
-int32 fmtlong
+static int32 fmtshort
             (char *x);
 
-int32 fmtshort
+static int32 fmtdouble
             (char *x);
 
-int32 fmtdouble
-            (char *x);
+static intn dumpattr
+            (int32 vid, intn full, intn isvs);
 
 int
 main(int ac, char **av)
@@ -82,8 +88,8 @@ main(int ac, char **av)
     char        fields[VSFIELDMAX*FIELDNAMELENMAX], vgname[VGNAMELENMAX];
     char        vsname[VSNAMELENMAX];
     char        vgclass[VGNAMELENMAX], vsclass[VSNAMELENMAX];
-    const char *name;
-    int32       fulldump = 0, start = 1;
+    char *name;
+    int32       fulldump = 0, full;
 
 #if defined __MWERKS__
     ac = ccommand(&av);
@@ -92,7 +98,7 @@ main(int ac, char **av)
     if (ac == 3)
         if (av[2][0] == '-' || av[2][0] == '+')
           {
-              sscanf(&(av[2][1]), "%ld", &vsno);
+              sscanf(&(av[2][1]), "%d", &vsno);
               if (vsno == 0)
                 {
                     printf("FULL DUMP\n");
@@ -106,7 +112,6 @@ main(int ac, char **av)
                   condensed = 1;
               else
                   condensed = 0;
-              start = 2;
           }
 
     if (ac < 2)
@@ -141,7 +146,7 @@ main(int ac, char **av)
               HDstrcat(vgname, "NoName");
           printf("\nvg:%d <%d/%d> (%s {%s}) has %d entries:\n",
            (int) nvg, (int) vgotag, (int) vgoref, vgname, vgclass, (int) n);
-
+          dumpattr(vg, fulldump, 0);
           for (t = 0; t < Vntagrefs(vg); t++)
             {
                 Vgettagref(vg, t, &vstag, &vsid);
@@ -170,7 +175,10 @@ main(int ac, char **av)
                           vsdumpfull(vs);
                       else if (fulldump && vsno == vsoref)
                           vsdumpfull(vs);
-
+                      /* dump attributes */
+                      full = fulldump && (vsno == 0 || vsno == vsoref);
+                      dumpattr(vs, full, 1);
+                                         
                       VSdetach(vs);
                   }
                 else if (vstag == DFTAG_VG)
@@ -192,6 +200,7 @@ main(int ac, char **av)
                       Vgetclass(vgt, vgclass);
                       printf("  vg:%d <%d/%d> ne=%d (%s {%s})\n",
                              (int) t, (int) vgotag, (int) vgoref, (int) ne, vgname, vgclass);
+                      dumpattr(vg, fulldump, 0);
                       Vdetach(vgt);
                   }
                 else
@@ -222,7 +231,7 @@ main(int ac, char **av)
       {
 
           printf("Lone vdatas:\n");
-          if (NULL == (lonevs = (int32 *) HDmalloc(sizeof(int) * nlone)))
+          if (NULL == (lonevs = (int32 *) HDmalloc(sizeof(int) * (size_t)nlone)))
             {
                 printf("%s: File has %d lone vdatas but ", av[0], (int) nlone);
                 printf("cannot alloc lonevs space. Quit.\n");
@@ -250,9 +259,11 @@ main(int ac, char **av)
                     vsdumpfull(vs);
                 else if (fulldump && vsno == vsoref)
                     vsdumpfull(vs);
+                full = fulldump && ( vsno == 0 || vsno == vsoref);
+                dumpattr(vs, full, 1);
                 VSdetach(vs);
             }
-          HDfree((VOIDP) lonevs);
+          HDfree(lonevs);
       }
 
     Vend(f);
@@ -266,14 +277,14 @@ static int32 cn = 0;
 
 /* ------------------------------------------------ */
 /* printing functions used by vsdumpfull(). */
-int32
+static int32
 fmtbyte(char *x)
 {
     cn += printf("%02x ", *x);
     return (1);
 }
 
-int32
+static int32
 fmtchar(char *x)
 {
     cn++;
@@ -281,7 +292,8 @@ fmtchar(char *x)
     return (1);
 }
 
-int32
+#ifdef UNUSED
+static int32
 fmtint(char *x)
 {
     int         i = 0;
@@ -289,17 +301,18 @@ fmtint(char *x)
     cn += printf("%d", i);
     return (1);
 }
+#endif /* UNUSED */
 
-int32
+static int32
 fmtfloat(char *x)
 {
-    float       f = 0;
+    float       f = (float)0.0;
     HDmemcpy(&f, x, sizeof(float32));
     cn += printf("%f", f);
     return (1);
 }
 
-int32
+static int32
 fmtlong(char *x)
 {
     long        l = 0;
@@ -308,7 +321,7 @@ fmtlong(char *x)
     return (1);
 }
 
-int32
+static int32
 fmtshort(char *x)
 {
     short       s = 0;
@@ -317,10 +330,10 @@ fmtshort(char *x)
     return (1);
 }
 
-int32
+static int32
 fmtdouble(char *x)
 {
-    double      d = 0;
+    double      d = 0.0;
     HDmemcpy(&d, x, sizeof(float64));
     cn += printf("%f", d);
     return (1);
@@ -330,7 +343,7 @@ fmtdouble(char *x)
 
 /* ------------------------------------------------ */
 
-int32
+static int32
 vsdumpfull(int32 vs)
 {
     char        vsname[100], fields[VSFIELDMAX*FIELDNAMELENMAX];
@@ -378,7 +391,7 @@ vsdumpfull(int32 vs)
       {
           printf("%d: fld [%s], type=%d, order=%d\n", (int) i, w->name[i], w->type[i], w->order[i]);
 
-          order[i] = w->order[i];
+          order[i] = (int32)w->order[i];
           off[i] = DFKNTsize(w->type[i] | DFNT_NATIVE);
 
           switch (w->type[i])
@@ -474,10 +487,251 @@ vsdumpfull(int32 vs)
 
     /* ============================================ */
 
-    HDfree((VOIDP) bb);
-    printf("\n\n");
+    HDfree(bb);
+    printf("\n");
 
     return (1);
 
 }   /* vsdumpfull */
+/* ------------------------------------------------ */
+static intn dumpattr(int32 vid, intn full, intn isvs)
+{
+   intn i, j, k, cn=0;
+   VDATA *vs;
+   vsinstance_t *vs_inst;
+   VGROUP *vg;
+   vginstance_t *v;
+   intn ret, nattrs, f_nattrs, alloc_flag=0;
+   vs_attr_t *vs_alist;
+   vg_attr_t *v_alist;
+   int32 i_type, i_count, i_size, off;
+   uint8 attrbuf[BUFFER], *buf=NULL, *ptr;
+   int32 (*fmtfn)(char *) =NULL;
+   char name[FIELDNAMELENMAX+1];
+
+   if (isvs)  {
+      vs_inst = (vsinstance_t *)HAatom_object(vid);
+      if (vs_inst == NULL)  {
+         printf(">>>dumpattr:failed in getting vdata instance.\n");
+         return FAIL;
+      } 
+      vs = vs_inst->vs;
+      if (vs == NULL)  {
+         printf(">>>dumpattr:Failed in getting vs. \n");
+         return FAIL;
+      }
+      if (0 == (nattrs = VSnattrs(vid))) {
+          printf(" 0 attributes.\n");
+          return SUCCEED;
+      }
+      vs_alist = vs->alist;
+      if (!full) {
+          printf("     %d attributes:  attr_tag/ref     attr_of_field\n",
+                       nattrs);
+          for (i=0; i<nattrs; i++)  {
+             if (vs_alist->findex != (int)_HDF_VDATA)
+                 printf("     %d:               %d/%d               %d\n",
+                 i, vs_alist->atag, vs_alist->aref,(int)vs_alist->findex);
+             else
+                 printf("     %d:               %d/%d         %s\n",
+                 i, vs_alist->atag, vs_alist->aref, "VDATA");
+             vs_alist++;
+         }
+         return SUCCEED;
+      }
+      printf("%d attributes:\n", nattrs);
+      for (j=-1; j<vs->wlist.n; j++)  
+      {	  int32 temp;
+
+	  temp = (j == -1) ? (int)_HDF_VDATA : j;
+          f_nattrs = VSfnattrs(vid, temp);
+          if (f_nattrs == 0) continue;  /* no attr for this field */
+          if (j == -1)
+              printf("   Attrs of vdata:\n");
+          else
+  	      printf("   Attrs of field %d:\n", j);
+          for (i = 0; i<f_nattrs; i++)  {   /* dump the attrs */
+              ret = VSattrinfo(vid, temp, i, name, &i_type, &i_count, &i_size);
+              if (ret == FAIL) {
+                 printf(">>>dumpattr: failed in getting attr info.\n");
+                 continue;
+              }
+              printf("     %d: name=%s type=%d count=%d size=%d\n",
+                       i, name, (int)i_type, (int)i_count, (int)i_size);
+              if (i_size > BUFFER) {
+                  if (NULL == (buf = HDmalloc(i_size)))  {
+                     printf(">>>dumpattr:can't allocate buf.\n");
+                     continue;
+                  }
+                  alloc_flag = 1;
+                  if ( FAIL==VSgetattr(vid, temp, i, buf)) {
+                     printf(">>>dympattr: failed in VSgetattr.\n");
+                     continue;
+                  }
+              }
+              else 
+              {
+                 if ( FAIL==VSgetattr(vid, temp, i, attrbuf)) {
+                     printf(">>>dympattr: failed in VSgetattr.\n");
+                     continue;
+                  }
+              }
+              /* format output */
+              switch (i_type)  {
+                 case DFNT_CHAR:
+                 case DFNT_UCHAR:
+                      fmtfn = fmtchar;
+                      break;
+                 case DFNT_UINT8:
+                 case DFNT_INT8:
+                      fmtfn = fmtbyte;
+                      break;
+                 case DFNT_UINT16:
+                 case DFNT_INT16:
+                      fmtfn = fmtshort;
+                      break;
+                 case DFNT_UINT32:
+                 case DFNT_INT32:
+                      fmtfn = fmtlong;
+                      break;
+                 case DFNT_FLOAT32:
+                      fmtfn = fmtfloat;
+                      break;
+                 case DFNT_FLOAT64:
+                      fmtfn = fmtdouble;
+                      break;
+                default:
+                    printf(">>>dumpattr: sorry, type [%d] not supported\n", (int) i_type);
+                    break;
+              }
+              off = DFKNTsize(i_type | DFNT_NATIVE);
+              ptr = (alloc_flag) ? buf : attrbuf;
+              putchar('\t');
+              cn = 0;
+              for (k=0; k<i_count; k++)  {
+                  fmtfn((char *)ptr);
+                  ptr += off;
+                  putchar(' ');
+                  cn++;
+                  if (cn > 55)  {
+                     putchar('\n');
+                     putchar('\t');
+                     cn = 0;
+                  }
+              }
+              if (cn) putchar('\n');
+              if (alloc_flag) {
+                 if ( buf != NULL) 
+                    HDfree(buf);
+                 alloc_flag = 0;
+              }
+          }  /*  attr */
+      }   /* field   */
+   }  /* isvs */
+
+   else {  /* vgroup */
+      v = (vginstance_t *)HAatom_object(vid);
+      if (v== NULL)  {
+         printf(">>>dumpattr:failed in getting vgroup instance.\n");
+         return FAIL;
+      } 
+      vg = v->vg;
+      if (vg == NULL)  {
+         printf(">>>dumpattr:Failed in getting vg. \n");
+         return FAIL;
+      }
+      if (0 == (nattrs = Vnattrs(vid)))  {
+          printf("  0 attributes.\n");
+          return SUCCEED;
+      }
+      v_alist = vg->alist;
+      if (!full) {
+          printf("%d attributes:       attr_tag/ref  \n", nattrs);
+          for (i=0; i<nattrs; i++)  {
+             printf("     %d:               %d/%d    \n",
+                 i, v_alist->atag, v_alist->aref);
+             v_alist++;
+         }
+         return SUCCEED;
+      }
+      printf("%d attributes:\n", nattrs);
+      for (i = 0; i<nattrs; i++)  {   /* dump the attrs */
+          ret = Vattrinfo(vid, i, name, &i_type, &i_count, &i_size);
+          if (ret == FAIL) {
+              printf(">>>dumpattr: failed in getting attr info.\n");
+              continue;
+          }
+          printf("   %d: name=%s type=%d count=%d size=%d\n",
+                     i,  name, (int)i_type, (int)i_count, (int)i_size);
+          if (i_size > BUFFER) {
+              if (NULL == (buf = HDmalloc(i_size)))  {
+                 printf(">>>dumpattr:can't allocate buf.\n");
+                 continue;
+              }
+              alloc_flag = 1;
+              if ( FAIL == Vgetattr(vid, i, buf)) {
+                 printf(">>>dympattr: failed in Vgetattr.\n");
+                 continue;
+              }
+          }
+          else 
+          {
+             if ( FAIL == Vgetattr(vid, i, attrbuf)) {
+                 printf(">>>dympattr: failed in Vgetattr.\n");
+                 continue;
+              }
+          }
+          /* format output */
+          switch (i_type)  {
+             case DFNT_CHAR:
+             case DFNT_UCHAR:
+                  fmtfn = fmtchar;
+                  break;
+             case DFNT_UINT8:
+             case DFNT_INT8:
+                  fmtfn = fmtbyte;
+                  break;
+             case DFNT_UINT16:
+             case DFNT_INT16:
+                  fmtfn = fmtshort;
+                  break;
+             case DFNT_UINT32:
+             case DFNT_INT32:
+                  fmtfn = fmtlong;
+                  break;
+             case DFNT_FLOAT32:
+                  fmtfn = fmtfloat;
+                  break;
+             case DFNT_FLOAT64:
+                  fmtfn = fmtdouble;
+                  break;
+             default:
+                printf(">>>dumpattr: sorry, type [%d] not supported\n", (int) i_type);
+                break;
+          }
+          off = DFKNTsize(i_type | DFNT_NATIVE);
+          ptr = (alloc_flag) ? buf : attrbuf;
+          putchar('\t');
+          cn = 0;
+          for (k=0; k<i_count; k++)  {
+              fmtfn((char *)ptr);
+              ptr += off;
+              putchar(' ');
+              cn++;
+              if (cn > 55)  {
+                 putchar('\n');
+                 putchar('\t');
+                 cn = 0;
+              }
+          }
+          if (cn) putchar('\n');
+          if (alloc_flag) {
+             if ( buf != NULL) 
+                HDfree(buf);
+             alloc_flag = 0;
+          }
+      }  /*  attr */
+   }  /* vgroup */
+   return SUCCEED;
+}                   
 /* ------------------------------------- */
